#include <ruby.h>
#include "constants.h"
#include <X11/keysym.h>

void create_mark_constants()
{
  rb_define_global_const( "MARK_START_RELEASE",                 INT2FIX( MARK_START_RELEASE));
  rb_define_global_const( "MARK_END_RELEASE_START_PUSH",        INT2FIX( MARK_END_RELEASE_START_PUSH));
  rb_define_global_const( "MARK_END_PUSH",                      INT2FIX( MARK_END_PUSH));
  rb_define_global_const( "MARK_IDLE",                          INT2FIX( MARK_IDLE));
}

void create_usb_constants()
{
  /* These constants are generated in process_USB_shift_keys. The resulting code
   * is -bitNr-1. The bit number to key mapping can be found in the HID 1.11
   * spec, pg. 56. */
  rb_define_global_const( "USB_CONTROL_L",              INT2FIX( -1));
  rb_define_global_const( "USB_SHIFT_L",                INT2FIX( -2));
  rb_define_global_const( "USB_ALT_L",                  INT2FIX( -3));
  rb_define_global_const( "USB_GUI_L",                  INT2FIX( -4));
  rb_define_global_const( "USB_CONTROL_R",              INT2FIX( -5));
  rb_define_global_const( "USB_SHIFT_R",                INT2FIX( -6));
  rb_define_global_const( "USB_ALT_R",                  INT2FIX( -7));
  rb_define_global_const( "USB_GUI_R",                  INT2FIX( -8));

  /* These constants come from the HID usage tables, version 1.12, pp. 54-60.
   * All codes are present, with the exception of the zero, which is assumed
   * to indicate a reserved byte in the response packet. 
   *
   * The extranous modifier keys (shift etc.) are present. Does anyone
   * know/use them?
   *
   * The key names always represent the unshifted US-keyboard label. Keys not
   * present on the US keyboard are labelled INTL (international), e.g.
   * USB_BACKSLASH is code 49 and USB_INTL_BACKSLASH is code 100.
   * Key names are as simple as possible, e.g. USB_BACKTICK for the left
   * leaning accent (grave accent).
   * 
   * */
  rb_define_global_const( "USB_ERROR_ROLLOVER",         INT2FIX( 1));
  rb_define_global_const( "USB_ERROR_POSTFAIL",         INT2FIX( 2));
  rb_define_global_const( "USB_ERROR_UNDEFINED",        INT2FIX( 3));
  rb_define_global_const( "USB_a",                      INT2FIX( 4));
  rb_define_global_const( "USB_b",                      INT2FIX( 5));
  rb_define_global_const( "USB_c",                      INT2FIX( 6));
  rb_define_global_const( "USB_d",                      INT2FIX( 7));
  rb_define_global_const( "USB_e",                      INT2FIX( 8));
  rb_define_global_const( "USB_f",                      INT2FIX( 9));
  rb_define_global_const( "USB_g",                      INT2FIX( 10));
  rb_define_global_const( "USB_h",                      INT2FIX( 11));
  rb_define_global_const( "USB_i",                      INT2FIX( 12));
  rb_define_global_const( "USB_j",                      INT2FIX( 13));
  rb_define_global_const( "USB_k",                      INT2FIX( 14));
  rb_define_global_const( "USB_l",                      INT2FIX( 15));
  rb_define_global_const( "USB_m",                      INT2FIX( 16));
  rb_define_global_const( "USB_n",                      INT2FIX( 17));
  rb_define_global_const( "USB_o",                      INT2FIX( 18));
  rb_define_global_const( "USB_p",                      INT2FIX( 19));
  rb_define_global_const( "USB_q",                      INT2FIX( 20));
  rb_define_global_const( "USB_r",                      INT2FIX( 21));
  rb_define_global_const( "USB_s",                      INT2FIX( 22));
  rb_define_global_const( "USB_t",                      INT2FIX( 23));
  rb_define_global_const( "USB_u",                      INT2FIX( 24));
  rb_define_global_const( "USB_v",                      INT2FIX( 25));
  rb_define_global_const( "USB_w",                      INT2FIX( 26));
  rb_define_global_const( "USB_x",                      INT2FIX( 27));
  rb_define_global_const( "USB_y",                      INT2FIX( 28));
  rb_define_global_const( "USB_z",                      INT2FIX( 29));
  rb_define_global_const( "USB_1",                      INT2FIX( 30));
  rb_define_global_const( "USB_2",                      INT2FIX( 31));
  rb_define_global_const( "USB_3",                      INT2FIX( 32));
  rb_define_global_const( "USB_4",                      INT2FIX( 33));
  rb_define_global_const( "USB_5",                      INT2FIX( 34));
  rb_define_global_const( "USB_6",                      INT2FIX( 35));
  rb_define_global_const( "USB_7",                      INT2FIX( 36));
  rb_define_global_const( "USB_8",                      INT2FIX( 37));
  rb_define_global_const( "USB_9",                      INT2FIX( 38));
  rb_define_global_const( "USB_0",                      INT2FIX( 39));
  rb_define_global_const( "USB_Enter",                  INT2FIX( 40));
  rb_define_global_const( "USB_Escape",                 INT2FIX( 41));
  rb_define_global_const( "USB_Backspace",              INT2FIX( 42));
  rb_define_global_const( "USB_Tab",                    INT2FIX( 43));
  rb_define_global_const( "USB_Space",                  INT2FIX( 44));
  rb_define_global_const( "USB_Minus",                  INT2FIX( 45));
  rb_define_global_const( "USB_Equal",                  INT2FIX( 46));
  rb_define_global_const( "USB_Bracket_Open",           INT2FIX( 47));
  rb_define_global_const( "USB_Bracket_Close",          INT2FIX( 48));
  rb_define_global_const( "USB_Backslash",              INT2FIX( 49));
  rb_define_global_const( "USB_INTL_Hash",              INT2FIX( 50));
  rb_define_global_const( "USB_Semicolon",              INT2FIX( 51));
  rb_define_global_const( "USB_Tick",                   INT2FIX( 52));
  rb_define_global_const( "USB_Backtick",               INT2FIX( 53));
  rb_define_global_const( "USB_Comma",                  INT2FIX( 54));
  rb_define_global_const( "USB_Period",                 INT2FIX( 55));
  rb_define_global_const( "USB_Slash",                  INT2FIX( 56));
  rb_define_global_const( "USB_CapsLock",               INT2FIX( 57));
  rb_define_global_const( "USB_F1",                     INT2FIX( 58));
  rb_define_global_const( "USB_F2",                     INT2FIX( 59));
  rb_define_global_const( "USB_F3",                     INT2FIX( 60));
  rb_define_global_const( "USB_F4",                     INT2FIX( 61));
  rb_define_global_const( "USB_F5",                     INT2FIX( 62));
  rb_define_global_const( "USB_F6",                     INT2FIX( 63));
  rb_define_global_const( "USB_F7",                     INT2FIX( 64));
  rb_define_global_const( "USB_F8",                     INT2FIX( 65));
  rb_define_global_const( "USB_F9",                     INT2FIX( 66));
  rb_define_global_const( "USB_F10",                    INT2FIX( 67));
  rb_define_global_const( "USB_F11",                    INT2FIX( 68));
  rb_define_global_const( "USB_F12",                    INT2FIX( 69));
  rb_define_global_const( "USB_PrintScreen",            INT2FIX( 70));
  rb_define_global_const( "USB_ScrollLock",             INT2FIX( 71));
  rb_define_global_const( "USB_Pause",                  INT2FIX( 72));
  rb_define_global_const( "USB_Insert",                 INT2FIX( 73));
  rb_define_global_const( "USB_Home",                   INT2FIX( 74));
  rb_define_global_const( "USB_PageUp",                 INT2FIX( 75));
  rb_define_global_const( "USB_Delete",                 INT2FIX( 76));
  rb_define_global_const( "USB_End",                    INT2FIX( 77));
  rb_define_global_const( "USB_PageDown",               INT2FIX( 78));
  rb_define_global_const( "USB_Right",                  INT2FIX( 79));
  rb_define_global_const( "USB_Left",                   INT2FIX( 80));
  rb_define_global_const( "USB_Down",                   INT2FIX( 81));
  rb_define_global_const( "USB_Up",                     INT2FIX( 82));
  rb_define_global_const( "USB_NumLock",                INT2FIX( 83));
  rb_define_global_const( "USB_KP_Divide",              INT2FIX( 84));
  rb_define_global_const( "USB_KP_Multiply",            INT2FIX( 85));
  rb_define_global_const( "USB_KP_Minus",               INT2FIX( 86));
  rb_define_global_const( "USB_KP_Plus",                INT2FIX( 87));
  rb_define_global_const( "USB_KP_Enter",               INT2FIX( 88));
  rb_define_global_const( "USB_KP_1",                   INT2FIX( 89));
  rb_define_global_const( "USB_KP_2",                   INT2FIX( 90));
  rb_define_global_const( "USB_KP_3",                   INT2FIX( 91));
  rb_define_global_const( "USB_KP_4",                   INT2FIX( 92));
  rb_define_global_const( "USB_KP_5",                   INT2FIX( 93));
  rb_define_global_const( "USB_KP_6",                   INT2FIX( 94));
  rb_define_global_const( "USB_KP_7",                   INT2FIX( 95));
  rb_define_global_const( "USB_KP_8",                   INT2FIX( 96));
  rb_define_global_const( "USB_KP_9",                   INT2FIX( 97));
  rb_define_global_const( "USB_KP_0",                   INT2FIX( 98));
  rb_define_global_const( "USB_KP_Point",               INT2FIX( 99));
  rb_define_global_const( "USB_INTL_Backslash",         INT2FIX( 100));
  rb_define_global_const( "USB_Application",            INT2FIX( 101));
  rb_define_global_const( "USB_Power",                  INT2FIX( 102));
  rb_define_global_const( "USB_KP_Equal",               INT2FIX( 103));
  rb_define_global_const( "USB_F13",                    INT2FIX( 104));
  rb_define_global_const( "USB_F14",                    INT2FIX( 105));
  rb_define_global_const( "USB_F15",                    INT2FIX( 106));
  rb_define_global_const( "USB_F16",                    INT2FIX( 107));
  rb_define_global_const( "USB_F17",                    INT2FIX( 108));
  rb_define_global_const( "USB_F18",                    INT2FIX( 109));
  rb_define_global_const( "USB_F19",                    INT2FIX( 110));
  rb_define_global_const( "USB_F20",                    INT2FIX( 111));
  rb_define_global_const( "USB_F21",                    INT2FIX( 112));
  rb_define_global_const( "USB_F22",                    INT2FIX( 113));
  rb_define_global_const( "USB_F23",                    INT2FIX( 114));
  rb_define_global_const( "USB_F24",                    INT2FIX( 115));
  rb_define_global_const( "USB_Execute",                INT2FIX( 116));
  rb_define_global_const( "USB_Help",                   INT2FIX( 117));
  rb_define_global_const( "USB_Menu",                   INT2FIX( 118));
  rb_define_global_const( "USB_Select",                 INT2FIX( 119));
  rb_define_global_const( "USB_Stop",                   INT2FIX( 120));
  rb_define_global_const( "USB_Again",                  INT2FIX( 121));
  rb_define_global_const( "USB_Undo",                   INT2FIX( 122));
  rb_define_global_const( "USB_Cut",                    INT2FIX( 123));
  rb_define_global_const( "USB_Copy",                   INT2FIX( 124));
  rb_define_global_const( "USB_Paste",                  INT2FIX( 125));
  rb_define_global_const( "USB_Find",                   INT2FIX( 126));
  rb_define_global_const( "USB_Mute",                   INT2FIX( 127));
  rb_define_global_const( "USB_Volume_Up",              INT2FIX( 128));
  rb_define_global_const( "USB_Volume_Down",            INT2FIX( 129));
  rb_define_global_const( "USB_Locking_Caps_Lock",      INT2FIX( 130));
  rb_define_global_const( "USB_Locking_Num_Lock",       INT2FIX( 131));
  rb_define_global_const( "USB_Locking_Scroll_Lock",    INT2FIX( 132));
  rb_define_global_const( "USB_KP_Comma",               INT2FIX( 133));
  rb_define_global_const( "USB_KP_Equal_Sign",          INT2FIX( 134));
  rb_define_global_const( "USB_INTL_1",                 INT2FIX( 135));
  rb_define_global_const( "USB_INTL_2",                 INT2FIX( 136));
  rb_define_global_const( "USB_INTL_3",                 INT2FIX( 137));
  rb_define_global_const( "USB_INTL_4",                 INT2FIX( 138));
  rb_define_global_const( "USB_INTL_5",                 INT2FIX( 139));
  rb_define_global_const( "USB_INTL_6",                 INT2FIX( 140));
  rb_define_global_const( "USB_INTL_7",                 INT2FIX( 141));
  rb_define_global_const( "USB_INTL_8",                 INT2FIX( 142));
  rb_define_global_const( "USB_INTL_9",                 INT2FIX( 143));
  rb_define_global_const( "USB_LANG1",                  INT2FIX( 144));
  rb_define_global_const( "USB_LANG2",                  INT2FIX( 145));
  rb_define_global_const( "USB_LANG3",                  INT2FIX( 146));
  rb_define_global_const( "USB_LANG4",                  INT2FIX( 147));
  rb_define_global_const( "USB_LANG5",                  INT2FIX( 148));
  rb_define_global_const( "USB_LANG6",                  INT2FIX( 149));
  rb_define_global_const( "USB_LANG7",                  INT2FIX( 150));
  rb_define_global_const( "USB_LANG8",                  INT2FIX( 151));
  rb_define_global_const( "USB_LANG9",                  INT2FIX( 152));
  rb_define_global_const( "USB_Alt_Erase",              INT2FIX( 153));
  rb_define_global_const( "USB_SysReq",                 INT2FIX( 154));
  rb_define_global_const( "USB_Cancel",                 INT2FIX( 155));
  rb_define_global_const( "USB_Clear",                  INT2FIX( 156));
  rb_define_global_const( "USB_Prior",                  INT2FIX( 157));
  rb_define_global_const( "USB_Return",                 INT2FIX( 158));
  rb_define_global_const( "USB_Separator",              INT2FIX( 159));
  rb_define_global_const( "USB_Out",                    INT2FIX( 160));
  rb_define_global_const( "USB_Oper",                   INT2FIX( 161));
  rb_define_global_const( "USB_Clear_Again",            INT2FIX( 162));
  rb_define_global_const( "USB_CrSel",                  INT2FIX( 163));
  rb_define_global_const( "USB_ExSel",                  INT2FIX( 164));

  rb_define_global_const( "USB_KP_00",                  INT2FIX( 176));
  rb_define_global_const( "USB_KP_000",                 INT2FIX( 177));
  rb_define_global_const( "USB_Thousands_Sep",          INT2FIX( 178));
  rb_define_global_const( "USB_Decimal_Sep",            INT2FIX( 179));
  rb_define_global_const( "USB_Currency_Unit",          INT2FIX( 180));
  rb_define_global_const( "USB_Currency_Subunit",       INT2FIX( 181));
  rb_define_global_const( "USB_KP_Parenthesis_Open",    INT2FIX( 182));
  rb_define_global_const( "USB_KP_Parenthesis_Close",   INT2FIX( 183));
  rb_define_global_const( "USB_KP_Brace_Open",          INT2FIX( 184));
  rb_define_global_const( "USB_KP_Brace_Close",         INT2FIX( 185));
  rb_define_global_const( "USB_KP_Tab",                 INT2FIX( 186));
  rb_define_global_const( "USB_KP_Backspace",           INT2FIX( 187));
  rb_define_global_const( "USB_KP_A",                   INT2FIX( 188));
  rb_define_global_const( "USB_KP_B",                   INT2FIX( 189));
  rb_define_global_const( "USB_KP_C",                   INT2FIX( 190));
  rb_define_global_const( "USB_KP_D",                   INT2FIX( 191));
  rb_define_global_const( "USB_KP_E",                   INT2FIX( 192));
  rb_define_global_const( "USB_KP_F",                   INT2FIX( 193));
  rb_define_global_const( "USB_KP_XOR",                 INT2FIX( 194));
  rb_define_global_const( "USB_KP_Hat",                 INT2FIX( 195));
  rb_define_global_const( "USB_KP_Percent",             INT2FIX( 196));
  rb_define_global_const( "USB_KP_Smaller",             INT2FIX( 197));
  rb_define_global_const( "USB_KP_Larger",              INT2FIX( 198));
  rb_define_global_const( "USB_KP_And",                 INT2FIX( 199));
  rb_define_global_const( "USB_KP_AndAnd",              INT2FIX( 200));
  rb_define_global_const( "USB_KP_Or",                  INT2FIX( 201));
  rb_define_global_const( "USB_KP_OrOr",                INT2FIX( 202));
  rb_define_global_const( "USB_KP_Colon",               INT2FIX( 203));
  rb_define_global_const( "USB_KP_Hash",                INT2FIX( 204));
  rb_define_global_const( "USB_KP_Space",               INT2FIX( 205));
  rb_define_global_const( "USB_KP_At",                  INT2FIX( 206));
  rb_define_global_const( "USB_KP_Exclamation",         INT2FIX( 207));
  rb_define_global_const( "USB_KP_Memory_Store",        INT2FIX( 208));
  rb_define_global_const( "USB_KP_Memory_Recall",       INT2FIX( 209));
  rb_define_global_const( "USB_KP_Memory_Clear",        INT2FIX( 210));
  rb_define_global_const( "USB_KP_Memory_Add",          INT2FIX( 211));
  rb_define_global_const( "USB_KP_Memory_Subtract",     INT2FIX( 212));
  rb_define_global_const( "USB_KP_Memory_Multiply",     INT2FIX( 213));
  rb_define_global_const( "USB_KP_Memory_Divide",       INT2FIX( 214));
  rb_define_global_const( "USB_KP_PlusMinus",           INT2FIX( 215));
  rb_define_global_const( "USB_KP_Clear",               INT2FIX( 216));
  rb_define_global_const( "USB_KP_ClearEntry",          INT2FIX( 217));
  rb_define_global_const( "USB_KP_Binary",              INT2FIX( 218));
  rb_define_global_const( "USB_KP_Octal",               INT2FIX( 219));
  rb_define_global_const( "USB_KP_Decimal",             INT2FIX( 220));
  rb_define_global_const( "USB_KP_Hexadecimal",         INT2FIX( 221));

  rb_define_global_const( "USB_KB_CONTROL_L",           INT2FIX( 224));
  rb_define_global_const( "USB_KB_SHIFT_L",             INT2FIX( 225));
  rb_define_global_const( "USB_KB_ALT_L",               INT2FIX( 226));
  rb_define_global_const( "USB_KB_GUI_L",               INT2FIX( 227));
  rb_define_global_const( "USB_KB_CONTROL_R",           INT2FIX( 228));
  rb_define_global_const( "USB_KB_SHIFT_R",             INT2FIX( 229));
  rb_define_global_const( "USB_KB_ALT_R",               INT2FIX( 230));
  rb_define_global_const( "USB_KB_GUI_R",               INT2FIX( 231));
}

void create_win_constants()
{
  /* based on X11/keysymdef.h */

  rb_define_global_const( "WIN_VoidSymbol",                     INT2FIX( XK_VoidSymbol));
  rb_define_global_const( "WIN_BackSpace",                      INT2FIX( XK_BackSpace));
  rb_define_global_const( "WIN_Tab",                            INT2FIX( XK_Tab));                       
  rb_define_global_const( "WIN_Linefeed",                       INT2FIX( XK_Linefeed));                  
  rb_define_global_const( "WIN_Clear",                          INT2FIX( XK_Clear));                     
  rb_define_global_const( "WIN_Return",                         INT2FIX( XK_Return));                    
  rb_define_global_const( "WIN_Pause",                          INT2FIX( XK_Pause));                     
  rb_define_global_const( "WIN_Scroll_Lock",                    INT2FIX( XK_Scroll_Lock));               
  rb_define_global_const( "WIN_Sys_Req",                        INT2FIX( XK_Sys_Req));                   
  rb_define_global_const( "WIN_Escape",                         INT2FIX( XK_Escape));                    
  rb_define_global_const( "WIN_Delete",                         INT2FIX( XK_Delete));                    
  rb_define_global_const( "WIN_Multi_key",                      INT2FIX( XK_Multi_key));                 
  rb_define_global_const( "WIN_Codeinput",                      INT2FIX( XK_Codeinput));                 
  rb_define_global_const( "WIN_SingleCandidate",                INT2FIX( XK_SingleCandidate));           
  rb_define_global_const( "WIN_MultipleCandidate",              INT2FIX( XK_MultipleCandidate));         
  rb_define_global_const( "WIN_PreviousCandidate",              INT2FIX( XK_PreviousCandidate));         
  rb_define_global_const( "WIN_Kanji",                          INT2FIX( XK_Kanji));                     
  rb_define_global_const( "WIN_Muhenkan",                       INT2FIX( XK_Muhenkan));                  
  rb_define_global_const( "WIN_Henkan_Mode",                    INT2FIX( XK_Henkan_Mode));               
  rb_define_global_const( "WIN_Henkan",                         INT2FIX( XK_Henkan));                    
  rb_define_global_const( "WIN_Romaji",                         INT2FIX( XK_Romaji));                    
  rb_define_global_const( "WIN_Hiragana",                       INT2FIX( XK_Hiragana));                  
  rb_define_global_const( "WIN_Katakana",                       INT2FIX( XK_Katakana));                  
  rb_define_global_const( "WIN_Hiragana_Katakana",              INT2FIX( XK_Hiragana_Katakana));         
  rb_define_global_const( "WIN_Zenkaku",                        INT2FIX( XK_Zenkaku));                   
  rb_define_global_const( "WIN_Hankaku",                        INT2FIX( XK_Hankaku));                   
  rb_define_global_const( "WIN_Zenkaku_Hankaku",                INT2FIX( XK_Zenkaku_Hankaku));           
  rb_define_global_const( "WIN_Touroku",                        INT2FIX( XK_Touroku));                   
  rb_define_global_const( "WIN_Massyo",                         INT2FIX( XK_Massyo));                    
  rb_define_global_const( "WIN_Kana_Lock",                      INT2FIX( XK_Kana_Lock));                 
  rb_define_global_const( "WIN_Kana_Shift",                     INT2FIX( XK_Kana_Shift));                
  rb_define_global_const( "WIN_Eisu_Shift",                     INT2FIX( XK_Eisu_Shift));                
  rb_define_global_const( "WIN_Eisu_toggle",                    INT2FIX( XK_Eisu_toggle));               
  rb_define_global_const( "WIN_Kanji_Bangou",                   INT2FIX( XK_Kanji_Bangou));              
  rb_define_global_const( "WIN_Zen_Koho",                       INT2FIX( XK_Zen_Koho));                  
  rb_define_global_const( "WIN_Mae_Koho",                       INT2FIX( XK_Mae_Koho));                  
  rb_define_global_const( "WIN_Home",                           INT2FIX( XK_Home));                      
  rb_define_global_const( "WIN_Left",                           INT2FIX( XK_Left));                      
  rb_define_global_const( "WIN_Up",                             INT2FIX( XK_Up));                        
  rb_define_global_const( "WIN_Right",                          INT2FIX( XK_Right));                     
  rb_define_global_const( "WIN_Down",                           INT2FIX( XK_Down));                      
  rb_define_global_const( "WIN_Prior",                          INT2FIX( XK_Prior));                     
  rb_define_global_const( "WIN_Page_Up",                        INT2FIX( XK_Page_Up));                   
  rb_define_global_const( "WIN_Next",                           INT2FIX( XK_Next));                      
  rb_define_global_const( "WIN_Page_Down",                      INT2FIX( XK_Page_Down));                 
  rb_define_global_const( "WIN_End",                            INT2FIX( XK_End));                       
  rb_define_global_const( "WIN_Begin",                          INT2FIX( XK_Begin));                     
  rb_define_global_const( "WIN_Select",                         INT2FIX( XK_Select));                    
  rb_define_global_const( "WIN_Print",                          INT2FIX( XK_Print));                     
  rb_define_global_const( "WIN_Execute",                        INT2FIX( XK_Execute));                   
  rb_define_global_const( "WIN_Insert",                         INT2FIX( XK_Insert));                    
  rb_define_global_const( "WIN_Undo",                           INT2FIX( XK_Undo));                      
  rb_define_global_const( "WIN_Redo",                           INT2FIX( XK_Redo));                      
  rb_define_global_const( "WIN_Menu",                           INT2FIX( XK_Menu));                      
  rb_define_global_const( "WIN_Find",                           INT2FIX( XK_Find));                      
  rb_define_global_const( "WIN_Cancel",                         INT2FIX( XK_Cancel));                    
  rb_define_global_const( "WIN_Help",                           INT2FIX( XK_Help));                      
  rb_define_global_const( "WIN_Break",                          INT2FIX( XK_Break));                     
  rb_define_global_const( "WIN_Mode_switch",                    INT2FIX( XK_Mode_switch));               
  rb_define_global_const( "WIN_script_switch",                  INT2FIX( XK_script_switch));             
  rb_define_global_const( "WIN_Num_Lock",                       INT2FIX( XK_Num_Lock));                  
  rb_define_global_const( "WIN_KP_Space",                       INT2FIX( XK_KP_Space));                  
  rb_define_global_const( "WIN_KP_Tab",                         INT2FIX( XK_KP_Tab));                    
  rb_define_global_const( "WIN_KP_Enter",                       INT2FIX( XK_KP_Enter));                  
  rb_define_global_const( "WIN_KP_F1",                          INT2FIX( XK_KP_F1));                     
  rb_define_global_const( "WIN_KP_F2",                          INT2FIX( XK_KP_F2));                     
  rb_define_global_const( "WIN_KP_F3",                          INT2FIX( XK_KP_F3));                     
  rb_define_global_const( "WIN_KP_F4",                          INT2FIX( XK_KP_F4));                     
  rb_define_global_const( "WIN_KP_Home",                        INT2FIX( XK_KP_Home));                   
  rb_define_global_const( "WIN_KP_Left",                        INT2FIX( XK_KP_Left));                   
  rb_define_global_const( "WIN_KP_Up",                          INT2FIX( XK_KP_Up));                     
  rb_define_global_const( "WIN_KP_Right",                       INT2FIX( XK_KP_Right));                  
  rb_define_global_const( "WIN_KP_Down",                        INT2FIX( XK_KP_Down));                   
  rb_define_global_const( "WIN_KP_Prior",                       INT2FIX( XK_KP_Prior));                  
  rb_define_global_const( "WIN_KP_Page_Up",                     INT2FIX( XK_KP_Page_Up));                
  rb_define_global_const( "WIN_KP_Next",                        INT2FIX( XK_KP_Next));                   
  rb_define_global_const( "WIN_KP_Page_Down",                   INT2FIX( XK_KP_Page_Down));              
  rb_define_global_const( "WIN_KP_End",                         INT2FIX( XK_KP_End));                    
  rb_define_global_const( "WIN_KP_Begin",                       INT2FIX( XK_KP_Begin));                  
  rb_define_global_const( "WIN_KP_Insert",                      INT2FIX( XK_KP_Insert));                 
  rb_define_global_const( "WIN_KP_Delete",                      INT2FIX( XK_KP_Delete));                 
  rb_define_global_const( "WIN_KP_Equal",                       INT2FIX( XK_KP_Equal));                  
  rb_define_global_const( "WIN_KP_Multiply",                    INT2FIX( XK_KP_Multiply));               
  rb_define_global_const( "WIN_KP_Add",                         INT2FIX( XK_KP_Add));                    
  rb_define_global_const( "WIN_KP_Separator",                   INT2FIX( XK_KP_Separator));              
  rb_define_global_const( "WIN_KP_Subtract",                    INT2FIX( XK_KP_Subtract));               
  rb_define_global_const( "WIN_KP_Decimal",                     INT2FIX( XK_KP_Decimal));                
  rb_define_global_const( "WIN_KP_Divide",                      INT2FIX( XK_KP_Divide));                 
  rb_define_global_const( "WIN_KP_0",                           INT2FIX( XK_KP_0));                      
  rb_define_global_const( "WIN_KP_1",                           INT2FIX( XK_KP_1));                      
  rb_define_global_const( "WIN_KP_2",                           INT2FIX( XK_KP_2));                      
  rb_define_global_const( "WIN_KP_3",                           INT2FIX( XK_KP_3));                      
  rb_define_global_const( "WIN_KP_4",                           INT2FIX( XK_KP_4));                      
  rb_define_global_const( "WIN_KP_5",                           INT2FIX( XK_KP_5));                      
  rb_define_global_const( "WIN_KP_6",                           INT2FIX( XK_KP_6));                      
  rb_define_global_const( "WIN_KP_7",                           INT2FIX( XK_KP_7));                      
  rb_define_global_const( "WIN_KP_8",                           INT2FIX( XK_KP_8));                      
  rb_define_global_const( "WIN_KP_9",                           INT2FIX( XK_KP_9));                      
  rb_define_global_const( "WIN_F1",                             INT2FIX( XK_F1));                        
  rb_define_global_const( "WIN_F2",                             INT2FIX( XK_F2));                        
  rb_define_global_const( "WIN_F3",                             INT2FIX( XK_F3));                        
  rb_define_global_const( "WIN_F4",                             INT2FIX( XK_F4));                        
  rb_define_global_const( "WIN_F5",                             INT2FIX( XK_F5));                        
  rb_define_global_const( "WIN_F6",                             INT2FIX( XK_F6));                        
  rb_define_global_const( "WIN_F7",                             INT2FIX( XK_F7));                        
  rb_define_global_const( "WIN_F8",                             INT2FIX( XK_F8));                        
  rb_define_global_const( "WIN_F9",                             INT2FIX( XK_F9));                        
  rb_define_global_const( "WIN_F10",                            INT2FIX( XK_F10));                       
  rb_define_global_const( "WIN_F11",                            INT2FIX( XK_F11));                       
  rb_define_global_const( "WIN_L1",                             INT2FIX( XK_L1));                        
  rb_define_global_const( "WIN_F12",                            INT2FIX( XK_F12));                       
  rb_define_global_const( "WIN_L2",                             INT2FIX( XK_L2));                        
  rb_define_global_const( "WIN_F13",                            INT2FIX( XK_F13));                       
  rb_define_global_const( "WIN_L3",                             INT2FIX( XK_L3));                        
  rb_define_global_const( "WIN_F14",                            INT2FIX( XK_F14));                       
  rb_define_global_const( "WIN_L4",                             INT2FIX( XK_L4));                        
  rb_define_global_const( "WIN_F15",                            INT2FIX( XK_F15));                       
  rb_define_global_const( "WIN_L5",                             INT2FIX( XK_L5));                        
  rb_define_global_const( "WIN_F16",                            INT2FIX( XK_F16));                       
  rb_define_global_const( "WIN_L6",                             INT2FIX( XK_L6));                        
  rb_define_global_const( "WIN_F17",                            INT2FIX( XK_F17));                       
  rb_define_global_const( "WIN_L7",                             INT2FIX( XK_L7));                        
  rb_define_global_const( "WIN_F18",                            INT2FIX( XK_F18));                       
  rb_define_global_const( "WIN_L8",                             INT2FIX( XK_L8));                        
  rb_define_global_const( "WIN_F19",                            INT2FIX( XK_F19));                       
  rb_define_global_const( "WIN_L9",                             INT2FIX( XK_L9));                        
  rb_define_global_const( "WIN_F20",                            INT2FIX( XK_F20));                       
  rb_define_global_const( "WIN_L10",                            INT2FIX( XK_L10));                       
  rb_define_global_const( "WIN_F21",                            INT2FIX( XK_F21));                       
  rb_define_global_const( "WIN_R1",                             INT2FIX( XK_R1));                        
  rb_define_global_const( "WIN_F22",                            INT2FIX( XK_F22));                       
  rb_define_global_const( "WIN_R2",                             INT2FIX( XK_R2));                        
  rb_define_global_const( "WIN_F23",                            INT2FIX( XK_F23));                       
  rb_define_global_const( "WIN_R3",                             INT2FIX( XK_R3));                        
  rb_define_global_const( "WIN_F24",                            INT2FIX( XK_F24));                       
  rb_define_global_const( "WIN_R4",                             INT2FIX( XK_R4));                        
  rb_define_global_const( "WIN_F25",                            INT2FIX( XK_F25));                       
  rb_define_global_const( "WIN_R5",                             INT2FIX( XK_R5));                        
  rb_define_global_const( "WIN_F26",                            INT2FIX( XK_F26));                       
  rb_define_global_const( "WIN_R6",                             INT2FIX( XK_R6));                        
  rb_define_global_const( "WIN_F27",                            INT2FIX( XK_F27));                       
  rb_define_global_const( "WIN_R7",                             INT2FIX( XK_R7));                        
  rb_define_global_const( "WIN_F28",                            INT2FIX( XK_F28));                       
  rb_define_global_const( "WIN_R8",                             INT2FIX( XK_R8));                        
  rb_define_global_const( "WIN_F29",                            INT2FIX( XK_F29));                       
  rb_define_global_const( "WIN_R9",                             INT2FIX( XK_R9));                        
  rb_define_global_const( "WIN_F30",                            INT2FIX( XK_F30));                       
  rb_define_global_const( "WIN_R10",                            INT2FIX( XK_R10));                       
  rb_define_global_const( "WIN_F31",                            INT2FIX( XK_F31));                       
  rb_define_global_const( "WIN_R11",                            INT2FIX( XK_R11));                       
  rb_define_global_const( "WIN_F32",                            INT2FIX( XK_F32));                       
  rb_define_global_const( "WIN_R12",                            INT2FIX( XK_R12));                       
  rb_define_global_const( "WIN_F33",                            INT2FIX( XK_F33));                       
  rb_define_global_const( "WIN_R13",                            INT2FIX( XK_R13));                       
  rb_define_global_const( "WIN_F34",                            INT2FIX( XK_F34));                       
  rb_define_global_const( "WIN_R14",                            INT2FIX( XK_R14));                       
  rb_define_global_const( "WIN_F35",                            INT2FIX( XK_F35));                       
  rb_define_global_const( "WIN_R15",                            INT2FIX( XK_R15));                       
  rb_define_global_const( "WIN_Shift_L",                        INT2FIX( XK_Shift_L));                   
  rb_define_global_const( "WIN_Shift_R",                        INT2FIX( XK_Shift_R));                   
  rb_define_global_const( "WIN_Control_L",                      INT2FIX( XK_Control_L));                 
  rb_define_global_const( "WIN_Control_R",                      INT2FIX( XK_Control_R));                 
  rb_define_global_const( "WIN_Caps_Lock",                      INT2FIX( XK_Caps_Lock));                 
  rb_define_global_const( "WIN_Shift_Lock",                     INT2FIX( XK_Shift_Lock));                
  rb_define_global_const( "WIN_Meta_L",                         INT2FIX( XK_Meta_L));                    
  rb_define_global_const( "WIN_Meta_R",                         INT2FIX( XK_Meta_R));                    
  rb_define_global_const( "WIN_Alt_L",                          INT2FIX( XK_Alt_L));                     
  rb_define_global_const( "WIN_Alt_R",                          INT2FIX( XK_Alt_R));                     
  rb_define_global_const( "WIN_Super_L",                        INT2FIX( XK_Super_L));                   
  rb_define_global_const( "WIN_Super_R",                        INT2FIX( XK_Super_R));                   
  rb_define_global_const( "WIN_Hyper_L",                        INT2FIX( XK_Hyper_L));                   
  rb_define_global_const( "WIN_Hyper_R",                        INT2FIX( XK_Hyper_R));                   
  rb_define_global_const( "WIN_ISO_Lock",                       INT2FIX( XK_ISO_Lock));                  
  rb_define_global_const( "WIN_ISO_Level2_Latch",               INT2FIX( XK_ISO_Level2_Latch));          
  rb_define_global_const( "WIN_ISO_Level3_Shift",               INT2FIX( XK_ISO_Level3_Shift));          
  rb_define_global_const( "WIN_ISO_Level3_Latch",               INT2FIX( XK_ISO_Level3_Latch));          
  rb_define_global_const( "WIN_ISO_Level3_Lock",                INT2FIX( XK_ISO_Level3_Lock));           
  rb_define_global_const( "WIN_ISO_Group_Shift",                INT2FIX( XK_ISO_Group_Shift));           
  rb_define_global_const( "WIN_ISO_Group_Latch",                INT2FIX( XK_ISO_Group_Latch));           
  rb_define_global_const( "WIN_ISO_Group_Lock",                 INT2FIX( XK_ISO_Group_Lock));            
  rb_define_global_const( "WIN_ISO_Next_Group",                 INT2FIX( XK_ISO_Next_Group));            
  rb_define_global_const( "WIN_ISO_Next_Group_Lock",            INT2FIX( XK_ISO_Next_Group_Lock));       
  rb_define_global_const( "WIN_ISO_Prev_Group",                 INT2FIX( XK_ISO_Prev_Group));            
  rb_define_global_const( "WIN_ISO_Prev_Group_Lock",            INT2FIX( XK_ISO_Prev_Group_Lock));       
  rb_define_global_const( "WIN_ISO_First_Group",                INT2FIX( XK_ISO_First_Group));           
  rb_define_global_const( "WIN_ISO_First_Group_Lock",           INT2FIX( XK_ISO_First_Group_Lock));      
  rb_define_global_const( "WIN_ISO_Last_Group",                 INT2FIX( XK_ISO_Last_Group));            
  rb_define_global_const( "WIN_ISO_Last_Group_Lock",            INT2FIX( XK_ISO_Last_Group_Lock));       
  rb_define_global_const( "WIN_ISO_Left_Tab",                   INT2FIX( XK_ISO_Left_Tab));              
  rb_define_global_const( "WIN_ISO_Move_Line_Up",               INT2FIX( XK_ISO_Move_Line_Up));          
  rb_define_global_const( "WIN_ISO_Move_Line_Down",             INT2FIX( XK_ISO_Move_Line_Down));        
  rb_define_global_const( "WIN_ISO_Partial_Line_Up",            INT2FIX( XK_ISO_Partial_Line_Up));       
  rb_define_global_const( "WIN_ISO_Partial_Line_Down",          INT2FIX( XK_ISO_Partial_Line_Down));     
  rb_define_global_const( "WIN_ISO_Partial_Space_Left",         INT2FIX( XK_ISO_Partial_Space_Left));    
  rb_define_global_const( "WIN_ISO_Partial_Space_Right",        INT2FIX( XK_ISO_Partial_Space_Right));   
  rb_define_global_const( "WIN_ISO_Set_Margin_Left",            INT2FIX( XK_ISO_Set_Margin_Left));       
  rb_define_global_const( "WIN_ISO_Set_Margin_Right",           INT2FIX( XK_ISO_Set_Margin_Right));      
  rb_define_global_const( "WIN_ISO_Release_Margin_Left",        INT2FIX( XK_ISO_Release_Margin_Left));   
  rb_define_global_const( "WIN_ISO_Release_Margin_Right",       INT2FIX( XK_ISO_Release_Margin_Right));  
  rb_define_global_const( "WIN_ISO_Release_Both_Margins",       INT2FIX( XK_ISO_Release_Both_Margins));  
  rb_define_global_const( "WIN_ISO_Fast_Cursor_Left",           INT2FIX( XK_ISO_Fast_Cursor_Left));      
  rb_define_global_const( "WIN_ISO_Fast_Cursor_Right",          INT2FIX( XK_ISO_Fast_Cursor_Right));     
  rb_define_global_const( "WIN_ISO_Fast_Cursor_Up",             INT2FIX( XK_ISO_Fast_Cursor_Up));        
  rb_define_global_const( "WIN_ISO_Fast_Cursor_Down",           INT2FIX( XK_ISO_Fast_Cursor_Down));      
  rb_define_global_const( "WIN_ISO_Continuous_Underline",       INT2FIX( XK_ISO_Continuous_Underline));  
  rb_define_global_const( "WIN_ISO_Discontinuous_Underline",    INT2FIX( XK_ISO_Discontinuous_Underline));
  rb_define_global_const( "WIN_ISO_Emphasize",                  INT2FIX( XK_ISO_Emphasize));             
  rb_define_global_const( "WIN_ISO_Center_Object",              INT2FIX( XK_ISO_Center_Object));         
  rb_define_global_const( "WIN_ISO_Enter",                      INT2FIX( XK_ISO_Enter));                 
  rb_define_global_const( "WIN_dead_grave",                     INT2FIX( XK_dead_grave));                
  rb_define_global_const( "WIN_dead_acute",                     INT2FIX( XK_dead_acute));                
  rb_define_global_const( "WIN_dead_circumflex",                INT2FIX( XK_dead_circumflex));           
  rb_define_global_const( "WIN_dead_tilde",                     INT2FIX( XK_dead_tilde));                
  rb_define_global_const( "WIN_dead_macron",                    INT2FIX( XK_dead_macron));               
  rb_define_global_const( "WIN_dead_breve",                     INT2FIX( XK_dead_breve));                
  rb_define_global_const( "WIN_dead_abovedot",                  INT2FIX( XK_dead_abovedot));             
  rb_define_global_const( "WIN_dead_diaeresis",                 INT2FIX( XK_dead_diaeresis));            
  rb_define_global_const( "WIN_dead_abovering",                 INT2FIX( XK_dead_abovering));            
  rb_define_global_const( "WIN_dead_doubleacute",               INT2FIX( XK_dead_doubleacute));          
  rb_define_global_const( "WIN_dead_caron",                     INT2FIX( XK_dead_caron));                
  rb_define_global_const( "WIN_dead_cedilla",                   INT2FIX( XK_dead_cedilla));              
  rb_define_global_const( "WIN_dead_ogonek",                    INT2FIX( XK_dead_ogonek));               
  rb_define_global_const( "WIN_dead_iota",                      INT2FIX( XK_dead_iota));                 
  rb_define_global_const( "WIN_dead_voiced_sound",              INT2FIX( XK_dead_voiced_sound));         
  rb_define_global_const( "WIN_dead_semivoiced_sound",          INT2FIX( XK_dead_semivoiced_sound));     
  rb_define_global_const( "WIN_dead_belowdot",                  INT2FIX( XK_dead_belowdot));             
  rb_define_global_const( "WIN_dead_hook",                      INT2FIX( XK_dead_hook));                 
  rb_define_global_const( "WIN_dead_horn",                      INT2FIX( XK_dead_horn));                 
  rb_define_global_const( "WIN_dead_stroke",                    INT2FIX( XK_dead_stroke));               
  rb_define_global_const( "WIN_First_Virtual_Screen",           INT2FIX( XK_First_Virtual_Screen));      
  rb_define_global_const( "WIN_Prev_Virtual_Screen",            INT2FIX( XK_Prev_Virtual_Screen));       
  rb_define_global_const( "WIN_Next_Virtual_Screen",            INT2FIX( XK_Next_Virtual_Screen));       
  rb_define_global_const( "WIN_Last_Virtual_Screen",            INT2FIX( XK_Last_Virtual_Screen));       
  rb_define_global_const( "WIN_Terminate_Server",               INT2FIX( XK_Terminate_Server));          
  rb_define_global_const( "WIN_AccessX_Enable",                 INT2FIX( XK_AccessX_Enable));            
  rb_define_global_const( "WIN_AccessX_Feedback_Enable",        INT2FIX( XK_AccessX_Feedback_Enable));   
  rb_define_global_const( "WIN_RepeatKeys_Enable",              INT2FIX( XK_RepeatKeys_Enable));         
  rb_define_global_const( "WIN_SlowKeys_Enable",                INT2FIX( XK_SlowKeys_Enable));           
  rb_define_global_const( "WIN_BounceKeys_Enable",              INT2FIX( XK_BounceKeys_Enable));         
  rb_define_global_const( "WIN_StickyKeys_Enable",              INT2FIX( XK_StickyKeys_Enable));         
  rb_define_global_const( "WIN_MouseKeys_Enable",               INT2FIX( XK_MouseKeys_Enable));          
  rb_define_global_const( "WIN_MouseKeys_Accel_Enable",         INT2FIX( XK_MouseKeys_Accel_Enable));    
  rb_define_global_const( "WIN_Overlay1_Enable",                INT2FIX( XK_Overlay1_Enable));           
  rb_define_global_const( "WIN_Overlay2_Enable",                INT2FIX( XK_Overlay2_Enable));           
  rb_define_global_const( "WIN_AudibleBell_Enable",             INT2FIX( XK_AudibleBell_Enable));        
  rb_define_global_const( "WIN_Pointer_Left",                   INT2FIX( XK_Pointer_Left));              
  rb_define_global_const( "WIN_Pointer_Right",                  INT2FIX( XK_Pointer_Right));             
  rb_define_global_const( "WIN_Pointer_Up",                     INT2FIX( XK_Pointer_Up));                
  rb_define_global_const( "WIN_Pointer_Down",                   INT2FIX( XK_Pointer_Down));              
  rb_define_global_const( "WIN_Pointer_UpLeft",                 INT2FIX( XK_Pointer_UpLeft));            
  rb_define_global_const( "WIN_Pointer_UpRight",                INT2FIX( XK_Pointer_UpRight));           
  rb_define_global_const( "WIN_Pointer_DownLeft",               INT2FIX( XK_Pointer_DownLeft));          
  rb_define_global_const( "WIN_Pointer_DownRight",              INT2FIX( XK_Pointer_DownRight));         
  rb_define_global_const( "WIN_Pointer_Button_Dflt",            INT2FIX( XK_Pointer_Button_Dflt));       
  rb_define_global_const( "WIN_Pointer_Button1",                INT2FIX( XK_Pointer_Button1));           
  rb_define_global_const( "WIN_Pointer_Button2",                INT2FIX( XK_Pointer_Button2));           
  rb_define_global_const( "WIN_Pointer_Button3",                INT2FIX( XK_Pointer_Button3));           
  rb_define_global_const( "WIN_Pointer_Button4",                INT2FIX( XK_Pointer_Button4));           
  rb_define_global_const( "WIN_Pointer_Button5",                INT2FIX( XK_Pointer_Button5));           
  rb_define_global_const( "WIN_Pointer_DblClick_Dflt",          INT2FIX( XK_Pointer_DblClick_Dflt));     
  rb_define_global_const( "WIN_Pointer_DblClick1",              INT2FIX( XK_Pointer_DblClick1));         
  rb_define_global_const( "WIN_Pointer_DblClick2",              INT2FIX( XK_Pointer_DblClick2));         
  rb_define_global_const( "WIN_Pointer_DblClick3",              INT2FIX( XK_Pointer_DblClick3));         
  rb_define_global_const( "WIN_Pointer_DblClick4",              INT2FIX( XK_Pointer_DblClick4));         
  rb_define_global_const( "WIN_Pointer_DblClick5",              INT2FIX( XK_Pointer_DblClick5));         
  rb_define_global_const( "WIN_Pointer_Drag_Dflt",              INT2FIX( XK_Pointer_Drag_Dflt));         
  rb_define_global_const( "WIN_Pointer_Drag1",                  INT2FIX( XK_Pointer_Drag1));             
  rb_define_global_const( "WIN_Pointer_Drag2",                  INT2FIX( XK_Pointer_Drag2));             
  rb_define_global_const( "WIN_Pointer_Drag3",                  INT2FIX( XK_Pointer_Drag3));             
  rb_define_global_const( "WIN_Pointer_Drag4",                  INT2FIX( XK_Pointer_Drag4));             
  rb_define_global_const( "WIN_Pointer_Drag5",                  INT2FIX( XK_Pointer_Drag5));             
  rb_define_global_const( "WIN_Pointer_EnableKeys",             INT2FIX( XK_Pointer_EnableKeys));        
  rb_define_global_const( "WIN_Pointer_Accelerate",             INT2FIX( XK_Pointer_Accelerate));        
  rb_define_global_const( "WIN_Pointer_DfltBtnNext",            INT2FIX( XK_Pointer_DfltBtnNext));       
  rb_define_global_const( "WIN_Pointer_DfltBtnPrev",            INT2FIX( XK_Pointer_DfltBtnPrev));       

#ifdef XK_3270
  rb_define_global_const( "WIN_3270_Duplicate",                 INT2FIX( XK_3270_Duplicate));            
  rb_define_global_const( "WIN_3270_FieldMark",                 INT2FIX( XK_3270_FieldMark));            
  rb_define_global_const( "WIN_3270_Right2",                    INT2FIX( XK_3270_Right2));               
  rb_define_global_const( "WIN_3270_Left2",                     INT2FIX( XK_3270_Left2));                
  rb_define_global_const( "WIN_3270_BackTab",                   INT2FIX( XK_3270_BackTab));              
  rb_define_global_const( "WIN_3270_EraseEOF",                  INT2FIX( XK_3270_EraseEOF));             
  rb_define_global_const( "WIN_3270_EraseInput",                INT2FIX( XK_3270_EraseInput));           
  rb_define_global_const( "WIN_3270_Reset",                     INT2FIX( XK_3270_Reset));                
  rb_define_global_const( "WIN_3270_Quit",                      INT2FIX( XK_3270_Quit));                 
  rb_define_global_const( "WIN_3270_PA1",                       INT2FIX( XK_3270_PA1));                  
  rb_define_global_const( "WIN_3270_PA2",                       INT2FIX( XK_3270_PA2));                  
  rb_define_global_const( "WIN_3270_PA3",                       INT2FIX( XK_3270_PA3));                  
  rb_define_global_const( "WIN_3270_Test",                      INT2FIX( XK_3270_Test));                 
  rb_define_global_const( "WIN_3270_Attn",                      INT2FIX( XK_3270_Attn));                 
  rb_define_global_const( "WIN_3270_CursorBlink",               INT2FIX( XK_3270_CursorBlink));          
  rb_define_global_const( "WIN_3270_AltCursor",                 INT2FIX( XK_3270_AltCursor));            
  rb_define_global_const( "WIN_3270_KeyClick",                  INT2FIX( XK_3270_KeyClick));             
  rb_define_global_const( "WIN_3270_Jump",                      INT2FIX( XK_3270_Jump));                 
  rb_define_global_const( "WIN_3270_Ident",                     INT2FIX( XK_3270_Ident));                
  rb_define_global_const( "WIN_3270_Rule",                      INT2FIX( XK_3270_Rule));                 
  rb_define_global_const( "WIN_3270_Copy",                      INT2FIX( XK_3270_Copy));                 
  rb_define_global_const( "WIN_3270_Play",                      INT2FIX( XK_3270_Play));                 
  rb_define_global_const( "WIN_3270_Setup",                     INT2FIX( XK_3270_Setup));                
  rb_define_global_const( "WIN_3270_Record",                    INT2FIX( XK_3270_Record));               
  rb_define_global_const( "WIN_3270_ChangeScreen",              INT2FIX( XK_3270_ChangeScreen));         
  rb_define_global_const( "WIN_3270_DeleteWord",                INT2FIX( XK_3270_DeleteWord));           
  rb_define_global_const( "WIN_3270_ExSelect",                  INT2FIX( XK_3270_ExSelect));             
  rb_define_global_const( "WIN_3270_CursorSelect",              INT2FIX( XK_3270_CursorSelect));         
  rb_define_global_const( "WIN_3270_PrintScreen",               INT2FIX( XK_3270_PrintScreen));          
  rb_define_global_const( "WIN_3270_Enter",                     INT2FIX( XK_3270_Enter));                
#endif

  rb_define_global_const( "WIN_space",                          INT2FIX( XK_space));                     
  rb_define_global_const( "WIN_exclam",                         INT2FIX( XK_exclam));                    
  rb_define_global_const( "WIN_quotedbl",                       INT2FIX( XK_quotedbl));                  
  rb_define_global_const( "WIN_numbersign",                     INT2FIX( XK_numbersign));                
  rb_define_global_const( "WIN_dollar",                         INT2FIX( XK_dollar));                    
  rb_define_global_const( "WIN_percent",                        INT2FIX( XK_percent));                   
  rb_define_global_const( "WIN_ampersand",                      INT2FIX( XK_ampersand));                 
  rb_define_global_const( "WIN_apostrophe",                     INT2FIX( XK_apostrophe));                
  rb_define_global_const( "WIN_quoteright",                     INT2FIX( XK_quoteright));                
  rb_define_global_const( "WIN_parenleft",                      INT2FIX( XK_parenleft));                 
  rb_define_global_const( "WIN_parenright",                     INT2FIX( XK_parenright));                
  rb_define_global_const( "WIN_asterisk",                       INT2FIX( XK_asterisk));                  
  rb_define_global_const( "WIN_plus",                           INT2FIX( XK_plus));                      
  rb_define_global_const( "WIN_comma",                          INT2FIX( XK_comma));                     
  rb_define_global_const( "WIN_minus",                          INT2FIX( XK_minus));                     
  rb_define_global_const( "WIN_period",                         INT2FIX( XK_period));                    
  rb_define_global_const( "WIN_slash",                          INT2FIX( XK_slash));                     
  rb_define_global_const( "WIN_0",                              INT2FIX( XK_0));                         
  rb_define_global_const( "WIN_1",                              INT2FIX( XK_1));                         
  rb_define_global_const( "WIN_2",                              INT2FIX( XK_2));                         
  rb_define_global_const( "WIN_3",                              INT2FIX( XK_3));                         
  rb_define_global_const( "WIN_4",                              INT2FIX( XK_4));                         
  rb_define_global_const( "WIN_5",                              INT2FIX( XK_5));                         
  rb_define_global_const( "WIN_6",                              INT2FIX( XK_6));                         
  rb_define_global_const( "WIN_7",                              INT2FIX( XK_7));                         
  rb_define_global_const( "WIN_8",                              INT2FIX( XK_8));                         
  rb_define_global_const( "WIN_9",                              INT2FIX( XK_9));                         
  rb_define_global_const( "WIN_colon",                          INT2FIX( XK_colon));                     
  rb_define_global_const( "WIN_semicolon",                      INT2FIX( XK_semicolon));                 
  rb_define_global_const( "WIN_less",                           INT2FIX( XK_less));                      
  rb_define_global_const( "WIN_equal",                          INT2FIX( XK_equal));                     
  rb_define_global_const( "WIN_greater",                        INT2FIX( XK_greater));                   
  rb_define_global_const( "WIN_question",                       INT2FIX( XK_question));                  
  rb_define_global_const( "WIN_at",                             INT2FIX( XK_at));                        
  rb_define_global_const( "WIN_A",                              INT2FIX( XK_A));                         
  rb_define_global_const( "WIN_B",                              INT2FIX( XK_B));                         
  rb_define_global_const( "WIN_C",                              INT2FIX( XK_C));                         
  rb_define_global_const( "WIN_D",                              INT2FIX( XK_D));                         
  rb_define_global_const( "WIN_E",                              INT2FIX( XK_E));                         
  rb_define_global_const( "WIN_F",                              INT2FIX( XK_F));                         
  rb_define_global_const( "WIN_G",                              INT2FIX( XK_G));                         
  rb_define_global_const( "WIN_H",                              INT2FIX( XK_H));                         
  rb_define_global_const( "WIN_I",                              INT2FIX( XK_I));                         
  rb_define_global_const( "WIN_J",                              INT2FIX( XK_J));                         
  rb_define_global_const( "WIN_K",                              INT2FIX( XK_K));                         
  rb_define_global_const( "WIN_L",                              INT2FIX( XK_L));                         
  rb_define_global_const( "WIN_M",                              INT2FIX( XK_M));                         
  rb_define_global_const( "WIN_N",                              INT2FIX( XK_N));                         
  rb_define_global_const( "WIN_O",                              INT2FIX( XK_O));                         
  rb_define_global_const( "WIN_P",                              INT2FIX( XK_P));                         
  rb_define_global_const( "WIN_Q",                              INT2FIX( XK_Q));                         
  rb_define_global_const( "WIN_R",                              INT2FIX( XK_R));                         
  rb_define_global_const( "WIN_S",                              INT2FIX( XK_S));                         
  rb_define_global_const( "WIN_T",                              INT2FIX( XK_T));                         
  rb_define_global_const( "WIN_U",                              INT2FIX( XK_U));                         
  rb_define_global_const( "WIN_V",                              INT2FIX( XK_V));                         
  rb_define_global_const( "WIN_W",                              INT2FIX( XK_W));                         
  rb_define_global_const( "WIN_X",                              INT2FIX( XK_X));                         
  rb_define_global_const( "WIN_Y",                              INT2FIX( XK_Y));                         
  rb_define_global_const( "WIN_Z",                              INT2FIX( XK_Z));                         
  rb_define_global_const( "WIN_bracketleft",                    INT2FIX( XK_bracketleft));               
  rb_define_global_const( "WIN_backslash",                      INT2FIX( XK_backslash));                 
  rb_define_global_const( "WIN_bracketright",                   INT2FIX( XK_bracketright));              
  rb_define_global_const( "WIN_asciicircum",                    INT2FIX( XK_asciicircum));               
  rb_define_global_const( "WIN_underscore",                     INT2FIX( XK_underscore));                
  rb_define_global_const( "WIN_grave",                          INT2FIX( XK_grave));                     
  rb_define_global_const( "WIN_quoteleft",                      INT2FIX( XK_quoteleft));                 
  rb_define_global_const( "WIN_a",                              INT2FIX( XK_a));                         
  rb_define_global_const( "WIN_b",                              INT2FIX( XK_b));                         
  rb_define_global_const( "WIN_c",                              INT2FIX( XK_c));                         
  rb_define_global_const( "WIN_d",                              INT2FIX( XK_d));                         
  rb_define_global_const( "WIN_e",                              INT2FIX( XK_e));                         
  rb_define_global_const( "WIN_f",                              INT2FIX( XK_f));                         
  rb_define_global_const( "WIN_g",                              INT2FIX( XK_g));                         
  rb_define_global_const( "WIN_h",                              INT2FIX( XK_h));                         
  rb_define_global_const( "WIN_i",                              INT2FIX( XK_i));                         
  rb_define_global_const( "WIN_j",                              INT2FIX( XK_j));                         
  rb_define_global_const( "WIN_k",                              INT2FIX( XK_k));                         
  rb_define_global_const( "WIN_l",                              INT2FIX( XK_l));                         
  rb_define_global_const( "WIN_m",                              INT2FIX( XK_m));                         
  rb_define_global_const( "WIN_n",                              INT2FIX( XK_n));                         
  rb_define_global_const( "WIN_o",                              INT2FIX( XK_o));                         
  rb_define_global_const( "WIN_p",                              INT2FIX( XK_p));                         
  rb_define_global_const( "WIN_q",                              INT2FIX( XK_q));                         
  rb_define_global_const( "WIN_r",                              INT2FIX( XK_r));                         
  rb_define_global_const( "WIN_s",                              INT2FIX( XK_s));                         
  rb_define_global_const( "WIN_t",                              INT2FIX( XK_t));                         
  rb_define_global_const( "WIN_u",                              INT2FIX( XK_u));                         
  rb_define_global_const( "WIN_v",                              INT2FIX( XK_v));                         
  rb_define_global_const( "WIN_w",                              INT2FIX( XK_w));                         
  rb_define_global_const( "WIN_x",                              INT2FIX( XK_x));                         
  rb_define_global_const( "WIN_y",                              INT2FIX( XK_y));                         
  rb_define_global_const( "WIN_z",                              INT2FIX( XK_z));                         
  rb_define_global_const( "WIN_braceleft",                      INT2FIX( XK_braceleft));                 
  rb_define_global_const( "WIN_bar",                            INT2FIX( XK_bar));                       
  rb_define_global_const( "WIN_braceright",                     INT2FIX( XK_braceright));                
  rb_define_global_const( "WIN_asciitilde",                     INT2FIX( XK_asciitilde));                
  rb_define_global_const( "WIN_nobreakspace",                   INT2FIX( XK_nobreakspace));              
  rb_define_global_const( "WIN_exclamdown",                     INT2FIX( XK_exclamdown));                
  rb_define_global_const( "WIN_cent",                           INT2FIX( XK_cent));                      
  rb_define_global_const( "WIN_sterling",                       INT2FIX( XK_sterling));                  
  rb_define_global_const( "WIN_currency",                       INT2FIX( XK_currency));                  
  rb_define_global_const( "WIN_yen",                            INT2FIX( XK_yen));                       
  rb_define_global_const( "WIN_brokenbar",                      INT2FIX( XK_brokenbar));                 
  rb_define_global_const( "WIN_section",                        INT2FIX( XK_section));                   
  rb_define_global_const( "WIN_diaeresis",                      INT2FIX( XK_diaeresis));                 
  rb_define_global_const( "WIN_copyright",                      INT2FIX( XK_copyright));                 
  rb_define_global_const( "WIN_ordfeminine",                    INT2FIX( XK_ordfeminine));               
  rb_define_global_const( "WIN_guillemotleft",                  INT2FIX( XK_guillemotleft));             
  rb_define_global_const( "WIN_notsign",                        INT2FIX( XK_notsign));                   
  rb_define_global_const( "WIN_hyphen",                         INT2FIX( XK_hyphen));                    
  rb_define_global_const( "WIN_registered",                     INT2FIX( XK_registered));                
  rb_define_global_const( "WIN_macron",                         INT2FIX( XK_macron));                    
  rb_define_global_const( "WIN_degree",                         INT2FIX( XK_degree));                    
  rb_define_global_const( "WIN_plusminus",                      INT2FIX( XK_plusminus));                 
  rb_define_global_const( "WIN_twosuperior",                    INT2FIX( XK_twosuperior));               
  rb_define_global_const( "WIN_threesuperior",                  INT2FIX( XK_threesuperior));             
  rb_define_global_const( "WIN_acute",                          INT2FIX( XK_acute));                     
  rb_define_global_const( "WIN_mu",                             INT2FIX( XK_mu));                        
  rb_define_global_const( "WIN_paragraph",                      INT2FIX( XK_paragraph));                 
  rb_define_global_const( "WIN_periodcentered",                 INT2FIX( XK_periodcentered));            
  rb_define_global_const( "WIN_cedilla",                        INT2FIX( XK_cedilla));                   
  rb_define_global_const( "WIN_onesuperior",                    INT2FIX( XK_onesuperior));               
  rb_define_global_const( "WIN_masculine",                      INT2FIX( XK_masculine));                 
  rb_define_global_const( "WIN_guillemotright",                 INT2FIX( XK_guillemotright));            
  rb_define_global_const( "WIN_onequarter",                     INT2FIX( XK_onequarter));                
  rb_define_global_const( "WIN_onehalf",                        INT2FIX( XK_onehalf));                   
  rb_define_global_const( "WIN_threequarters",                  INT2FIX( XK_threequarters));             
  rb_define_global_const( "WIN_questiondown",                   INT2FIX( XK_questiondown));              
  rb_define_global_const( "WIN_Agrave",                         INT2FIX( XK_Agrave));                    
  rb_define_global_const( "WIN_Aacute",                         INT2FIX( XK_Aacute));                    
  rb_define_global_const( "WIN_Acircumflex",                    INT2FIX( XK_Acircumflex));               
  rb_define_global_const( "WIN_Atilde",                         INT2FIX( XK_Atilde));                    
  rb_define_global_const( "WIN_Adiaeresis",                     INT2FIX( XK_Adiaeresis));                
  rb_define_global_const( "WIN_Aring",                          INT2FIX( XK_Aring));                     
  rb_define_global_const( "WIN_AE",                             INT2FIX( XK_AE));                        
  rb_define_global_const( "WIN_Ccedilla",                       INT2FIX( XK_Ccedilla));                  
  rb_define_global_const( "WIN_Egrave",                         INT2FIX( XK_Egrave));                    
  rb_define_global_const( "WIN_Eacute",                         INT2FIX( XK_Eacute));                    
  rb_define_global_const( "WIN_Ecircumflex",                    INT2FIX( XK_Ecircumflex));               
  rb_define_global_const( "WIN_Ediaeresis",                     INT2FIX( XK_Ediaeresis));                
  rb_define_global_const( "WIN_Igrave",                         INT2FIX( XK_Igrave));                    
  rb_define_global_const( "WIN_Iacute",                         INT2FIX( XK_Iacute));                    
  rb_define_global_const( "WIN_Icircumflex",                    INT2FIX( XK_Icircumflex));               
  rb_define_global_const( "WIN_Idiaeresis",                     INT2FIX( XK_Idiaeresis));                
  rb_define_global_const( "WIN_ETH",                            INT2FIX( XK_ETH));                       
  rb_define_global_const( "WIN_Eth",                            INT2FIX( XK_Eth));                       
  rb_define_global_const( "WIN_Ntilde",                         INT2FIX( XK_Ntilde));                    
  rb_define_global_const( "WIN_Ograve",                         INT2FIX( XK_Ograve));                    
  rb_define_global_const( "WIN_Oacute",                         INT2FIX( XK_Oacute));                    
  rb_define_global_const( "WIN_Ocircumflex",                    INT2FIX( XK_Ocircumflex));               
  rb_define_global_const( "WIN_Otilde",                         INT2FIX( XK_Otilde));                    
  rb_define_global_const( "WIN_Odiaeresis",                     INT2FIX( XK_Odiaeresis));                
  rb_define_global_const( "WIN_multiply",                       INT2FIX( XK_multiply));                  
  rb_define_global_const( "WIN_Oslash",                         INT2FIX( XK_Oslash));                    
  rb_define_global_const( "WIN_Ooblique",                       INT2FIX( XK_Ooblique));                  
  rb_define_global_const( "WIN_Ugrave",                         INT2FIX( XK_Ugrave));                    
  rb_define_global_const( "WIN_Uacute",                         INT2FIX( XK_Uacute));                    
  rb_define_global_const( "WIN_Ucircumflex",                    INT2FIX( XK_Ucircumflex));               
  rb_define_global_const( "WIN_Udiaeresis",                     INT2FIX( XK_Udiaeresis));                
  rb_define_global_const( "WIN_Yacute",                         INT2FIX( XK_Yacute));                    
  rb_define_global_const( "WIN_THORN",                          INT2FIX( XK_THORN));                     
  rb_define_global_const( "WIN_Thorn",                          INT2FIX( XK_Thorn));                     
  rb_define_global_const( "WIN_ssharp",                         INT2FIX( XK_ssharp));                    
  rb_define_global_const( "WIN_agrave",                         INT2FIX( XK_agrave));                    
  rb_define_global_const( "WIN_aacute",                         INT2FIX( XK_aacute));                    
  rb_define_global_const( "WIN_acircumflex",                    INT2FIX( XK_acircumflex));               
  rb_define_global_const( "WIN_atilde",                         INT2FIX( XK_atilde));                    
  rb_define_global_const( "WIN_adiaeresis",                     INT2FIX( XK_adiaeresis));                
  rb_define_global_const( "WIN_aring",                          INT2FIX( XK_aring));                     
  rb_define_global_const( "WIN_ae",                             INT2FIX( XK_ae));                        
  rb_define_global_const( "WIN_ccedilla",                       INT2FIX( XK_ccedilla));                  
  rb_define_global_const( "WIN_egrave",                         INT2FIX( XK_egrave));                    
  rb_define_global_const( "WIN_eacute",                         INT2FIX( XK_eacute));                    
  rb_define_global_const( "WIN_ecircumflex",                    INT2FIX( XK_ecircumflex));               
  rb_define_global_const( "WIN_ediaeresis",                     INT2FIX( XK_ediaeresis));                
  rb_define_global_const( "WIN_igrave",                         INT2FIX( XK_igrave));                    
  rb_define_global_const( "WIN_iacute",                         INT2FIX( XK_iacute));                    
  rb_define_global_const( "WIN_icircumflex",                    INT2FIX( XK_icircumflex));               
  rb_define_global_const( "WIN_idiaeresis",                     INT2FIX( XK_idiaeresis));                
  rb_define_global_const( "WIN_eth",                            INT2FIX( XK_eth));                       
  rb_define_global_const( "WIN_ntilde",                         INT2FIX( XK_ntilde));                    
  rb_define_global_const( "WIN_ograve",                         INT2FIX( XK_ograve));                    
  rb_define_global_const( "WIN_oacute",                         INT2FIX( XK_oacute));                    
  rb_define_global_const( "WIN_ocircumflex",                    INT2FIX( XK_ocircumflex));               
  rb_define_global_const( "WIN_otilde",                         INT2FIX( XK_otilde));                    
  rb_define_global_const( "WIN_odiaeresis",                     INT2FIX( XK_odiaeresis));                
  rb_define_global_const( "WIN_division",                       INT2FIX( XK_division));                  
  rb_define_global_const( "WIN_oslash",                         INT2FIX( XK_oslash));                    
  rb_define_global_const( "WIN_ooblique",                       INT2FIX( XK_ooblique));                  
  rb_define_global_const( "WIN_ugrave",                         INT2FIX( XK_ugrave));                    
  rb_define_global_const( "WIN_uacute",                         INT2FIX( XK_uacute));                    
  rb_define_global_const( "WIN_ucircumflex",                    INT2FIX( XK_ucircumflex));               
  rb_define_global_const( "WIN_udiaeresis",                     INT2FIX( XK_udiaeresis));                
  rb_define_global_const( "WIN_yacute",                         INT2FIX( XK_yacute));                    
  rb_define_global_const( "WIN_thorn",                          INT2FIX( XK_thorn));                     
  rb_define_global_const( "WIN_ydiaeresis",                     INT2FIX( XK_ydiaeresis));                
  rb_define_global_const( "WIN_Aogonek",                        INT2FIX( XK_Aogonek));                   
  rb_define_global_const( "WIN_breve",                          INT2FIX( XK_breve));                     
  rb_define_global_const( "WIN_Lstroke",                        INT2FIX( XK_Lstroke));                   
  rb_define_global_const( "WIN_Lcaron",                         INT2FIX( XK_Lcaron));                    
  rb_define_global_const( "WIN_Sacute",                         INT2FIX( XK_Sacute));                    
  rb_define_global_const( "WIN_Scaron",                         INT2FIX( XK_Scaron));                    
  rb_define_global_const( "WIN_Scedilla",                       INT2FIX( XK_Scedilla));                  
  rb_define_global_const( "WIN_Tcaron",                         INT2FIX( XK_Tcaron));                    
  rb_define_global_const( "WIN_Zacute",                         INT2FIX( XK_Zacute));                    
  rb_define_global_const( "WIN_Zcaron",                         INT2FIX( XK_Zcaron));                    
  rb_define_global_const( "WIN_Zabovedot",                      INT2FIX( XK_Zabovedot));                 
  rb_define_global_const( "WIN_aogonek",                        INT2FIX( XK_aogonek));                   
  rb_define_global_const( "WIN_ogonek",                         INT2FIX( XK_ogonek));                    
  rb_define_global_const( "WIN_lstroke",                        INT2FIX( XK_lstroke));                   
  rb_define_global_const( "WIN_lcaron",                         INT2FIX( XK_lcaron));                    
  rb_define_global_const( "WIN_sacute",                         INT2FIX( XK_sacute));                    
  rb_define_global_const( "WIN_caron",                          INT2FIX( XK_caron));                     
  rb_define_global_const( "WIN_scaron",                         INT2FIX( XK_scaron));                    
  rb_define_global_const( "WIN_scedilla",                       INT2FIX( XK_scedilla));                  
  rb_define_global_const( "WIN_tcaron",                         INT2FIX( XK_tcaron));                    
  rb_define_global_const( "WIN_zacute",                         INT2FIX( XK_zacute));                    
  rb_define_global_const( "WIN_doubleacute",                    INT2FIX( XK_doubleacute));               
  rb_define_global_const( "WIN_zcaron",                         INT2FIX( XK_zcaron));                    
  rb_define_global_const( "WIN_zabovedot",                      INT2FIX( XK_zabovedot));                 
  rb_define_global_const( "WIN_Racute",                         INT2FIX( XK_Racute));                    
  rb_define_global_const( "WIN_Abreve",                         INT2FIX( XK_Abreve));                    
  rb_define_global_const( "WIN_Lacute",                         INT2FIX( XK_Lacute));                    
  rb_define_global_const( "WIN_Cacute",                         INT2FIX( XK_Cacute));                    
  rb_define_global_const( "WIN_Ccaron",                         INT2FIX( XK_Ccaron));                    
  rb_define_global_const( "WIN_Eogonek",                        INT2FIX( XK_Eogonek));                   
  rb_define_global_const( "WIN_Ecaron",                         INT2FIX( XK_Ecaron));                    
  rb_define_global_const( "WIN_Dcaron",                         INT2FIX( XK_Dcaron));                    
  rb_define_global_const( "WIN_Dstroke",                        INT2FIX( XK_Dstroke));                   
  rb_define_global_const( "WIN_Nacute",                         INT2FIX( XK_Nacute));                    
  rb_define_global_const( "WIN_Ncaron",                         INT2FIX( XK_Ncaron));                    
  rb_define_global_const( "WIN_Odoubleacute",                   INT2FIX( XK_Odoubleacute));              
  rb_define_global_const( "WIN_Rcaron",                         INT2FIX( XK_Rcaron));                    
  rb_define_global_const( "WIN_Uring",                          INT2FIX( XK_Uring));                     
  rb_define_global_const( "WIN_Udoubleacute",                   INT2FIX( XK_Udoubleacute));              
  rb_define_global_const( "WIN_Tcedilla",                       INT2FIX( XK_Tcedilla));                  
  rb_define_global_const( "WIN_racute",                         INT2FIX( XK_racute));                    
  rb_define_global_const( "WIN_abreve",                         INT2FIX( XK_abreve));                    
  rb_define_global_const( "WIN_lacute",                         INT2FIX( XK_lacute));                    
  rb_define_global_const( "WIN_cacute",                         INT2FIX( XK_cacute));                    
  rb_define_global_const( "WIN_ccaron",                         INT2FIX( XK_ccaron));                    
  rb_define_global_const( "WIN_eogonek",                        INT2FIX( XK_eogonek));                   
  rb_define_global_const( "WIN_ecaron",                         INT2FIX( XK_ecaron));                    
  rb_define_global_const( "WIN_dcaron",                         INT2FIX( XK_dcaron));                    
  rb_define_global_const( "WIN_dstroke",                        INT2FIX( XK_dstroke));                   
  rb_define_global_const( "WIN_nacute",                         INT2FIX( XK_nacute));                    
  rb_define_global_const( "WIN_ncaron",                         INT2FIX( XK_ncaron));                    
  rb_define_global_const( "WIN_odoubleacute",                   INT2FIX( XK_odoubleacute));              
  rb_define_global_const( "WIN_udoubleacute",                   INT2FIX( XK_udoubleacute));              
  rb_define_global_const( "WIN_rcaron",                         INT2FIX( XK_rcaron));                    
  rb_define_global_const( "WIN_uring",                          INT2FIX( XK_uring));                     
  rb_define_global_const( "WIN_tcedilla",                       INT2FIX( XK_tcedilla));                  
  rb_define_global_const( "WIN_abovedot",                       INT2FIX( XK_abovedot));                  
  rb_define_global_const( "WIN_Hstroke",                        INT2FIX( XK_Hstroke));                   
  rb_define_global_const( "WIN_Hcircumflex",                    INT2FIX( XK_Hcircumflex));               
  rb_define_global_const( "WIN_Iabovedot",                      INT2FIX( XK_Iabovedot));                 
  rb_define_global_const( "WIN_Gbreve",                         INT2FIX( XK_Gbreve));                    
  rb_define_global_const( "WIN_Jcircumflex",                    INT2FIX( XK_Jcircumflex));               
  rb_define_global_const( "WIN_hstroke",                        INT2FIX( XK_hstroke));                   
  rb_define_global_const( "WIN_hcircumflex",                    INT2FIX( XK_hcircumflex));               
  rb_define_global_const( "WIN_idotless",                       INT2FIX( XK_idotless));                  
  rb_define_global_const( "WIN_gbreve",                         INT2FIX( XK_gbreve));                    
  rb_define_global_const( "WIN_jcircumflex",                    INT2FIX( XK_jcircumflex));               
  rb_define_global_const( "WIN_Cabovedot",                      INT2FIX( XK_Cabovedot));                 
  rb_define_global_const( "WIN_Ccircumflex",                    INT2FIX( XK_Ccircumflex));               
  rb_define_global_const( "WIN_Gabovedot",                      INT2FIX( XK_Gabovedot));                 
  rb_define_global_const( "WIN_Gcircumflex",                    INT2FIX( XK_Gcircumflex));               
  rb_define_global_const( "WIN_Ubreve",                         INT2FIX( XK_Ubreve));                    
  rb_define_global_const( "WIN_Scircumflex",                    INT2FIX( XK_Scircumflex));               
  rb_define_global_const( "WIN_cabovedot",                      INT2FIX( XK_cabovedot));                 
  rb_define_global_const( "WIN_ccircumflex",                    INT2FIX( XK_ccircumflex));               
  rb_define_global_const( "WIN_gabovedot",                      INT2FIX( XK_gabovedot));                 
  rb_define_global_const( "WIN_gcircumflex",                    INT2FIX( XK_gcircumflex));               
  rb_define_global_const( "WIN_ubreve",                         INT2FIX( XK_ubreve));                    
  rb_define_global_const( "WIN_scircumflex",                    INT2FIX( XK_scircumflex));               
  rb_define_global_const( "WIN_kra",                            INT2FIX( XK_kra));                       
  rb_define_global_const( "WIN_kappa",                          INT2FIX( XK_kappa));                     
  rb_define_global_const( "WIN_Rcedilla",                       INT2FIX( XK_Rcedilla));                  
  rb_define_global_const( "WIN_Itilde",                         INT2FIX( XK_Itilde));                    
  rb_define_global_const( "WIN_Lcedilla",                       INT2FIX( XK_Lcedilla));                  
  rb_define_global_const( "WIN_Emacron",                        INT2FIX( XK_Emacron));                   
  rb_define_global_const( "WIN_Gcedilla",                       INT2FIX( XK_Gcedilla));                  
  rb_define_global_const( "WIN_Tslash",                         INT2FIX( XK_Tslash));                    
  rb_define_global_const( "WIN_rcedilla",                       INT2FIX( XK_rcedilla));                  
  rb_define_global_const( "WIN_itilde",                         INT2FIX( XK_itilde));                    
  rb_define_global_const( "WIN_lcedilla",                       INT2FIX( XK_lcedilla));                  
  rb_define_global_const( "WIN_emacron",                        INT2FIX( XK_emacron));                   
  rb_define_global_const( "WIN_gcedilla",                       INT2FIX( XK_gcedilla));                  
  rb_define_global_const( "WIN_tslash",                         INT2FIX( XK_tslash));                    
  rb_define_global_const( "WIN_ENG",                            INT2FIX( XK_ENG));                       
  rb_define_global_const( "WIN_eng",                            INT2FIX( XK_eng));                       
  rb_define_global_const( "WIN_Amacron",                        INT2FIX( XK_Amacron));                   
  rb_define_global_const( "WIN_Iogonek",                        INT2FIX( XK_Iogonek));                   
  rb_define_global_const( "WIN_Eabovedot",                      INT2FIX( XK_Eabovedot));                 
  rb_define_global_const( "WIN_Imacron",                        INT2FIX( XK_Imacron));                   
  rb_define_global_const( "WIN_Ncedilla",                       INT2FIX( XK_Ncedilla));                  
  rb_define_global_const( "WIN_Omacron",                        INT2FIX( XK_Omacron));                   
  rb_define_global_const( "WIN_Kcedilla",                       INT2FIX( XK_Kcedilla));                  
  rb_define_global_const( "WIN_Uogonek",                        INT2FIX( XK_Uogonek));                   
  rb_define_global_const( "WIN_Utilde",                         INT2FIX( XK_Utilde));                    
  rb_define_global_const( "WIN_Umacron",                        INT2FIX( XK_Umacron));                   
  rb_define_global_const( "WIN_amacron",                        INT2FIX( XK_amacron));                   
  rb_define_global_const( "WIN_iogonek",                        INT2FIX( XK_iogonek));                   
  rb_define_global_const( "WIN_eabovedot",                      INT2FIX( XK_eabovedot));                 
  rb_define_global_const( "WIN_imacron",                        INT2FIX( XK_imacron));                   
  rb_define_global_const( "WIN_ncedilla",                       INT2FIX( XK_ncedilla));                  
  rb_define_global_const( "WIN_omacron",                        INT2FIX( XK_omacron));                   
  rb_define_global_const( "WIN_kcedilla",                       INT2FIX( XK_kcedilla));                  
  rb_define_global_const( "WIN_uogonek",                        INT2FIX( XK_uogonek));                   
  rb_define_global_const( "WIN_utilde",                         INT2FIX( XK_utilde));                    
  rb_define_global_const( "WIN_umacron",                        INT2FIX( XK_umacron));                   
  rb_define_global_const( "WIN_Babovedot",                      INT2FIX( XK_Babovedot));                 
  rb_define_global_const( "WIN_babovedot",                      INT2FIX( XK_babovedot));                 
  rb_define_global_const( "WIN_Dabovedot",                      INT2FIX( XK_Dabovedot));                 
  rb_define_global_const( "WIN_Wgrave",                         INT2FIX( XK_Wgrave));                    
  rb_define_global_const( "WIN_Wacute",                         INT2FIX( XK_Wacute));                    
  rb_define_global_const( "WIN_dabovedot",                      INT2FIX( XK_dabovedot));                 
  rb_define_global_const( "WIN_Ygrave",                         INT2FIX( XK_Ygrave));                    
  rb_define_global_const( "WIN_Fabovedot",                      INT2FIX( XK_Fabovedot));                 
  rb_define_global_const( "WIN_fabovedot",                      INT2FIX( XK_fabovedot));                 
  rb_define_global_const( "WIN_Mabovedot",                      INT2FIX( XK_Mabovedot));                 
  rb_define_global_const( "WIN_mabovedot",                      INT2FIX( XK_mabovedot));                 
  rb_define_global_const( "WIN_Pabovedot",                      INT2FIX( XK_Pabovedot));                 
  rb_define_global_const( "WIN_wgrave",                         INT2FIX( XK_wgrave));                    
  rb_define_global_const( "WIN_pabovedot",                      INT2FIX( XK_pabovedot));                 
  rb_define_global_const( "WIN_wacute",                         INT2FIX( XK_wacute));                    
  rb_define_global_const( "WIN_Sabovedot",                      INT2FIX( XK_Sabovedot));                 
  rb_define_global_const( "WIN_ygrave",                         INT2FIX( XK_ygrave));                    
  rb_define_global_const( "WIN_Wdiaeresis",                     INT2FIX( XK_Wdiaeresis));                
  rb_define_global_const( "WIN_wdiaeresis",                     INT2FIX( XK_wdiaeresis));                
  rb_define_global_const( "WIN_sabovedot",                      INT2FIX( XK_sabovedot));                 
  rb_define_global_const( "WIN_Wcircumflex",                    INT2FIX( XK_Wcircumflex));               
  rb_define_global_const( "WIN_Tabovedot",                      INT2FIX( XK_Tabovedot));                 
  rb_define_global_const( "WIN_Ycircumflex",                    INT2FIX( XK_Ycircumflex));               
  rb_define_global_const( "WIN_wcircumflex",                    INT2FIX( XK_wcircumflex));               
  rb_define_global_const( "WIN_tabovedot",                      INT2FIX( XK_tabovedot));                 
  rb_define_global_const( "WIN_ycircumflex",                    INT2FIX( XK_ycircumflex));               
  rb_define_global_const( "WIN_OE",                             INT2FIX( XK_OE));                        
  rb_define_global_const( "WIN_oe",                             INT2FIX( XK_oe));                        
  rb_define_global_const( "WIN_Ydiaeresis",                     INT2FIX( XK_Ydiaeresis));                
  rb_define_global_const( "WIN_overline",                       INT2FIX( XK_overline));                  
  rb_define_global_const( "WIN_kana_fullstop",                  INT2FIX( XK_kana_fullstop));             
  rb_define_global_const( "WIN_kana_openingbracket",            INT2FIX( XK_kana_openingbracket));       
  rb_define_global_const( "WIN_kana_closingbracket",            INT2FIX( XK_kana_closingbracket));       
  rb_define_global_const( "WIN_kana_comma",                     INT2FIX( XK_kana_comma));                
  rb_define_global_const( "WIN_kana_conjunctive",               INT2FIX( XK_kana_conjunctive));          
  rb_define_global_const( "WIN_kana_middledot",                 INT2FIX( XK_kana_middledot));            
  rb_define_global_const( "WIN_kana_WO",                        INT2FIX( XK_kana_WO));                   
  rb_define_global_const( "WIN_kana_a",                         INT2FIX( XK_kana_a));                    
  rb_define_global_const( "WIN_kana_i",                         INT2FIX( XK_kana_i));                    
  rb_define_global_const( "WIN_kana_u",                         INT2FIX( XK_kana_u));                    
  rb_define_global_const( "WIN_kana_e",                         INT2FIX( XK_kana_e));                    
  rb_define_global_const( "WIN_kana_o",                         INT2FIX( XK_kana_o));                    
  rb_define_global_const( "WIN_kana_ya",                        INT2FIX( XK_kana_ya));                   
  rb_define_global_const( "WIN_kana_yu",                        INT2FIX( XK_kana_yu));                   
  rb_define_global_const( "WIN_kana_yo",                        INT2FIX( XK_kana_yo));                   
  rb_define_global_const( "WIN_kana_tsu",                       INT2FIX( XK_kana_tsu));                  
  rb_define_global_const( "WIN_kana_tu",                        INT2FIX( XK_kana_tu));                   
  rb_define_global_const( "WIN_prolongedsound",                 INT2FIX( XK_prolongedsound));            
  rb_define_global_const( "WIN_kana_A",                         INT2FIX( XK_kana_A));                    
  rb_define_global_const( "WIN_kana_I",                         INT2FIX( XK_kana_I));                    
  rb_define_global_const( "WIN_kana_U",                         INT2FIX( XK_kana_U));                    
  rb_define_global_const( "WIN_kana_E",                         INT2FIX( XK_kana_E));                    
  rb_define_global_const( "WIN_kana_O",                         INT2FIX( XK_kana_O));                    
  rb_define_global_const( "WIN_kana_KA",                        INT2FIX( XK_kana_KA));                   
  rb_define_global_const( "WIN_kana_KI",                        INT2FIX( XK_kana_KI));                   
  rb_define_global_const( "WIN_kana_KU",                        INT2FIX( XK_kana_KU));                   
  rb_define_global_const( "WIN_kana_KE",                        INT2FIX( XK_kana_KE));                   
  rb_define_global_const( "WIN_kana_KO",                        INT2FIX( XK_kana_KO));                   
  rb_define_global_const( "WIN_kana_SA",                        INT2FIX( XK_kana_SA));                   
  rb_define_global_const( "WIN_kana_SHI",                       INT2FIX( XK_kana_SHI));                  
  rb_define_global_const( "WIN_kana_SU",                        INT2FIX( XK_kana_SU));                   
  rb_define_global_const( "WIN_kana_SE",                        INT2FIX( XK_kana_SE));                   
  rb_define_global_const( "WIN_kana_SO",                        INT2FIX( XK_kana_SO));                   
  rb_define_global_const( "WIN_kana_TA",                        INT2FIX( XK_kana_TA));                   
  rb_define_global_const( "WIN_kana_CHI",                       INT2FIX( XK_kana_CHI));                  
  rb_define_global_const( "WIN_kana_TI",                        INT2FIX( XK_kana_TI));                   
  rb_define_global_const( "WIN_kana_TSU",                       INT2FIX( XK_kana_TSU));                  
  rb_define_global_const( "WIN_kana_TU",                        INT2FIX( XK_kana_TU));                   
  rb_define_global_const( "WIN_kana_TE",                        INT2FIX( XK_kana_TE));                   
  rb_define_global_const( "WIN_kana_TO",                        INT2FIX( XK_kana_TO));                   
  rb_define_global_const( "WIN_kana_NA",                        INT2FIX( XK_kana_NA));                   
  rb_define_global_const( "WIN_kana_NI",                        INT2FIX( XK_kana_NI));                   
  rb_define_global_const( "WIN_kana_NU",                        INT2FIX( XK_kana_NU));                   
  rb_define_global_const( "WIN_kana_NE",                        INT2FIX( XK_kana_NE));                   
  rb_define_global_const( "WIN_kana_NO",                        INT2FIX( XK_kana_NO));                   
  rb_define_global_const( "WIN_kana_HA",                        INT2FIX( XK_kana_HA));                   
  rb_define_global_const( "WIN_kana_HI",                        INT2FIX( XK_kana_HI));                   
  rb_define_global_const( "WIN_kana_FU",                        INT2FIX( XK_kana_FU));                   
  rb_define_global_const( "WIN_kana_HU",                        INT2FIX( XK_kana_HU));                   
  rb_define_global_const( "WIN_kana_HE",                        INT2FIX( XK_kana_HE));                   
  rb_define_global_const( "WIN_kana_HO",                        INT2FIX( XK_kana_HO));                   
  rb_define_global_const( "WIN_kana_MA",                        INT2FIX( XK_kana_MA));                   
  rb_define_global_const( "WIN_kana_MI",                        INT2FIX( XK_kana_MI));                   
  rb_define_global_const( "WIN_kana_MU",                        INT2FIX( XK_kana_MU));                   
  rb_define_global_const( "WIN_kana_ME",                        INT2FIX( XK_kana_ME));                   
  rb_define_global_const( "WIN_kana_MO",                        INT2FIX( XK_kana_MO));                   
  rb_define_global_const( "WIN_kana_YA",                        INT2FIX( XK_kana_YA));                   
  rb_define_global_const( "WIN_kana_YU",                        INT2FIX( XK_kana_YU));                   
  rb_define_global_const( "WIN_kana_YO",                        INT2FIX( XK_kana_YO));                   
  rb_define_global_const( "WIN_kana_RA",                        INT2FIX( XK_kana_RA));                   
  rb_define_global_const( "WIN_kana_RI",                        INT2FIX( XK_kana_RI));                   
  rb_define_global_const( "WIN_kana_RU",                        INT2FIX( XK_kana_RU));                   
  rb_define_global_const( "WIN_kana_RE",                        INT2FIX( XK_kana_RE));                   
  rb_define_global_const( "WIN_kana_RO",                        INT2FIX( XK_kana_RO));                   
  rb_define_global_const( "WIN_kana_WA",                        INT2FIX( XK_kana_WA));                   
  rb_define_global_const( "WIN_kana_N",                         INT2FIX( XK_kana_N));                    
  rb_define_global_const( "WIN_voicedsound",                    INT2FIX( XK_voicedsound));               
  rb_define_global_const( "WIN_semivoicedsound",                INT2FIX( XK_semivoicedsound));           
  rb_define_global_const( "WIN_kana_switch",                    INT2FIX( XK_kana_switch));               
  rb_define_global_const( "WIN_Farsi_0",                        INT2FIX( XK_Farsi_0));                   
  rb_define_global_const( "WIN_Farsi_1",                        INT2FIX( XK_Farsi_1));                   
  rb_define_global_const( "WIN_Farsi_2",                        INT2FIX( XK_Farsi_2));                   
  rb_define_global_const( "WIN_Farsi_3",                        INT2FIX( XK_Farsi_3));                   
  rb_define_global_const( "WIN_Farsi_4",                        INT2FIX( XK_Farsi_4));                   
  rb_define_global_const( "WIN_Farsi_5",                        INT2FIX( XK_Farsi_5));                   
  rb_define_global_const( "WIN_Farsi_6",                        INT2FIX( XK_Farsi_6));                   
  rb_define_global_const( "WIN_Farsi_7",                        INT2FIX( XK_Farsi_7));                   
  rb_define_global_const( "WIN_Farsi_8",                        INT2FIX( XK_Farsi_8));                   
  rb_define_global_const( "WIN_Farsi_9",                        INT2FIX( XK_Farsi_9));                   
  rb_define_global_const( "WIN_Arabic_percent",                 INT2FIX( XK_Arabic_percent));            
  rb_define_global_const( "WIN_Arabic_superscript_alef",        INT2FIX( XK_Arabic_superscript_alef));   
  rb_define_global_const( "WIN_Arabic_tteh",                    INT2FIX( XK_Arabic_tteh));               
  rb_define_global_const( "WIN_Arabic_peh",                     INT2FIX( XK_Arabic_peh));                
  rb_define_global_const( "WIN_Arabic_tcheh",                   INT2FIX( XK_Arabic_tcheh));              
  rb_define_global_const( "WIN_Arabic_ddal",                    INT2FIX( XK_Arabic_ddal));               
  rb_define_global_const( "WIN_Arabic_rreh",                    INT2FIX( XK_Arabic_rreh));               
  rb_define_global_const( "WIN_Arabic_comma",                   INT2FIX( XK_Arabic_comma));              
  rb_define_global_const( "WIN_Arabic_fullstop",                INT2FIX( XK_Arabic_fullstop));           
  rb_define_global_const( "WIN_Arabic_0",                       INT2FIX( XK_Arabic_0));                  
  rb_define_global_const( "WIN_Arabic_1",                       INT2FIX( XK_Arabic_1));                  
  rb_define_global_const( "WIN_Arabic_2",                       INT2FIX( XK_Arabic_2));                  
  rb_define_global_const( "WIN_Arabic_3",                       INT2FIX( XK_Arabic_3));                  
  rb_define_global_const( "WIN_Arabic_4",                       INT2FIX( XK_Arabic_4));                  
  rb_define_global_const( "WIN_Arabic_5",                       INT2FIX( XK_Arabic_5));                  
  rb_define_global_const( "WIN_Arabic_6",                       INT2FIX( XK_Arabic_6));                  
  rb_define_global_const( "WIN_Arabic_7",                       INT2FIX( XK_Arabic_7));                  
  rb_define_global_const( "WIN_Arabic_8",                       INT2FIX( XK_Arabic_8));                  
  rb_define_global_const( "WIN_Arabic_9",                       INT2FIX( XK_Arabic_9));                  
  rb_define_global_const( "WIN_Arabic_semicolon",               INT2FIX( XK_Arabic_semicolon));          
  rb_define_global_const( "WIN_Arabic_question_mark",           INT2FIX( XK_Arabic_question_mark));      
  rb_define_global_const( "WIN_Arabic_hamza",                   INT2FIX( XK_Arabic_hamza));              
  rb_define_global_const( "WIN_Arabic_maddaonalef",             INT2FIX( XK_Arabic_maddaonalef));        
  rb_define_global_const( "WIN_Arabic_hamzaonalef",             INT2FIX( XK_Arabic_hamzaonalef));        
  rb_define_global_const( "WIN_Arabic_hamzaonwaw",              INT2FIX( XK_Arabic_hamzaonwaw));         
  rb_define_global_const( "WIN_Arabic_hamzaunderalef",          INT2FIX( XK_Arabic_hamzaunderalef));     
  rb_define_global_const( "WIN_Arabic_hamzaonyeh",              INT2FIX( XK_Arabic_hamzaonyeh));         
  rb_define_global_const( "WIN_Arabic_alef",                    INT2FIX( XK_Arabic_alef));               
  rb_define_global_const( "WIN_Arabic_beh",                     INT2FIX( XK_Arabic_beh));                
  rb_define_global_const( "WIN_Arabic_tehmarbuta",              INT2FIX( XK_Arabic_tehmarbuta));         
  rb_define_global_const( "WIN_Arabic_teh",                     INT2FIX( XK_Arabic_teh));                
  rb_define_global_const( "WIN_Arabic_theh",                    INT2FIX( XK_Arabic_theh));               
  rb_define_global_const( "WIN_Arabic_jeem",                    INT2FIX( XK_Arabic_jeem));               
  rb_define_global_const( "WIN_Arabic_hah",                     INT2FIX( XK_Arabic_hah));                
  rb_define_global_const( "WIN_Arabic_khah",                    INT2FIX( XK_Arabic_khah));               
  rb_define_global_const( "WIN_Arabic_dal",                     INT2FIX( XK_Arabic_dal));                
  rb_define_global_const( "WIN_Arabic_thal",                    INT2FIX( XK_Arabic_thal));               
  rb_define_global_const( "WIN_Arabic_ra",                      INT2FIX( XK_Arabic_ra));                 
  rb_define_global_const( "WIN_Arabic_zain",                    INT2FIX( XK_Arabic_zain));               
  rb_define_global_const( "WIN_Arabic_seen",                    INT2FIX( XK_Arabic_seen));               
  rb_define_global_const( "WIN_Arabic_sheen",                   INT2FIX( XK_Arabic_sheen));              
  rb_define_global_const( "WIN_Arabic_sad",                     INT2FIX( XK_Arabic_sad));                
  rb_define_global_const( "WIN_Arabic_dad",                     INT2FIX( XK_Arabic_dad));                
  rb_define_global_const( "WIN_Arabic_tah",                     INT2FIX( XK_Arabic_tah));                
  rb_define_global_const( "WIN_Arabic_zah",                     INT2FIX( XK_Arabic_zah));                
  rb_define_global_const( "WIN_Arabic_ain",                     INT2FIX( XK_Arabic_ain));                
  rb_define_global_const( "WIN_Arabic_ghain",                   INT2FIX( XK_Arabic_ghain));              
  rb_define_global_const( "WIN_Arabic_tatweel",                 INT2FIX( XK_Arabic_tatweel));            
  rb_define_global_const( "WIN_Arabic_feh",                     INT2FIX( XK_Arabic_feh));                
  rb_define_global_const( "WIN_Arabic_qaf",                     INT2FIX( XK_Arabic_qaf));                
  rb_define_global_const( "WIN_Arabic_kaf",                     INT2FIX( XK_Arabic_kaf));                
  rb_define_global_const( "WIN_Arabic_lam",                     INT2FIX( XK_Arabic_lam));                
  rb_define_global_const( "WIN_Arabic_meem",                    INT2FIX( XK_Arabic_meem));               
  rb_define_global_const( "WIN_Arabic_noon",                    INT2FIX( XK_Arabic_noon));               
  rb_define_global_const( "WIN_Arabic_ha",                      INT2FIX( XK_Arabic_ha));                 
  rb_define_global_const( "WIN_Arabic_heh",                     INT2FIX( XK_Arabic_heh));                
  rb_define_global_const( "WIN_Arabic_waw",                     INT2FIX( XK_Arabic_waw));                
  rb_define_global_const( "WIN_Arabic_alefmaksura",             INT2FIX( XK_Arabic_alefmaksura));        
  rb_define_global_const( "WIN_Arabic_yeh",                     INT2FIX( XK_Arabic_yeh));                
  rb_define_global_const( "WIN_Arabic_fathatan",                INT2FIX( XK_Arabic_fathatan));           
  rb_define_global_const( "WIN_Arabic_dammatan",                INT2FIX( XK_Arabic_dammatan));           
  rb_define_global_const( "WIN_Arabic_kasratan",                INT2FIX( XK_Arabic_kasratan));           
  rb_define_global_const( "WIN_Arabic_fatha",                   INT2FIX( XK_Arabic_fatha));              
  rb_define_global_const( "WIN_Arabic_damma",                   INT2FIX( XK_Arabic_damma));              
  rb_define_global_const( "WIN_Arabic_kasra",                   INT2FIX( XK_Arabic_kasra));              
  rb_define_global_const( "WIN_Arabic_shadda",                  INT2FIX( XK_Arabic_shadda));             
  rb_define_global_const( "WIN_Arabic_sukun",                   INT2FIX( XK_Arabic_sukun));              
  rb_define_global_const( "WIN_Arabic_madda_above",             INT2FIX( XK_Arabic_madda_above));        
  rb_define_global_const( "WIN_Arabic_hamza_above",             INT2FIX( XK_Arabic_hamza_above));        
  rb_define_global_const( "WIN_Arabic_hamza_below",             INT2FIX( XK_Arabic_hamza_below));        
  rb_define_global_const( "WIN_Arabic_jeh",                     INT2FIX( XK_Arabic_jeh));                
  rb_define_global_const( "WIN_Arabic_veh",                     INT2FIX( XK_Arabic_veh));                
  rb_define_global_const( "WIN_Arabic_keheh",                   INT2FIX( XK_Arabic_keheh));              
  rb_define_global_const( "WIN_Arabic_gaf",                     INT2FIX( XK_Arabic_gaf));                
  rb_define_global_const( "WIN_Arabic_noon_ghunna",             INT2FIX( XK_Arabic_noon_ghunna));        
  rb_define_global_const( "WIN_Arabic_heh_doachashmee",         INT2FIX( XK_Arabic_heh_doachashmee));    
  rb_define_global_const( "WIN_Farsi_yeh",                      INT2FIX( XK_Farsi_yeh));                 
  rb_define_global_const( "WIN_Arabic_farsi_yeh",               INT2FIX( XK_Arabic_farsi_yeh));          
  rb_define_global_const( "WIN_Arabic_yeh_baree",               INT2FIX( XK_Arabic_yeh_baree));          
  rb_define_global_const( "WIN_Arabic_heh_goal",                INT2FIX( XK_Arabic_heh_goal));           
  rb_define_global_const( "WIN_Arabic_switch",                  INT2FIX( XK_Arabic_switch));             
  rb_define_global_const( "WIN_Cyrillic_GHE_bar",               INT2FIX( XK_Cyrillic_GHE_bar));          
  rb_define_global_const( "WIN_Cyrillic_ghe_bar",               INT2FIX( XK_Cyrillic_ghe_bar));          
  rb_define_global_const( "WIN_Cyrillic_ZHE_descender",         INT2FIX( XK_Cyrillic_ZHE_descender));    
  rb_define_global_const( "WIN_Cyrillic_zhe_descender",         INT2FIX( XK_Cyrillic_zhe_descender));    
  rb_define_global_const( "WIN_Cyrillic_KA_descender",          INT2FIX( XK_Cyrillic_KA_descender));     
  rb_define_global_const( "WIN_Cyrillic_ka_descender",          INT2FIX( XK_Cyrillic_ka_descender));     
  rb_define_global_const( "WIN_Cyrillic_KA_vertstroke",         INT2FIX( XK_Cyrillic_KA_vertstroke));    
  rb_define_global_const( "WIN_Cyrillic_ka_vertstroke",         INT2FIX( XK_Cyrillic_ka_vertstroke));    
  rb_define_global_const( "WIN_Cyrillic_EN_descender",          INT2FIX( XK_Cyrillic_EN_descender));     
  rb_define_global_const( "WIN_Cyrillic_en_descender",          INT2FIX( XK_Cyrillic_en_descender));     
  rb_define_global_const( "WIN_Cyrillic_U_straight",            INT2FIX( XK_Cyrillic_U_straight));       
  rb_define_global_const( "WIN_Cyrillic_u_straight",            INT2FIX( XK_Cyrillic_u_straight));       
  rb_define_global_const( "WIN_Cyrillic_U_straight_bar",        INT2FIX( XK_Cyrillic_U_straight_bar));   
  rb_define_global_const( "WIN_Cyrillic_u_straight_bar",        INT2FIX( XK_Cyrillic_u_straight_bar));   
  rb_define_global_const( "WIN_Cyrillic_HA_descender",          INT2FIX( XK_Cyrillic_HA_descender));     
  rb_define_global_const( "WIN_Cyrillic_ha_descender",          INT2FIX( XK_Cyrillic_ha_descender));     
  rb_define_global_const( "WIN_Cyrillic_CHE_descender",         INT2FIX( XK_Cyrillic_CHE_descender));    
  rb_define_global_const( "WIN_Cyrillic_che_descender",         INT2FIX( XK_Cyrillic_che_descender));    
  rb_define_global_const( "WIN_Cyrillic_CHE_vertstroke",        INT2FIX( XK_Cyrillic_CHE_vertstroke));   
  rb_define_global_const( "WIN_Cyrillic_che_vertstroke",        INT2FIX( XK_Cyrillic_che_vertstroke));   
  rb_define_global_const( "WIN_Cyrillic_SHHA",                  INT2FIX( XK_Cyrillic_SHHA));             
  rb_define_global_const( "WIN_Cyrillic_shha",                  INT2FIX( XK_Cyrillic_shha));             
  rb_define_global_const( "WIN_Cyrillic_SCHWA",                 INT2FIX( XK_Cyrillic_SCHWA));            
  rb_define_global_const( "WIN_Cyrillic_schwa",                 INT2FIX( XK_Cyrillic_schwa));            
  rb_define_global_const( "WIN_Cyrillic_I_macron",              INT2FIX( XK_Cyrillic_I_macron));         
  rb_define_global_const( "WIN_Cyrillic_i_macron",              INT2FIX( XK_Cyrillic_i_macron));         
  rb_define_global_const( "WIN_Cyrillic_O_bar",                 INT2FIX( XK_Cyrillic_O_bar));            
  rb_define_global_const( "WIN_Cyrillic_o_bar",                 INT2FIX( XK_Cyrillic_o_bar));            
  rb_define_global_const( "WIN_Cyrillic_U_macron",              INT2FIX( XK_Cyrillic_U_macron));         
  rb_define_global_const( "WIN_Cyrillic_u_macron",              INT2FIX( XK_Cyrillic_u_macron));         
  rb_define_global_const( "WIN_Serbian_dje",                    INT2FIX( XK_Serbian_dje));               
  rb_define_global_const( "WIN_Macedonia_gje",                  INT2FIX( XK_Macedonia_gje));             
  rb_define_global_const( "WIN_Cyrillic_io",                    INT2FIX( XK_Cyrillic_io));               
  rb_define_global_const( "WIN_Ukrainian_ie",                   INT2FIX( XK_Ukrainian_ie));              
  rb_define_global_const( "WIN_Ukranian_je",                    INT2FIX( XK_Ukranian_je));               
  rb_define_global_const( "WIN_Macedonia_dse",                  INT2FIX( XK_Macedonia_dse));             
  rb_define_global_const( "WIN_Ukrainian_i",                    INT2FIX( XK_Ukrainian_i));               
  rb_define_global_const( "WIN_Ukranian_i",                     INT2FIX( XK_Ukranian_i));                
  rb_define_global_const( "WIN_Ukrainian_yi",                   INT2FIX( XK_Ukrainian_yi));              
  rb_define_global_const( "WIN_Ukranian_yi",                    INT2FIX( XK_Ukranian_yi));               
  rb_define_global_const( "WIN_Cyrillic_je",                    INT2FIX( XK_Cyrillic_je));               
  rb_define_global_const( "WIN_Serbian_je",                     INT2FIX( XK_Serbian_je));                
  rb_define_global_const( "WIN_Cyrillic_lje",                   INT2FIX( XK_Cyrillic_lje));              
  rb_define_global_const( "WIN_Serbian_lje",                    INT2FIX( XK_Serbian_lje));               
  rb_define_global_const( "WIN_Cyrillic_nje",                   INT2FIX( XK_Cyrillic_nje));              
  rb_define_global_const( "WIN_Serbian_nje",                    INT2FIX( XK_Serbian_nje));               
  rb_define_global_const( "WIN_Serbian_tshe",                   INT2FIX( XK_Serbian_tshe));              
  rb_define_global_const( "WIN_Macedonia_kje",                  INT2FIX( XK_Macedonia_kje));             
  rb_define_global_const( "WIN_Ukrainian_ghe_with_upturn",      INT2FIX( XK_Ukrainian_ghe_with_upturn)); 
  rb_define_global_const( "WIN_Byelorussian_shortu",            INT2FIX( XK_Byelorussian_shortu));       
  rb_define_global_const( "WIN_Cyrillic_dzhe",                  INT2FIX( XK_Cyrillic_dzhe));             
  rb_define_global_const( "WIN_Serbian_dze",                    INT2FIX( XK_Serbian_dze));               
  rb_define_global_const( "WIN_numerosign",                     INT2FIX( XK_numerosign));                
  rb_define_global_const( "WIN_Serbian_DJE",                    INT2FIX( XK_Serbian_DJE));               
  rb_define_global_const( "WIN_Macedonia_GJE",                  INT2FIX( XK_Macedonia_GJE));             
  rb_define_global_const( "WIN_Cyrillic_IO",                    INT2FIX( XK_Cyrillic_IO));               
  rb_define_global_const( "WIN_Ukrainian_IE",                   INT2FIX( XK_Ukrainian_IE));              
  rb_define_global_const( "WIN_Ukranian_JE",                    INT2FIX( XK_Ukranian_JE));               
  rb_define_global_const( "WIN_Macedonia_DSE",                  INT2FIX( XK_Macedonia_DSE));             
  rb_define_global_const( "WIN_Ukrainian_I",                    INT2FIX( XK_Ukrainian_I));               
  rb_define_global_const( "WIN_Ukranian_I",                     INT2FIX( XK_Ukranian_I));                
  rb_define_global_const( "WIN_Ukrainian_YI",                   INT2FIX( XK_Ukrainian_YI));              
  rb_define_global_const( "WIN_Ukranian_YI",                    INT2FIX( XK_Ukranian_YI));               
  rb_define_global_const( "WIN_Cyrillic_JE",                    INT2FIX( XK_Cyrillic_JE));               
  rb_define_global_const( "WIN_Serbian_JE",                     INT2FIX( XK_Serbian_JE));                
  rb_define_global_const( "WIN_Cyrillic_LJE",                   INT2FIX( XK_Cyrillic_LJE));              
  rb_define_global_const( "WIN_Serbian_LJE",                    INT2FIX( XK_Serbian_LJE));               
  rb_define_global_const( "WIN_Cyrillic_NJE",                   INT2FIX( XK_Cyrillic_NJE));              
  rb_define_global_const( "WIN_Serbian_NJE",                    INT2FIX( XK_Serbian_NJE));               
  rb_define_global_const( "WIN_Serbian_TSHE",                   INT2FIX( XK_Serbian_TSHE));              
  rb_define_global_const( "WIN_Macedonia_KJE",                  INT2FIX( XK_Macedonia_KJE));             
  rb_define_global_const( "WIN_Ukrainian_GHE_WITH_UPTURN",      INT2FIX( XK_Ukrainian_GHE_WITH_UPTURN)); 
  rb_define_global_const( "WIN_Byelorussian_SHORTU",            INT2FIX( XK_Byelorussian_SHORTU));       
  rb_define_global_const( "WIN_Cyrillic_DZHE",                  INT2FIX( XK_Cyrillic_DZHE));             
  rb_define_global_const( "WIN_Serbian_DZE",                    INT2FIX( XK_Serbian_DZE));               
  rb_define_global_const( "WIN_Cyrillic_yu",                    INT2FIX( XK_Cyrillic_yu));               
  rb_define_global_const( "WIN_Cyrillic_a",                     INT2FIX( XK_Cyrillic_a));                
  rb_define_global_const( "WIN_Cyrillic_be",                    INT2FIX( XK_Cyrillic_be));               
  rb_define_global_const( "WIN_Cyrillic_tse",                   INT2FIX( XK_Cyrillic_tse));              
  rb_define_global_const( "WIN_Cyrillic_de",                    INT2FIX( XK_Cyrillic_de));               
  rb_define_global_const( "WIN_Cyrillic_ie",                    INT2FIX( XK_Cyrillic_ie));               
  rb_define_global_const( "WIN_Cyrillic_ef",                    INT2FIX( XK_Cyrillic_ef));               
  rb_define_global_const( "WIN_Cyrillic_ghe",                   INT2FIX( XK_Cyrillic_ghe));              
  rb_define_global_const( "WIN_Cyrillic_ha",                    INT2FIX( XK_Cyrillic_ha));               
  rb_define_global_const( "WIN_Cyrillic_i",                     INT2FIX( XK_Cyrillic_i));                
  rb_define_global_const( "WIN_Cyrillic_shorti",                INT2FIX( XK_Cyrillic_shorti));           
  rb_define_global_const( "WIN_Cyrillic_ka",                    INT2FIX( XK_Cyrillic_ka));               
  rb_define_global_const( "WIN_Cyrillic_el",                    INT2FIX( XK_Cyrillic_el));               
  rb_define_global_const( "WIN_Cyrillic_em",                    INT2FIX( XK_Cyrillic_em));               
  rb_define_global_const( "WIN_Cyrillic_en",                    INT2FIX( XK_Cyrillic_en));               
  rb_define_global_const( "WIN_Cyrillic_o",                     INT2FIX( XK_Cyrillic_o));                
  rb_define_global_const( "WIN_Cyrillic_pe",                    INT2FIX( XK_Cyrillic_pe));               
  rb_define_global_const( "WIN_Cyrillic_ya",                    INT2FIX( XK_Cyrillic_ya));               
  rb_define_global_const( "WIN_Cyrillic_er",                    INT2FIX( XK_Cyrillic_er));               
  rb_define_global_const( "WIN_Cyrillic_es",                    INT2FIX( XK_Cyrillic_es));               
  rb_define_global_const( "WIN_Cyrillic_te",                    INT2FIX( XK_Cyrillic_te));               
  rb_define_global_const( "WIN_Cyrillic_u",                     INT2FIX( XK_Cyrillic_u));                
  rb_define_global_const( "WIN_Cyrillic_zhe",                   INT2FIX( XK_Cyrillic_zhe));              
  rb_define_global_const( "WIN_Cyrillic_ve",                    INT2FIX( XK_Cyrillic_ve));               
  rb_define_global_const( "WIN_Cyrillic_softsign",              INT2FIX( XK_Cyrillic_softsign));         
  rb_define_global_const( "WIN_Cyrillic_yeru",                  INT2FIX( XK_Cyrillic_yeru));             
  rb_define_global_const( "WIN_Cyrillic_ze",                    INT2FIX( XK_Cyrillic_ze));               
  rb_define_global_const( "WIN_Cyrillic_sha",                   INT2FIX( XK_Cyrillic_sha));              
  rb_define_global_const( "WIN_Cyrillic_e",                     INT2FIX( XK_Cyrillic_e));                
  rb_define_global_const( "WIN_Cyrillic_shcha",                 INT2FIX( XK_Cyrillic_shcha));            
  rb_define_global_const( "WIN_Cyrillic_che",                   INT2FIX( XK_Cyrillic_che));              
  rb_define_global_const( "WIN_Cyrillic_hardsign",              INT2FIX( XK_Cyrillic_hardsign));         
  rb_define_global_const( "WIN_Cyrillic_YU",                    INT2FIX( XK_Cyrillic_YU));               
  rb_define_global_const( "WIN_Cyrillic_A",                     INT2FIX( XK_Cyrillic_A));                
  rb_define_global_const( "WIN_Cyrillic_BE",                    INT2FIX( XK_Cyrillic_BE));               
  rb_define_global_const( "WIN_Cyrillic_TSE",                   INT2FIX( XK_Cyrillic_TSE));              
  rb_define_global_const( "WIN_Cyrillic_DE",                    INT2FIX( XK_Cyrillic_DE));               
  rb_define_global_const( "WIN_Cyrillic_IE",                    INT2FIX( XK_Cyrillic_IE));               
  rb_define_global_const( "WIN_Cyrillic_EF",                    INT2FIX( XK_Cyrillic_EF));               
  rb_define_global_const( "WIN_Cyrillic_GHE",                   INT2FIX( XK_Cyrillic_GHE));              
  rb_define_global_const( "WIN_Cyrillic_HA",                    INT2FIX( XK_Cyrillic_HA));               
  rb_define_global_const( "WIN_Cyrillic_I",                     INT2FIX( XK_Cyrillic_I));                
  rb_define_global_const( "WIN_Cyrillic_SHORTI",                INT2FIX( XK_Cyrillic_SHORTI));           
  rb_define_global_const( "WIN_Cyrillic_KA",                    INT2FIX( XK_Cyrillic_KA));               
  rb_define_global_const( "WIN_Cyrillic_EL",                    INT2FIX( XK_Cyrillic_EL));               
  rb_define_global_const( "WIN_Cyrillic_EM",                    INT2FIX( XK_Cyrillic_EM));               
  rb_define_global_const( "WIN_Cyrillic_EN",                    INT2FIX( XK_Cyrillic_EN));               
  rb_define_global_const( "WIN_Cyrillic_O",                     INT2FIX( XK_Cyrillic_O));                
  rb_define_global_const( "WIN_Cyrillic_PE",                    INT2FIX( XK_Cyrillic_PE));               
  rb_define_global_const( "WIN_Cyrillic_YA",                    INT2FIX( XK_Cyrillic_YA));               
  rb_define_global_const( "WIN_Cyrillic_ER",                    INT2FIX( XK_Cyrillic_ER));               
  rb_define_global_const( "WIN_Cyrillic_ES",                    INT2FIX( XK_Cyrillic_ES));               
  rb_define_global_const( "WIN_Cyrillic_TE",                    INT2FIX( XK_Cyrillic_TE));               
  rb_define_global_const( "WIN_Cyrillic_U",                     INT2FIX( XK_Cyrillic_U));                
  rb_define_global_const( "WIN_Cyrillic_ZHE",                   INT2FIX( XK_Cyrillic_ZHE));              
  rb_define_global_const( "WIN_Cyrillic_VE",                    INT2FIX( XK_Cyrillic_VE));               
  rb_define_global_const( "WIN_Cyrillic_SOFTSIGN",              INT2FIX( XK_Cyrillic_SOFTSIGN));         
  rb_define_global_const( "WIN_Cyrillic_YERU",                  INT2FIX( XK_Cyrillic_YERU));             
  rb_define_global_const( "WIN_Cyrillic_ZE",                    INT2FIX( XK_Cyrillic_ZE));               
  rb_define_global_const( "WIN_Cyrillic_SHA",                   INT2FIX( XK_Cyrillic_SHA));              
  rb_define_global_const( "WIN_Cyrillic_E",                     INT2FIX( XK_Cyrillic_E));                
  rb_define_global_const( "WIN_Cyrillic_SHCHA",                 INT2FIX( XK_Cyrillic_SHCHA));            
  rb_define_global_const( "WIN_Cyrillic_CHE",                   INT2FIX( XK_Cyrillic_CHE));              
  rb_define_global_const( "WIN_Cyrillic_HARDSIGN",              INT2FIX( XK_Cyrillic_HARDSIGN));         
  rb_define_global_const( "WIN_Greek_ALPHAaccent",              INT2FIX( XK_Greek_ALPHAaccent));         
  rb_define_global_const( "WIN_Greek_EPSILONaccent",            INT2FIX( XK_Greek_EPSILONaccent));       
  rb_define_global_const( "WIN_Greek_ETAaccent",                INT2FIX( XK_Greek_ETAaccent));           
  rb_define_global_const( "WIN_Greek_IOTAaccent",               INT2FIX( XK_Greek_IOTAaccent));          
  rb_define_global_const( "WIN_Greek_IOTAdieresis",             INT2FIX( XK_Greek_IOTAdieresis));        
  rb_define_global_const( "WIN_Greek_IOTAdiaeresis",            INT2FIX( XK_Greek_IOTAdiaeresis));       
  rb_define_global_const( "WIN_Greek_OMICRONaccent",            INT2FIX( XK_Greek_OMICRONaccent));       
  rb_define_global_const( "WIN_Greek_UPSILONaccent",            INT2FIX( XK_Greek_UPSILONaccent));       
  rb_define_global_const( "WIN_Greek_UPSILONdieresis",          INT2FIX( XK_Greek_UPSILONdieresis));     
  rb_define_global_const( "WIN_Greek_OMEGAaccent",              INT2FIX( XK_Greek_OMEGAaccent));         
  rb_define_global_const( "WIN_Greek_accentdieresis",           INT2FIX( XK_Greek_accentdieresis));      
  rb_define_global_const( "WIN_Greek_horizbar",                 INT2FIX( XK_Greek_horizbar));            
  rb_define_global_const( "WIN_Greek_alphaaccent",              INT2FIX( XK_Greek_alphaaccent));         
  rb_define_global_const( "WIN_Greek_epsilonaccent",            INT2FIX( XK_Greek_epsilonaccent));       
  rb_define_global_const( "WIN_Greek_etaaccent",                INT2FIX( XK_Greek_etaaccent));           
  rb_define_global_const( "WIN_Greek_iotaaccent",               INT2FIX( XK_Greek_iotaaccent));          
  rb_define_global_const( "WIN_Greek_iotadieresis",             INT2FIX( XK_Greek_iotadieresis));        
  rb_define_global_const( "WIN_Greek_iotaaccentdieresis",       INT2FIX( XK_Greek_iotaaccentdieresis));  
  rb_define_global_const( "WIN_Greek_omicronaccent",            INT2FIX( XK_Greek_omicronaccent));       
  rb_define_global_const( "WIN_Greek_upsilonaccent",            INT2FIX( XK_Greek_upsilonaccent));       
  rb_define_global_const( "WIN_Greek_upsilondieresis",          INT2FIX( XK_Greek_upsilondieresis));     

  rb_define_global_const( "WIN_Greek_upsilonaccentdieresis",   INT2FIX( XK_Greek_upsilonaccentdieresis));
  rb_define_global_const( "WIN_Greek_omegaaccent",              INT2FIX( XK_Greek_omegaaccent));         
  rb_define_global_const( "WIN_Greek_ALPHA",                    INT2FIX( XK_Greek_ALPHA));               
  rb_define_global_const( "WIN_Greek_BETA",                     INT2FIX( XK_Greek_BETA));                
  rb_define_global_const( "WIN_Greek_GAMMA",                    INT2FIX( XK_Greek_GAMMA));               
  rb_define_global_const( "WIN_Greek_DELTA",                    INT2FIX( XK_Greek_DELTA));               
  rb_define_global_const( "WIN_Greek_EPSILON",                  INT2FIX( XK_Greek_EPSILON));             
  rb_define_global_const( "WIN_Greek_ZETA",                     INT2FIX( XK_Greek_ZETA));                
  rb_define_global_const( "WIN_Greek_ETA",                      INT2FIX( XK_Greek_ETA));                 
  rb_define_global_const( "WIN_Greek_THETA",                    INT2FIX( XK_Greek_THETA));               
  rb_define_global_const( "WIN_Greek_IOTA",                     INT2FIX( XK_Greek_IOTA));                
  rb_define_global_const( "WIN_Greek_KAPPA",                    INT2FIX( XK_Greek_KAPPA));               
  rb_define_global_const( "WIN_Greek_LAMDA",                    INT2FIX( XK_Greek_LAMDA));               
  rb_define_global_const( "WIN_Greek_LAMBDA",                   INT2FIX( XK_Greek_LAMBDA));              
  rb_define_global_const( "WIN_Greek_MU",                       INT2FIX( XK_Greek_MU));                  
  rb_define_global_const( "WIN_Greek_NU",                       INT2FIX( XK_Greek_NU));                  
  rb_define_global_const( "WIN_Greek_XI",                       INT2FIX( XK_Greek_XI));                  
  rb_define_global_const( "WIN_Greek_OMICRON",                  INT2FIX( XK_Greek_OMICRON));             
  rb_define_global_const( "WIN_Greek_PI",                       INT2FIX( XK_Greek_PI));                  
  rb_define_global_const( "WIN_Greek_RHO",                      INT2FIX( XK_Greek_RHO));                 
  rb_define_global_const( "WIN_Greek_SIGMA",                    INT2FIX( XK_Greek_SIGMA));               
  rb_define_global_const( "WIN_Greek_TAU",                      INT2FIX( XK_Greek_TAU));                 
  rb_define_global_const( "WIN_Greek_UPSILON",                  INT2FIX( XK_Greek_UPSILON));             
  rb_define_global_const( "WIN_Greek_PHI",                      INT2FIX( XK_Greek_PHI));                 
  rb_define_global_const( "WIN_Greek_CHI",                      INT2FIX( XK_Greek_CHI));                 
  rb_define_global_const( "WIN_Greek_PSI",                      INT2FIX( XK_Greek_PSI));                 
  rb_define_global_const( "WIN_Greek_OMEGA",                    INT2FIX( XK_Greek_OMEGA));               
  rb_define_global_const( "WIN_Greek_alpha",                    INT2FIX( XK_Greek_alpha));               
  rb_define_global_const( "WIN_Greek_beta",                     INT2FIX( XK_Greek_beta));                
  rb_define_global_const( "WIN_Greek_gamma",                    INT2FIX( XK_Greek_gamma));               
  rb_define_global_const( "WIN_Greek_delta",                    INT2FIX( XK_Greek_delta));               
  rb_define_global_const( "WIN_Greek_epsilon",                  INT2FIX( XK_Greek_epsilon));             
  rb_define_global_const( "WIN_Greek_zeta",                     INT2FIX( XK_Greek_zeta));                
  rb_define_global_const( "WIN_Greek_eta",                      INT2FIX( XK_Greek_eta));                 
  rb_define_global_const( "WIN_Greek_theta",                    INT2FIX( XK_Greek_theta));               
  rb_define_global_const( "WIN_Greek_iota",                     INT2FIX( XK_Greek_iota));                
  rb_define_global_const( "WIN_Greek_kappa",                    INT2FIX( XK_Greek_kappa));               
  rb_define_global_const( "WIN_Greek_lamda",                    INT2FIX( XK_Greek_lamda));               
  rb_define_global_const( "WIN_Greek_lambda",                   INT2FIX( XK_Greek_lambda));              
  rb_define_global_const( "WIN_Greek_mu",                       INT2FIX( XK_Greek_mu));                  
  rb_define_global_const( "WIN_Greek_nu",                       INT2FIX( XK_Greek_nu));                  
  rb_define_global_const( "WIN_Greek_xi",                       INT2FIX( XK_Greek_xi));                  
  rb_define_global_const( "WIN_Greek_omicron",                  INT2FIX( XK_Greek_omicron));             
  rb_define_global_const( "WIN_Greek_pi",                       INT2FIX( XK_Greek_pi));                  
  rb_define_global_const( "WIN_Greek_rho",                      INT2FIX( XK_Greek_rho));                 
  rb_define_global_const( "WIN_Greek_sigma",                    INT2FIX( XK_Greek_sigma));               
  rb_define_global_const( "WIN_Greek_finalsmallsigma",          INT2FIX( XK_Greek_finalsmallsigma));     
  rb_define_global_const( "WIN_Greek_tau",                      INT2FIX( XK_Greek_tau));                 
  rb_define_global_const( "WIN_Greek_upsilon",                  INT2FIX( XK_Greek_upsilon));             
  rb_define_global_const( "WIN_Greek_phi",                      INT2FIX( XK_Greek_phi));                 
  rb_define_global_const( "WIN_Greek_chi",                      INT2FIX( XK_Greek_chi));                 
  rb_define_global_const( "WIN_Greek_psi",                      INT2FIX( XK_Greek_psi));                 
  rb_define_global_const( "WIN_Greek_omega",                    INT2FIX( XK_Greek_omega));               
  rb_define_global_const( "WIN_Greek_switch",                   INT2FIX( XK_Greek_switch));              

#ifdef XK_technical
  rb_define_global_const( "WIN_leftradical",INT2FIX( XK_leftradical));               
  rb_define_global_const( "WIN_topleftradical",                 INT2FIX( XK_topleftradical));            
  rb_define_global_const( "WIN_horizconnector",                 INT2FIX( XK_horizconnector));            
  rb_define_global_const( "WIN_topintegral",                    INT2FIX( XK_topintegral));               
  rb_define_global_const( "WIN_botintegral",                    INT2FIX( XK_botintegral));               
  rb_define_global_const( "WIN_vertconnector",                  INT2FIX( XK_vertconnector));             
  rb_define_global_const( "WIN_topleftsqbracket",               INT2FIX( XK_topleftsqbracket));          
  rb_define_global_const( "WIN_botleftsqbracket",               INT2FIX( XK_botleftsqbracket));          
  rb_define_global_const( "WIN_toprightsqbracket",              INT2FIX( XK_toprightsqbracket));         
  rb_define_global_const( "WIN_botrightsqbracket",              INT2FIX( XK_botrightsqbracket));         
  rb_define_global_const( "WIN_topleftparens",                  INT2FIX( XK_topleftparens));             
  rb_define_global_const( "WIN_botleftparens",                  INT2FIX( XK_botleftparens));             
  rb_define_global_const( "WIN_toprightparens",                 INT2FIX( XK_toprightparens));            
  rb_define_global_const( "WIN_botrightparens",                 INT2FIX( XK_botrightparens));            
  rb_define_global_const( "WIN_leftmiddlecurlybrace",           INT2FIX( XK_leftmiddlecurlybrace));      
  rb_define_global_const( "WIN_rightmiddlecurlybrace",          INT2FIX( XK_rightmiddlecurlybrace));     
  rb_define_global_const( "WIN_topleftsummation",               INT2FIX( XK_topleftsummation));          
  rb_define_global_const( "WIN_botleftsummation",               INT2FIX( XK_botleftsummation));          
  rb_define_global_const( "WIN_topvertsummationconnector",      INT2FIX( XK_topvertsummationconnector)); 
  rb_define_global_const( "WIN_botvertsummationconnector",      INT2FIX( XK_botvertsummationconnector)); 
  rb_define_global_const( "WIN_toprightsummation",              INT2FIX( XK_toprightsummation));         
  rb_define_global_const( "WIN_botrightsummation",              INT2FIX( XK_botrightsummation));         
  rb_define_global_const( "WIN_rightmiddlesummation",           INT2FIX( XK_rightmiddlesummation));      
  rb_define_global_const( "WIN_lessthanequal",                  INT2FIX( XK_lessthanequal));             
  rb_define_global_const( "WIN_notequal",                       INT2FIX( XK_notequal));                  
  rb_define_global_const( "WIN_greaterthanequal",               INT2FIX( XK_greaterthanequal));          
  rb_define_global_const( "WIN_integral",                       INT2FIX( XK_integral));                  
  rb_define_global_const( "WIN_therefore",                      INT2FIX( XK_therefore));                 
  rb_define_global_const( "WIN_variation",                      INT2FIX( XK_variation));                 
  rb_define_global_const( "WIN_infinity",                       INT2FIX( XK_infinity));                  
  rb_define_global_const( "WIN_nabla",                          INT2FIX( XK_nabla));                     
  rb_define_global_const( "WIN_approximate",                    INT2FIX( XK_approximate));               
  rb_define_global_const( "WIN_similarequal",                   INT2FIX( XK_similarequal));              
  rb_define_global_const( "WIN_ifonlyif",                       INT2FIX( XK_ifonlyif));                  
  rb_define_global_const( "WIN_implies",                        INT2FIX( XK_implies));                   
  rb_define_global_const( "WIN_identical",                      INT2FIX( XK_identical));                 
  rb_define_global_const( "WIN_radical",                        INT2FIX( XK_radical));                   
  rb_define_global_const( "WIN_includedin",                     INT2FIX( XK_includedin));                
  rb_define_global_const( "WIN_includes",                       INT2FIX( XK_includes));                  
  rb_define_global_const( "WIN_intersection",                   INT2FIX( XK_intersection));              
  rb_define_global_const( "WIN_union",                          INT2FIX( XK_union));                     
  rb_define_global_const( "WIN_logicaland",                     INT2FIX( XK_logicaland));                
  rb_define_global_const( "WIN_logicalor",                      INT2FIX( XK_logicalor));                 
  rb_define_global_const( "WIN_partialderivative",              INT2FIX( XK_partialderivative));         
  rb_define_global_const( "WIN_function",                       INT2FIX( XK_function));                  
  rb_define_global_const( "WIN_leftarrow",                      INT2FIX( XK_leftarrow));                 
  rb_define_global_const( "WIN_uparrow",                        INT2FIX( XK_uparrow));                   
  rb_define_global_const( "WIN_rightarrow",                     INT2FIX( XK_rightarrow));                
  rb_define_global_const( "WIN_downarrow",                      INT2FIX( XK_downarrow));                 
#endif

#ifdef XK_SPECIAL
  rb_define_global_const( "WIN_blank",  INT2FIX( XK_blank));                     
  rb_define_global_const( "WIN_soliddiamond",                   INT2FIX( XK_soliddiamond));              
  rb_define_global_const( "WIN_checkerboard",                   INT2FIX( XK_checkerboard));              
  rb_define_global_const( "WIN_ht",                             INT2FIX( XK_ht));                        
  rb_define_global_const( "WIN_ff",                             INT2FIX( XK_ff));                        
  rb_define_global_const( "WIN_cr",                             INT2FIX( XK_cr));                        
  rb_define_global_const( "WIN_lf",                             INT2FIX( XK_lf));                        
  rb_define_global_const( "WIN_nl",                             INT2FIX( XK_nl));                        
  rb_define_global_const( "WIN_vt",                             INT2FIX( XK_vt));                        
  rb_define_global_const( "WIN_lowrightcorner",                 INT2FIX( XK_lowrightcorner));            
  rb_define_global_const( "WIN_uprightcorner",                  INT2FIX( XK_uprightcorner));             
  rb_define_global_const( "WIN_upleftcorner",                   INT2FIX( XK_upleftcorner));              
  rb_define_global_const( "WIN_lowleftcorner",                  INT2FIX( XK_lowleftcorner));             
  rb_define_global_const( "WIN_crossinglines",                  INT2FIX( XK_crossinglines));             
  rb_define_global_const( "WIN_horizlinescan1",                 INT2FIX( XK_horizlinescan1));            
  rb_define_global_const( "WIN_horizlinescan3",                 INT2FIX( XK_horizlinescan3));            
  rb_define_global_const( "WIN_horizlinescan5",                 INT2FIX( XK_horizlinescan5));            
  rb_define_global_const( "WIN_horizlinescan7",                 INT2FIX( XK_horizlinescan7));            
  rb_define_global_const( "WIN_horizlinescan9",                 INT2FIX( XK_horizlinescan9));            
  rb_define_global_const( "WIN_leftt",                          INT2FIX( XK_leftt));                     
  rb_define_global_const( "WIN_rightt",                         INT2FIX( XK_rightt));                    
  rb_define_global_const( "WIN_bott",                           INT2FIX( XK_bott));                      
  rb_define_global_const( "WIN_topt",                           INT2FIX( XK_topt));                      
  rb_define_global_const( "WIN_vertbar",                        INT2FIX( XK_vertbar));                   
#endif

#ifdef XK_PUBLISHING
  rb_define_global_const( "WIN_emspac",                                                                                        INT2FIX( XK_emspace));                   
  rb_define_global_const( "WIN_enspace",                        INT2FIX( XK_enspace));                   
  rb_define_global_const( "WIN_em3space",                       INT2FIX( XK_em3space));                  
  rb_define_global_const( "WIN_em4space",                       INT2FIX( XK_em4space));                  
  rb_define_global_const( "WIN_digitspace",                     INT2FIX( XK_digitspace));                
  rb_define_global_const( "WIN_punctspace",                     INT2FIX( XK_punctspace));                
  rb_define_global_const( "WIN_thinspace",                      INT2FIX( XK_thinspace));                 
  rb_define_global_const( "WIN_hairspace",                      INT2FIX( XK_hairspace));                 
  rb_define_global_const( "WIN_emdash",                         INT2FIX( XK_emdash));                    
  rb_define_global_const( "WIN_endash",                         INT2FIX( XK_endash));                    
  rb_define_global_const( "WIN_signifblank",                    INT2FIX( XK_signifblank));               
  rb_define_global_const( "WIN_ellipsis",                       INT2FIX( XK_ellipsis));                  
  rb_define_global_const( "WIN_doubbaselinedot",                INT2FIX( XK_doubbaselinedot));           
  rb_define_global_const( "WIN_onethird",                       INT2FIX( XK_onethird));                  
  rb_define_global_const( "WIN_twothirds",                      INT2FIX( XK_twothirds));                 
  rb_define_global_const( "WIN_onefifth",                       INT2FIX( XK_onefifth));                  
  rb_define_global_const( "WIN_twofifths",                      INT2FIX( XK_twofifths));                 
  rb_define_global_const( "WIN_threefifths",                    INT2FIX( XK_threefifths));               
  rb_define_global_const( "WIN_fourfifths",                     INT2FIX( XK_fourfifths));                
  rb_define_global_const( "WIN_onesixth",                       INT2FIX( XK_onesixth));                  
  rb_define_global_const( "WIN_fivesixths",                     INT2FIX( XK_fivesixths));                
  rb_define_global_const( "WIN_careof",                         INT2FIX( XK_careof));                    
  rb_define_global_const( "WIN_figdash",                        INT2FIX( XK_figdash));                   
  rb_define_global_const( "WIN_leftanglebracket",               INT2FIX( XK_leftanglebracket));          
  rb_define_global_const( "WIN_decimalpoint",                   INT2FIX( XK_decimalpoint));              
  rb_define_global_const( "WIN_rightanglebracket",              INT2FIX( XK_rightanglebracket));         
  rb_define_global_const( "WIN_marker",                         INT2FIX( XK_marker));                    
  rb_define_global_const( "WIN_oneeighth",                      INT2FIX( XK_oneeighth));                 
  rb_define_global_const( "WIN_threeeighths",                   INT2FIX( XK_threeeighths));              
  rb_define_global_const( "WIN_fiveeighths",                    INT2FIX( XK_fiveeighths));               
  rb_define_global_const( "WIN_seveneighths",                   INT2FIX( XK_seveneighths));              
  rb_define_global_const( "WIN_trademark",                      INT2FIX( XK_trademark));                 
  rb_define_global_const( "WIN_signaturemark",                  INT2FIX( XK_signaturemark));             
  rb_define_global_const( "WIN_trademarkincircle",              INT2FIX( XK_trademarkincircle));         
  rb_define_global_const( "WIN_leftopentriangle",               INT2FIX( XK_leftopentriangle));          
  rb_define_global_const( "WIN_rightopentriangle",              INT2FIX( XK_rightopentriangle));         
  rb_define_global_const( "WIN_emopencircle",                   INT2FIX( XK_emopencircle));              
  rb_define_global_const( "WIN_emopenrectangle",                INT2FIX( XK_emopenrectangle));           
  rb_define_global_const( "WIN_leftsinglequotemark",            INT2FIX( XK_leftsinglequotemark));       
  rb_define_global_const( "WIN_rightsinglequotemark",           INT2FIX( XK_rightsinglequotemark));      
  rb_define_global_const( "WIN_leftdoublequotemark",            INT2FIX( XK_leftdoublequotemark));       
  rb_define_global_const( "WIN_rightdoublequotemark",           INT2FIX( XK_rightdoublequotemark));      
  rb_define_global_const( "WIN_prescription",                   INT2FIX( XK_prescription));              
  rb_define_global_const( "WIN_minutes",                        INT2FIX( XK_minutes));                   
  rb_define_global_const( "WIN_seconds",                        INT2FIX( XK_seconds));                   
  rb_define_global_const( "WIN_latincross",                     INT2FIX( XK_latincross));                
  rb_define_global_const( "WIN_hexagram",                       INT2FIX( XK_hexagram));                  
  rb_define_global_const( "WIN_filledrectbullet",               INT2FIX( XK_filledrectbullet));          
  rb_define_global_const( "WIN_filledlefttribullet",            INT2FIX( XK_filledlefttribullet));       
  rb_define_global_const( "WIN_filledrighttribullet",           INT2FIX( XK_filledrighttribullet));      
  rb_define_global_const( "WIN_emfilledcircle",                 INT2FIX( XK_emfilledcircle));            
  rb_define_global_const( "WIN_emfilledrect",                   INT2FIX( XK_emfilledrect));              
  rb_define_global_const( "WIN_enopencircbullet",               INT2FIX( XK_enopencircbullet));          
  rb_define_global_const( "WIN_enopensquarebullet",             INT2FIX( XK_enopensquarebullet));        
  rb_define_global_const( "WIN_openrectbullet",                 INT2FIX( XK_openrectbullet));            
  rb_define_global_const( "WIN_opentribulletup",                INT2FIX( XK_opentribulletup));           
  rb_define_global_const( "WIN_opentribulletdown",              INT2FIX( XK_opentribulletdown));         
  rb_define_global_const( "WIN_openstar",                       INT2FIX( XK_openstar));                  
  rb_define_global_const( "WIN_enfilledcircbullet",             INT2FIX( XK_enfilledcircbullet));        
  rb_define_global_const( "WIN_enfilledsqbullet",               INT2FIX( XK_enfilledsqbullet));          
  rb_define_global_const( "WIN_filledtribulletup",              INT2FIX( XK_filledtribulletup));         
  rb_define_global_const( "WIN_filledtribulletdown",            INT2FIX( XK_filledtribulletdown));       
  rb_define_global_const( "WIN_leftpointer",                    INT2FIX( XK_leftpointer));               
  rb_define_global_const( "WIN_rightpointer",                   INT2FIX( XK_rightpointer));              
  rb_define_global_const( "WIN_club",                           INT2FIX( XK_club));                      
  rb_define_global_const( "WIN_diamond",                        INT2FIX( XK_diamond));                   
  rb_define_global_const( "WIN_heart",                          INT2FIX( XK_heart));                     
  rb_define_global_const( "WIN_maltesecross",                   INT2FIX( XK_maltesecross));              
  rb_define_global_const( "WIN_dagger",                         INT2FIX( XK_dagger));                    
  rb_define_global_const( "WIN_doubledagger",                   INT2FIX( XK_doubledagger));              
  rb_define_global_const( "WIN_checkmark",                      INT2FIX( XK_checkmark));                 
  rb_define_global_const( "WIN_ballotcross",                    INT2FIX( XK_ballotcross));               
  rb_define_global_const( "WIN_musicalsharp",                   INT2FIX( XK_musicalsharp));              
  rb_define_global_const( "WIN_musicalflat",                    INT2FIX( XK_musicalflat));               
  rb_define_global_const( "WIN_malesymbol",                     INT2FIX( XK_malesymbol));                
  rb_define_global_const( "WIN_femalesymbol",                   INT2FIX( XK_femalesymbol));              
  rb_define_global_const( "WIN_telephone",                      INT2FIX( XK_telephone));                 
  rb_define_global_const( "WIN_telephonerecorder",              INT2FIX( XK_telephonerecorder));         
  rb_define_global_const( "WIN_phonographcopyright",            INT2FIX( XK_phonographcopyright));       
  rb_define_global_const( "WIN_caret",                          INT2FIX( XK_caret));                     
  rb_define_global_const( "WIN_singlelowquotemark",             INT2FIX( XK_singlelowquotemark));        
  rb_define_global_const( "WIN_doublelowquotemark",             INT2FIX( XK_doublelowquotemark));        
  rb_define_global_const( "WIN_cursor",                         INT2FIX( XK_cursor));                    
#endif


#ifdef XK_APL
  rb_define_global_const( "WIN_leftcaret", INT2FIX( XK_leftcaret));                 
  rb_define_global_const( "WIN_rightcaret",                     INT2FIX( XK_rightcaret));                
  rb_define_global_const( "WIN_downcaret",                      INT2FIX( XK_downcaret));                 
  rb_define_global_const( "WIN_upcaret",                        INT2FIX( XK_upcaret));                   
  rb_define_global_const( "WIN_overbar",                        INT2FIX( XK_overbar));                   
  rb_define_global_const( "WIN_downtack",                       INT2FIX( XK_downtack));                  
  rb_define_global_const( "WIN_upshoe",                         INT2FIX( XK_upshoe));                    
  rb_define_global_const( "WIN_downstile",                      INT2FIX( XK_downstile));                 
  rb_define_global_const( "WIN_underbar",                       INT2FIX( XK_underbar));                  
  rb_define_global_const( "WIN_jot",                            INT2FIX( XK_jot));                       
  rb_define_global_const( "WIN_quad",                           INT2FIX( XK_quad));                      
  rb_define_global_const( "WIN_uptack",                         INT2FIX( XK_uptack));                    
  rb_define_global_const( "WIN_circle",                         INT2FIX( XK_circle));                    
  rb_define_global_const( "WIN_upstile",                        INT2FIX( XK_upstile));                   
  rb_define_global_const( "WIN_downshoe",                       INT2FIX( XK_downshoe));                  
  rb_define_global_const( "WIN_rightshoe",                      INT2FIX( XK_rightshoe));                 
  rb_define_global_const( "WIN_leftshoe",                       INT2FIX( XK_leftshoe));                  
  rb_define_global_const( "WIN_lefttack",                       INT2FIX( XK_lefttack));                  
  rb_define_global_const( "WIN_righttack",                      INT2FIX( XK_righttack));                 
#endif 

  rb_define_global_const( "WIN_hebrew_doublelowline",   INT2FIX( XK_hebrew_doublelowline));      
  rb_define_global_const( "WIN_hebrew_aleph",                   INT2FIX( XK_hebrew_aleph));              
  rb_define_global_const( "WIN_hebrew_bet",                     INT2FIX( XK_hebrew_bet));                
  rb_define_global_const( "WIN_hebrew_beth",                    INT2FIX( XK_hebrew_beth));               
  rb_define_global_const( "WIN_hebrew_gimel",                   INT2FIX( XK_hebrew_gimel));              
  rb_define_global_const( "WIN_hebrew_gimmel",                  INT2FIX( XK_hebrew_gimmel));             
  rb_define_global_const( "WIN_hebrew_dalet",                   INT2FIX( XK_hebrew_dalet));              
  rb_define_global_const( "WIN_hebrew_daleth",                  INT2FIX( XK_hebrew_daleth));             
  rb_define_global_const( "WIN_hebrew_he",                      INT2FIX( XK_hebrew_he));                 
  rb_define_global_const( "WIN_hebrew_waw",                     INT2FIX( XK_hebrew_waw));                
  rb_define_global_const( "WIN_hebrew_zain",                    INT2FIX( XK_hebrew_zain));               
  rb_define_global_const( "WIN_hebrew_zayin",                   INT2FIX( XK_hebrew_zayin));              
  rb_define_global_const( "WIN_hebrew_chet",                    INT2FIX( XK_hebrew_chet));               
  rb_define_global_const( "WIN_hebrew_het",                     INT2FIX( XK_hebrew_het));                
  rb_define_global_const( "WIN_hebrew_tet",                     INT2FIX( XK_hebrew_tet));                
  rb_define_global_const( "WIN_hebrew_teth",                    INT2FIX( XK_hebrew_teth));               
  rb_define_global_const( "WIN_hebrew_yod",                     INT2FIX( XK_hebrew_yod));                
  rb_define_global_const( "WIN_hebrew_finalkaph",               INT2FIX( XK_hebrew_finalkaph));          
  rb_define_global_const( "WIN_hebrew_kaph",                    INT2FIX( XK_hebrew_kaph));               
  rb_define_global_const( "WIN_hebrew_lamed",                   INT2FIX( XK_hebrew_lamed));              
  rb_define_global_const( "WIN_hebrew_finalmem",                INT2FIX( XK_hebrew_finalmem));           
  rb_define_global_const( "WIN_hebrew_mem",                     INT2FIX( XK_hebrew_mem));                
  rb_define_global_const( "WIN_hebrew_finalnun",                INT2FIX( XK_hebrew_finalnun));           
  rb_define_global_const( "WIN_hebrew_nun",                     INT2FIX( XK_hebrew_nun));                
  rb_define_global_const( "WIN_hebrew_samech",                  INT2FIX( XK_hebrew_samech));             
  rb_define_global_const( "WIN_hebrew_samekh",                  INT2FIX( XK_hebrew_samekh));             
  rb_define_global_const( "WIN_hebrew_ayin",                    INT2FIX( XK_hebrew_ayin));               
  rb_define_global_const( "WIN_hebrew_finalpe",                 INT2FIX( XK_hebrew_finalpe));            
  rb_define_global_const( "WIN_hebrew_pe",                      INT2FIX( XK_hebrew_pe));                 
  rb_define_global_const( "WIN_hebrew_finalzade",               INT2FIX( XK_hebrew_finalzade));          
  rb_define_global_const( "WIN_hebrew_finalzadi",               INT2FIX( XK_hebrew_finalzadi));          
  rb_define_global_const( "WIN_hebrew_zade",                    INT2FIX( XK_hebrew_zade));               
  rb_define_global_const( "WIN_hebrew_zadi",                    INT2FIX( XK_hebrew_zadi));               
  rb_define_global_const( "WIN_hebrew_qoph",                    INT2FIX( XK_hebrew_qoph));               
  rb_define_global_const( "WIN_hebrew_kuf",                     INT2FIX( XK_hebrew_kuf));                
  rb_define_global_const( "WIN_hebrew_resh",                    INT2FIX( XK_hebrew_resh));               
  rb_define_global_const( "WIN_hebrew_shin",                    INT2FIX( XK_hebrew_shin));               
  rb_define_global_const( "WIN_hebrew_taw",                     INT2FIX( XK_hebrew_taw));                
  rb_define_global_const( "WIN_hebrew_taf",                     INT2FIX( XK_hebrew_taf));                
  rb_define_global_const( "WIN_Hebrew_switch",                  INT2FIX( XK_Hebrew_switch));             
  rb_define_global_const( "WIN_Thai_kokai",                     INT2FIX( XK_Thai_kokai));                
  rb_define_global_const( "WIN_Thai_khokhai",                   INT2FIX( XK_Thai_khokhai));              
  rb_define_global_const( "WIN_Thai_khokhuat",                  INT2FIX( XK_Thai_khokhuat));             
  rb_define_global_const( "WIN_Thai_khokhwai",                  INT2FIX( XK_Thai_khokhwai));             
  rb_define_global_const( "WIN_Thai_khokhon",                   INT2FIX( XK_Thai_khokhon));              
  rb_define_global_const( "WIN_Thai_khorakhang",                INT2FIX( XK_Thai_khorakhang));           
  rb_define_global_const( "WIN_Thai_ngongu",                    INT2FIX( XK_Thai_ngongu));               
  rb_define_global_const( "WIN_Thai_chochan",                   INT2FIX( XK_Thai_chochan));              
  rb_define_global_const( "WIN_Thai_choching",                  INT2FIX( XK_Thai_choching));             
  rb_define_global_const( "WIN_Thai_chochang",                  INT2FIX( XK_Thai_chochang));             
  rb_define_global_const( "WIN_Thai_soso",                      INT2FIX( XK_Thai_soso));                 
  rb_define_global_const( "WIN_Thai_chochoe",                   INT2FIX( XK_Thai_chochoe));              
  rb_define_global_const( "WIN_Thai_yoying",                    INT2FIX( XK_Thai_yoying));               
  rb_define_global_const( "WIN_Thai_dochada",                   INT2FIX( XK_Thai_dochada));              
  rb_define_global_const( "WIN_Thai_topatak",                   INT2FIX( XK_Thai_topatak));              
  rb_define_global_const( "WIN_Thai_thothan",                   INT2FIX( XK_Thai_thothan));              
  rb_define_global_const( "WIN_Thai_thonangmontho",             INT2FIX( XK_Thai_thonangmontho));        
  rb_define_global_const( "WIN_Thai_thophuthao",                INT2FIX( XK_Thai_thophuthao));           
  rb_define_global_const( "WIN_Thai_nonen",                     INT2FIX( XK_Thai_nonen));                
  rb_define_global_const( "WIN_Thai_dodek",                     INT2FIX( XK_Thai_dodek));                
  rb_define_global_const( "WIN_Thai_totao",                     INT2FIX( XK_Thai_totao));                
  rb_define_global_const( "WIN_Thai_thothung",                  INT2FIX( XK_Thai_thothung));             
  rb_define_global_const( "WIN_Thai_thothahan",                 INT2FIX( XK_Thai_thothahan));            
  rb_define_global_const( "WIN_Thai_thothong",                  INT2FIX( XK_Thai_thothong));             
  rb_define_global_const( "WIN_Thai_nonu",                      INT2FIX( XK_Thai_nonu));                 
  rb_define_global_const( "WIN_Thai_bobaimai",                  INT2FIX( XK_Thai_bobaimai));             
  rb_define_global_const( "WIN_Thai_popla",                     INT2FIX( XK_Thai_popla));                
  rb_define_global_const( "WIN_Thai_phophung",                  INT2FIX( XK_Thai_phophung));             
  rb_define_global_const( "WIN_Thai_fofa",                      INT2FIX( XK_Thai_fofa));                 
  rb_define_global_const( "WIN_Thai_phophan",                   INT2FIX( XK_Thai_phophan));              
  rb_define_global_const( "WIN_Thai_fofan",                     INT2FIX( XK_Thai_fofan));                
  rb_define_global_const( "WIN_Thai_phosamphao",                INT2FIX( XK_Thai_phosamphao));           
  rb_define_global_const( "WIN_Thai_moma",                      INT2FIX( XK_Thai_moma));                 
  rb_define_global_const( "WIN_Thai_yoyak",                     INT2FIX( XK_Thai_yoyak));                
  rb_define_global_const( "WIN_Thai_rorua",                     INT2FIX( XK_Thai_rorua));                
  rb_define_global_const( "WIN_Thai_ru",                        INT2FIX( XK_Thai_ru));                   
  rb_define_global_const( "WIN_Thai_loling",                    INT2FIX( XK_Thai_loling));               
  rb_define_global_const( "WIN_Thai_lu",                        INT2FIX( XK_Thai_lu));                   
  rb_define_global_const( "WIN_Thai_wowaen",                    INT2FIX( XK_Thai_wowaen));               
  rb_define_global_const( "WIN_Thai_sosala",                    INT2FIX( XK_Thai_sosala));               
  rb_define_global_const( "WIN_Thai_sorusi",                    INT2FIX( XK_Thai_sorusi));               
  rb_define_global_const( "WIN_Thai_sosua",                     INT2FIX( XK_Thai_sosua));                
  rb_define_global_const( "WIN_Thai_hohip",                     INT2FIX( XK_Thai_hohip));                
  rb_define_global_const( "WIN_Thai_lochula",                   INT2FIX( XK_Thai_lochula));              
  rb_define_global_const( "WIN_Thai_oang",                      INT2FIX( XK_Thai_oang));                 
  rb_define_global_const( "WIN_Thai_honokhuk",                  INT2FIX( XK_Thai_honokhuk));             
  rb_define_global_const( "WIN_Thai_paiyannoi",                 INT2FIX( XK_Thai_paiyannoi));            
  rb_define_global_const( "WIN_Thai_saraa",                     INT2FIX( XK_Thai_saraa));                
  rb_define_global_const( "WIN_Thai_maihanakat",                INT2FIX( XK_Thai_maihanakat));           
  rb_define_global_const( "WIN_Thai_saraaa",                    INT2FIX( XK_Thai_saraaa));               
  rb_define_global_const( "WIN_Thai_saraam",                    INT2FIX( XK_Thai_saraam));               
  rb_define_global_const( "WIN_Thai_sarai",                     INT2FIX( XK_Thai_sarai));                
  rb_define_global_const( "WIN_Thai_saraii",                    INT2FIX( XK_Thai_saraii));               
  rb_define_global_const( "WIN_Thai_saraue",                    INT2FIX( XK_Thai_saraue));               
  rb_define_global_const( "WIN_Thai_sarauee",                   INT2FIX( XK_Thai_sarauee));              
  rb_define_global_const( "WIN_Thai_sarau",                     INT2FIX( XK_Thai_sarau));                
  rb_define_global_const( "WIN_Thai_sarauu",                    INT2FIX( XK_Thai_sarauu));               
  rb_define_global_const( "WIN_Thai_phinthu",                   INT2FIX( XK_Thai_phinthu));              
  rb_define_global_const( "WIN_Thai_maihanakat_maitho",         INT2FIX( XK_Thai_maihanakat_maitho));    
  rb_define_global_const( "WIN_Thai_baht",                      INT2FIX( XK_Thai_baht));                 
  rb_define_global_const( "WIN_Thai_sarae",                     INT2FIX( XK_Thai_sarae));                
  rb_define_global_const( "WIN_Thai_saraae",                    INT2FIX( XK_Thai_saraae));               
  rb_define_global_const( "WIN_Thai_sarao",                     INT2FIX( XK_Thai_sarao));                
  rb_define_global_const( "WIN_Thai_saraaimaimuan",             INT2FIX( XK_Thai_saraaimaimuan));        
  rb_define_global_const( "WIN_Thai_saraaimaimalai",            INT2FIX( XK_Thai_saraaimaimalai));       
  rb_define_global_const( "WIN_Thai_lakkhangyao",               INT2FIX( XK_Thai_lakkhangyao));          
  rb_define_global_const( "WIN_Thai_maiyamok",                  INT2FIX( XK_Thai_maiyamok));             
  rb_define_global_const( "WIN_Thai_maitaikhu",                 INT2FIX( XK_Thai_maitaikhu));            
  rb_define_global_const( "WIN_Thai_maiek",                     INT2FIX( XK_Thai_maiek));                
  rb_define_global_const( "WIN_Thai_maitho",                    INT2FIX( XK_Thai_maitho));               
  rb_define_global_const( "WIN_Thai_maitri",                    INT2FIX( XK_Thai_maitri));               
  rb_define_global_const( "WIN_Thai_maichattawa",               INT2FIX( XK_Thai_maichattawa));          
  rb_define_global_const( "WIN_Thai_thanthakhat",                                                                                       INT2FIX( XK_Thai_thanthakhat));          
  rb_define_global_const( "WIN_Thai_nikhahit",                  INT2FIX( XK_Thai_nikhahit));             
  rb_define_global_const( "WIN_Thai_leksun",                    INT2FIX( XK_Thai_leksun));               
  rb_define_global_const( "WIN_Thai_leknung",                   INT2FIX( XK_Thai_leknung));              
  rb_define_global_const( "WIN_Thai_leksong",                   INT2FIX( XK_Thai_leksong));              
  rb_define_global_const( "WIN_Thai_leksam",                    INT2FIX( XK_Thai_leksam));               
  rb_define_global_const( "WIN_Thai_leksi",                     INT2FIX( XK_Thai_leksi));                
  rb_define_global_const( "WIN_Thai_lekha",                     INT2FIX( XK_Thai_lekha));                
  rb_define_global_const( "WIN_Thai_lekhok",                    INT2FIX( XK_Thai_lekhok));               
  rb_define_global_const( "WIN_Thai_lekchet",                   INT2FIX( XK_Thai_lekchet));              
  rb_define_global_const( "WIN_Thai_lekpaet",                   INT2FIX( XK_Thai_lekpaet));              
  rb_define_global_const( "WIN_Thai_lekkao",                    INT2FIX( XK_Thai_lekkao));               
  rb_define_global_const( "WIN_Hangul",                         INT2FIX( XK_Hangul));                    
  rb_define_global_const( "WIN_Hangul_Start",                   INT2FIX( XK_Hangul_Start));              
  rb_define_global_const( "WIN_Hangul_End",                     INT2FIX( XK_Hangul_End));                
  rb_define_global_const( "WIN_Hangul_Hanja",                   INT2FIX( XK_Hangul_Hanja));              
  rb_define_global_const( "WIN_Hangul_Jamo",                    INT2FIX( XK_Hangul_Jamo));               
  rb_define_global_const( "WIN_Hangul_Romaja",                  INT2FIX( XK_Hangul_Romaja));             
  rb_define_global_const( "WIN_Hangul_Codeinput",               INT2FIX( XK_Hangul_Codeinput));          
  rb_define_global_const( "WIN_Hangul_Jeonja",                  INT2FIX( XK_Hangul_Jeonja));             
  rb_define_global_const( "WIN_Hangul_Banja",                   INT2FIX( XK_Hangul_Banja));              
  rb_define_global_const( "WIN_Hangul_PreHanja",                INT2FIX( XK_Hangul_PreHanja));           
  rb_define_global_const( "WIN_Hangul_PostHanja",               INT2FIX( XK_Hangul_PostHanja));          
  rb_define_global_const( "WIN_Hangul_SingleCandidate",         INT2FIX( XK_Hangul_SingleCandidate));    
  rb_define_global_const( "WIN_Hangul_MultipleCandidate",       INT2FIX( XK_Hangul_MultipleCandidate));  
  rb_define_global_const( "WIN_Hangul_PreviousCandidate",       INT2FIX( XK_Hangul_PreviousCandidate));  
  rb_define_global_const( "WIN_Hangul_Special",                 INT2FIX( XK_Hangul_Special));            
  rb_define_global_const( "WIN_Hangul_switch",                  INT2FIX( XK_Hangul_switch));             
  rb_define_global_const( "WIN_Hangul_Kiyeog",                  INT2FIX( XK_Hangul_Kiyeog));             
  rb_define_global_const( "WIN_Hangul_SsangKiyeog",             INT2FIX( XK_Hangul_SsangKiyeog));        
  rb_define_global_const( "WIN_Hangul_KiyeogSios",              INT2FIX( XK_Hangul_KiyeogSios));         
  rb_define_global_const( "WIN_Hangul_Nieun",                   INT2FIX( XK_Hangul_Nieun));              
  rb_define_global_const( "WIN_Hangul_NieunJieuj",              INT2FIX( XK_Hangul_NieunJieuj));         
  rb_define_global_const( "WIN_Hangul_NieunHieuh",              INT2FIX( XK_Hangul_NieunHieuh));         
  rb_define_global_const( "WIN_Hangul_Dikeud",                  INT2FIX( XK_Hangul_Dikeud));             
  rb_define_global_const( "WIN_Hangul_SsangDikeud",             INT2FIX( XK_Hangul_SsangDikeud));        
  rb_define_global_const( "WIN_Hangul_Rieul",                   INT2FIX( XK_Hangul_Rieul));              
  rb_define_global_const( "WIN_Hangul_RieulKiyeog",             INT2FIX( XK_Hangul_RieulKiyeog));        
  rb_define_global_const( "WIN_Hangul_RieulMieum",              INT2FIX( XK_Hangul_RieulMieum));         
  rb_define_global_const( "WIN_Hangul_RieulPieub",              INT2FIX( XK_Hangul_RieulPieub));         
  rb_define_global_const( "WIN_Hangul_RieulSios",               INT2FIX( XK_Hangul_RieulSios));          
  rb_define_global_const( "WIN_Hangul_RieulTieut",              INT2FIX( XK_Hangul_RieulTieut));         
  rb_define_global_const( "WIN_Hangul_RieulPhieuf",             INT2FIX( XK_Hangul_RieulPhieuf));        
  rb_define_global_const( "WIN_Hangul_RieulHieuh",              INT2FIX( XK_Hangul_RieulHieuh));         
  rb_define_global_const( "WIN_Hangul_Mieum",                   INT2FIX( XK_Hangul_Mieum));              
  rb_define_global_const( "WIN_Hangul_Pieub",                   INT2FIX( XK_Hangul_Pieub));              
  rb_define_global_const( "WIN_Hangul_SsangPieub",              INT2FIX( XK_Hangul_SsangPieub));         
  rb_define_global_const( "WIN_Hangul_PieubSios",               INT2FIX( XK_Hangul_PieubSios));          
  rb_define_global_const( "WIN_Hangul_Sios",                    INT2FIX( XK_Hangul_Sios));               
  rb_define_global_const( "WIN_Hangul_SsangSios",               INT2FIX( XK_Hangul_SsangSios));          
  rb_define_global_const( "WIN_Hangul_Ieung",                   INT2FIX( XK_Hangul_Ieung));              
  rb_define_global_const( "WIN_Hangul_Jieuj",                   INT2FIX( XK_Hangul_Jieuj));              
  rb_define_global_const( "WIN_Hangul_SsangJieuj",              INT2FIX( XK_Hangul_SsangJieuj));         
  rb_define_global_const( "WIN_Hangul_Cieuc",                   INT2FIX( XK_Hangul_Cieuc));              
  rb_define_global_const( "WIN_Hangul_Khieuq",                  INT2FIX( XK_Hangul_Khieuq));             
  rb_define_global_const( "WIN_Hangul_Tieut",                   INT2FIX( XK_Hangul_Tieut));              
  rb_define_global_const( "WIN_Hangul_Phieuf",                  INT2FIX( XK_Hangul_Phieuf));             
  rb_define_global_const( "WIN_Hangul_Hieuh",                   INT2FIX( XK_Hangul_Hieuh));              
  rb_define_global_const( "WIN_Hangul_A",                       INT2FIX( XK_Hangul_A));                  
  rb_define_global_const( "WIN_Hangul_AE",                      INT2FIX( XK_Hangul_AE));                 
  rb_define_global_const( "WIN_Hangul_YA",                      INT2FIX( XK_Hangul_YA));                 
  rb_define_global_const( "WIN_Hangul_YAE",                     INT2FIX( XK_Hangul_YAE));                
  rb_define_global_const( "WIN_Hangul_EO",                      INT2FIX( XK_Hangul_EO));                 
  rb_define_global_const( "WIN_Hangul_E",                       INT2FIX( XK_Hangul_E));                  
  rb_define_global_const( "WIN_Hangul_YEO",                     INT2FIX( XK_Hangul_YEO));                
  rb_define_global_const( "WIN_Hangul_YE",                      INT2FIX( XK_Hangul_YE));                 
  rb_define_global_const( "WIN_Hangul_O",                       INT2FIX( XK_Hangul_O));                  
  rb_define_global_const( "WIN_Hangul_WA",                      INT2FIX( XK_Hangul_WA));                 
  rb_define_global_const( "WIN_Hangul_WAE",                     INT2FIX( XK_Hangul_WAE));                
  rb_define_global_const( "WIN_Hangul_OE",                      INT2FIX( XK_Hangul_OE));                 
  rb_define_global_const( "WIN_Hangul_YO",                      INT2FIX( XK_Hangul_YO));                 
  rb_define_global_const( "WIN_Hangul_U",                       INT2FIX( XK_Hangul_U));                  
  rb_define_global_const( "WIN_Hangul_WEO",                     INT2FIX( XK_Hangul_WEO));                
  rb_define_global_const( "WIN_Hangul_WE",                      INT2FIX( XK_Hangul_WE));                 
  rb_define_global_const( "WIN_Hangul_WI",                      INT2FIX( XK_Hangul_WI));                 
  rb_define_global_const( "WIN_Hangul_YU",                      INT2FIX( XK_Hangul_YU));                 
  rb_define_global_const( "WIN_Hangul_EU",                      INT2FIX( XK_Hangul_EU));                 
  rb_define_global_const( "WIN_Hangul_YI",                      INT2FIX( XK_Hangul_YI));                 
  rb_define_global_const( "WIN_Hangul_I",                       INT2FIX( XK_Hangul_I));                  
  rb_define_global_const( "WIN_Hangul_J_Kiyeog",                INT2FIX( XK_Hangul_J_Kiyeog));           
  rb_define_global_const( "WIN_Hangul_J_SsangKiyeog",           INT2FIX( XK_Hangul_J_SsangKiyeog));      
  rb_define_global_const( "WIN_Hangul_J_KiyeogSios",            INT2FIX( XK_Hangul_J_KiyeogSios));       
  rb_define_global_const( "WIN_Hangul_J_Nieun",                 INT2FIX( XK_Hangul_J_Nieun));            
  rb_define_global_const( "WIN_Hangul_J_NieunJieuj",            INT2FIX( XK_Hangul_J_NieunJieuj));       
  rb_define_global_const( "WIN_Hangul_J_NieunHieuh",            INT2FIX( XK_Hangul_J_NieunHieuh));       
  rb_define_global_const( "WIN_Hangul_J_Dikeud",                INT2FIX( XK_Hangul_J_Dikeud));           
  rb_define_global_const( "WIN_Hangul_J_Rieul",                 INT2FIX( XK_Hangul_J_Rieul));            
  rb_define_global_const( "WIN_Hangul_J_RieulKiyeog",           INT2FIX( XK_Hangul_J_RieulKiyeog));      
  rb_define_global_const( "WIN_Hangul_J_RieulMieum",            INT2FIX( XK_Hangul_J_RieulMieum));       
  rb_define_global_const( "WIN_Hangul_J_RieulPieub",            INT2FIX( XK_Hangul_J_RieulPieub));       
  rb_define_global_const( "WIN_Hangul_J_RieulSios",             INT2FIX( XK_Hangul_J_RieulSios));        
  rb_define_global_const( "WIN_Hangul_J_RieulTieut",            INT2FIX( XK_Hangul_J_RieulTieut));       
  rb_define_global_const( "WIN_Hangul_J_RieulPhieuf",           INT2FIX( XK_Hangul_J_RieulPhieuf));      
  rb_define_global_const( "WIN_Hangul_J_RieulHieuh",            INT2FIX( XK_Hangul_J_RieulHieuh));       
  rb_define_global_const( "WIN_Hangul_J_Mieum",                 INT2FIX( XK_Hangul_J_Mieum));            
  rb_define_global_const( "WIN_Hangul_J_Pieub",                 INT2FIX( XK_Hangul_J_Pieub));            
  rb_define_global_const( "WIN_Hangul_J_PieubSios",             INT2FIX( XK_Hangul_J_PieubSios));        
  rb_define_global_const( "WIN_Hangul_J_Sios",                  INT2FIX( XK_Hangul_J_Sios));             
  rb_define_global_const( "WIN_Hangul_J_SsangSios",             INT2FIX( XK_Hangul_J_SsangSios));        
  rb_define_global_const( "WIN_Hangul_J_Ieung",                 INT2FIX( XK_Hangul_J_Ieung));            
  rb_define_global_const( "WIN_Hangul_J_Jieuj",                 INT2FIX( XK_Hangul_J_Jieuj));            
  rb_define_global_const( "WIN_Hangul_J_Cieuc",                 INT2FIX( XK_Hangul_J_Cieuc));            
  rb_define_global_const( "WIN_Hangul_J_Khieuq",                INT2FIX( XK_Hangul_J_Khieuq));           
  rb_define_global_const( "WIN_Hangul_J_Tieut",                 INT2FIX( XK_Hangul_J_Tieut));            
  rb_define_global_const( "WIN_Hangul_J_Phieuf",                INT2FIX( XK_Hangul_J_Phieuf));           
  rb_define_global_const( "WIN_Hangul_J_Hieuh",                 INT2FIX( XK_Hangul_J_Hieuh));            
  rb_define_global_const( "WIN_Hangul_RieulYeorinHieuh",        INT2FIX( XK_Hangul_RieulYeorinHieuh));   
  rb_define_global_const( "WIN_Hangul_SunkyeongeumMieum",       INT2FIX( XK_Hangul_SunkyeongeumMieum));  
  rb_define_global_const( "WIN_Hangul_SunkyeongeumPieub",       INT2FIX( XK_Hangul_SunkyeongeumPieub));  
  rb_define_global_const( "WIN_Hangul_PanSios",                 INT2FIX( XK_Hangul_PanSios));            
  rb_define_global_const( "WIN_Hangul_KkogjiDalrinIeung",       INT2FIX( XK_Hangul_KkogjiDalrinIeung));  
  rb_define_global_const( "WIN_Hangul_SunkyeongeumPhieuf",      INT2FIX( XK_Hangul_SunkyeongeumPhieuf)); 
  rb_define_global_const( "WIN_Hangul_YeorinHieuh",             INT2FIX( XK_Hangul_YeorinHieuh));        
  rb_define_global_const( "WIN_Hangul_AraeA",                   INT2FIX( XK_Hangul_AraeA));              
  rb_define_global_const( "WIN_Hangul_AraeAE",                  INT2FIX( XK_Hangul_AraeAE));             
  rb_define_global_const( "WIN_Hangul_J_PanSios",               INT2FIX( XK_Hangul_J_PanSios));          
  rb_define_global_const( "WIN_Hangul_J_KkogjiDalrinIeung",     INT2FIX( XK_Hangul_J_KkogjiDalrinIeung));
  rb_define_global_const( "WIN_Hangul_J_YeorinHieuh",           INT2FIX( XK_Hangul_J_YeorinHieuh));      
  rb_define_global_const( "WIN_Korean_Won",                     INT2FIX( XK_Korean_Won));                
  rb_define_global_const( "WIN_Armenian_ligature_ew",           INT2FIX( XK_Armenian_ligature_ew));      
  rb_define_global_const( "WIN_Armenian_full_stop",             INT2FIX( XK_Armenian_full_stop));        
  rb_define_global_const( "WIN_Armenian_verjaket",              INT2FIX( XK_Armenian_verjaket));         
  rb_define_global_const( "WIN_Armenian_separation_mark",       INT2FIX( XK_Armenian_separation_mark));  
  rb_define_global_const( "WIN_Armenian_but",                   INT2FIX( XK_Armenian_but));              
  rb_define_global_const( "WIN_Armenian_hyphen",                INT2FIX( XK_Armenian_hyphen));           
  rb_define_global_const( "WIN_Armenian_yentamna",              INT2FIX( XK_Armenian_yentamna));         
  rb_define_global_const( "WIN_Armenian_exclam",                INT2FIX( XK_Armenian_exclam));           
  rb_define_global_const( "WIN_Armenian_amanak",                INT2FIX( XK_Armenian_amanak));           
  rb_define_global_const( "WIN_Armenian_accent",                INT2FIX( XK_Armenian_accent));           
  rb_define_global_const( "WIN_Armenian_shesht",                INT2FIX( XK_Armenian_shesht));           
  rb_define_global_const( "WIN_Armenian_question",              INT2FIX( XK_Armenian_question));         
  rb_define_global_const( "WIN_Armenian_paruyk",                INT2FIX( XK_Armenian_paruyk));           
  rb_define_global_const( "WIN_Armenian_AYB",                   INT2FIX( XK_Armenian_AYB));              
  rb_define_global_const( "WIN_Armenian_ayb",                   INT2FIX( XK_Armenian_ayb));              
  rb_define_global_const( "WIN_Armenian_BEN",                   INT2FIX( XK_Armenian_BEN));              
  rb_define_global_const( "WIN_Armenian_ben",                   INT2FIX( XK_Armenian_ben));              
  rb_define_global_const( "WIN_Armenian_GIM",                   INT2FIX( XK_Armenian_GIM));              
  rb_define_global_const( "WIN_Armenian_gim",                   INT2FIX( XK_Armenian_gim));              
  rb_define_global_const( "WIN_Armenian_DA",                    INT2FIX( XK_Armenian_DA));               
  rb_define_global_const( "WIN_Armenian_da",                    INT2FIX( XK_Armenian_da));               
  rb_define_global_const( "WIN_Armenian_YECH",                  INT2FIX( XK_Armenian_YECH));             
  rb_define_global_const( "WIN_Armenian_yech",                  INT2FIX( XK_Armenian_yech));             
  rb_define_global_const( "WIN_Armenian_ZA",                    INT2FIX( XK_Armenian_ZA));               
  rb_define_global_const( "WIN_Armenian_za",                    INT2FIX( XK_Armenian_za));               
  rb_define_global_const( "WIN_Armenian_E",                     INT2FIX( XK_Armenian_E));                
  rb_define_global_const( "WIN_Armenian_e",                     INT2FIX( XK_Armenian_e));                
  rb_define_global_const( "WIN_Armenian_AT",                    INT2FIX( XK_Armenian_AT));               
  rb_define_global_const( "WIN_Armenian_at",                    INT2FIX( XK_Armenian_at));               
  rb_define_global_const( "WIN_Armenian_TO",                    INT2FIX( XK_Armenian_TO));               
  rb_define_global_const( "WIN_Armenian_to",                    INT2FIX( XK_Armenian_to));               
  rb_define_global_const( "WIN_Armenian_ZHE",                   INT2FIX( XK_Armenian_ZHE));              
  rb_define_global_const( "WIN_Armenian_zhe",                   INT2FIX( XK_Armenian_zhe));              
  rb_define_global_const( "WIN_Armenian_INI",                   INT2FIX( XK_Armenian_INI));              
  rb_define_global_const( "WIN_Armenian_ini",                   INT2FIX( XK_Armenian_ini));              
  rb_define_global_const( "WIN_Armenian_LYUN",                  INT2FIX( XK_Armenian_LYUN));             
  rb_define_global_const( "WIN_Armenian_lyun",                  INT2FIX( XK_Armenian_lyun));             
  rb_define_global_const( "WIN_Armenian_KHE",                   INT2FIX( XK_Armenian_KHE));              
  rb_define_global_const( "WIN_Armenian_khe",                   INT2FIX( XK_Armenian_khe));              
  rb_define_global_const( "WIN_Armenian_TSA",                   INT2FIX( XK_Armenian_TSA));              
  rb_define_global_const( "WIN_Armenian_tsa",                   INT2FIX( XK_Armenian_tsa));              
  rb_define_global_const( "WIN_Armenian_KEN",                   INT2FIX( XK_Armenian_KEN));              
  rb_define_global_const( "WIN_Armenian_ken",                   INT2FIX( XK_Armenian_ken));              
  rb_define_global_const( "WIN_Armenian_HO",                    INT2FIX( XK_Armenian_HO));               
  rb_define_global_const( "WIN_Armenian_ho",                    INT2FIX( XK_Armenian_ho));               
  rb_define_global_const( "WIN_Armenian_DZA",                   INT2FIX( XK_Armenian_DZA));              
  rb_define_global_const( "WIN_Armenian_dza",                   INT2FIX( XK_Armenian_dza));              
  rb_define_global_const( "WIN_Armenian_GHAT",                  INT2FIX( XK_Armenian_GHAT));             
  rb_define_global_const( "WIN_Armenian_ghat",                  INT2FIX( XK_Armenian_ghat));             
  rb_define_global_const( "WIN_Armenian_TCHE",                  INT2FIX( XK_Armenian_TCHE));             
  rb_define_global_const( "WIN_Armenian_tche",                  INT2FIX( XK_Armenian_tche));             
  rb_define_global_const( "WIN_Armenian_MEN",                   INT2FIX( XK_Armenian_MEN));              
  rb_define_global_const( "WIN_Armenian_men",                   INT2FIX( XK_Armenian_men));              
  rb_define_global_const( "WIN_Armenian_HI",                    INT2FIX( XK_Armenian_HI));               
  rb_define_global_const( "WIN_Armenian_hi",                    INT2FIX( XK_Armenian_hi));               
  rb_define_global_const( "WIN_Armenian_NU",                    INT2FIX( XK_Armenian_NU));               
  rb_define_global_const( "WIN_Armenian_nu",                    INT2FIX( XK_Armenian_nu));               
  rb_define_global_const( "WIN_Armenian_SHA",                   INT2FIX( XK_Armenian_SHA));              
  rb_define_global_const( "WIN_Armenian_sha",                   INT2FIX( XK_Armenian_sha));              
  rb_define_global_const( "WIN_Armenian_VO",                    INT2FIX( XK_Armenian_VO));               
  rb_define_global_const( "WIN_Armenian_vo",                    INT2FIX( XK_Armenian_vo));               
  rb_define_global_const( "WIN_Armenian_CHA",                   INT2FIX( XK_Armenian_CHA));              
  rb_define_global_const( "WIN_Armenian_cha",                   INT2FIX( XK_Armenian_cha));              
  rb_define_global_const( "WIN_Armenian_PE",                    INT2FIX( XK_Armenian_PE));               
  rb_define_global_const( "WIN_Armenian_pe",                    INT2FIX( XK_Armenian_pe));               
  rb_define_global_const( "WIN_Armenian_JE",                    INT2FIX( XK_Armenian_JE));               
  rb_define_global_const( "WIN_Armenian_je",                    INT2FIX( XK_Armenian_je));               
  rb_define_global_const( "WIN_Armenian_RA",                    INT2FIX( XK_Armenian_RA));               
  rb_define_global_const( "WIN_Armenian_ra",                    INT2FIX( XK_Armenian_ra));               
  rb_define_global_const( "WIN_Armenian_SE",                    INT2FIX( XK_Armenian_SE));               
  rb_define_global_const( "WIN_Armenian_se",                    INT2FIX( XK_Armenian_se));               
  rb_define_global_const( "WIN_Armenian_VEV",                   INT2FIX( XK_Armenian_VEV));              
  rb_define_global_const( "WIN_Armenian_vev",                   INT2FIX( XK_Armenian_vev));              
  rb_define_global_const( "WIN_Armenian_TYUN",                  INT2FIX( XK_Armenian_TYUN));             
  rb_define_global_const( "WIN_Armenian_tyun",                  INT2FIX( XK_Armenian_tyun));             
  rb_define_global_const( "WIN_Armenian_RE",                    INT2FIX( XK_Armenian_RE));               
  rb_define_global_const( "WIN_Armenian_re",                    INT2FIX( XK_Armenian_re));               
  rb_define_global_const( "WIN_Armenian_TSO",                   INT2FIX( XK_Armenian_TSO));              
  rb_define_global_const( "WIN_Armenian_tso",                   INT2FIX( XK_Armenian_tso));              
  rb_define_global_const( "WIN_Armenian_VYUN",                  INT2FIX( XK_Armenian_VYUN));             
  rb_define_global_const( "WIN_Armenian_vyun",                  INT2FIX( XK_Armenian_vyun));             
  rb_define_global_const( "WIN_Armenian_PYUR",                  INT2FIX( XK_Armenian_PYUR));             
  rb_define_global_const( "WIN_Armenian_pyur",                  INT2FIX( XK_Armenian_pyur));             
  rb_define_global_const( "WIN_Armenian_KE",                    INT2FIX( XK_Armenian_KE));               
  rb_define_global_const( "WIN_Armenian_ke",                    INT2FIX( XK_Armenian_ke));               
  rb_define_global_const( "WIN_Armenian_O",                     INT2FIX( XK_Armenian_O));                
  rb_define_global_const( "WIN_Armenian_o",                     INT2FIX( XK_Armenian_o));                
  rb_define_global_const( "WIN_Armenian_FE",                    INT2FIX( XK_Armenian_FE));               
  rb_define_global_const( "WIN_Armenian_fe",                    INT2FIX( XK_Armenian_fe));               
  rb_define_global_const( "WIN_Armenian_apostrophe",            INT2FIX( XK_Armenian_apostrophe));       
  rb_define_global_const( "WIN_Georgian_an",                    INT2FIX( XK_Georgian_an));               
  rb_define_global_const( "WIN_Georgian_ban",                   INT2FIX( XK_Georgian_ban));              
  rb_define_global_const( "WIN_Georgian_gan",                   INT2FIX( XK_Georgian_gan));              
  rb_define_global_const( "WIN_Georgian_don",                   INT2FIX( XK_Georgian_don));              
  rb_define_global_const( "WIN_Georgian_en",                    INT2FIX( XK_Georgian_en));               
  rb_define_global_const( "WIN_Georgian_vin",                   INT2FIX( XK_Georgian_vin));              
  rb_define_global_const( "WIN_Georgian_zen",                   INT2FIX( XK_Georgian_zen));              
  rb_define_global_const( "WIN_Georgian_tan",                   INT2FIX( XK_Georgian_tan));              
  rb_define_global_const( "WIN_Georgian_in",                    INT2FIX( XK_Georgian_in));               
  rb_define_global_const( "WIN_Georgian_kan",                   INT2FIX( XK_Georgian_kan));              
  rb_define_global_const( "WIN_Georgian_las",                   INT2FIX( XK_Georgian_las));              
  rb_define_global_const( "WIN_Georgian_man",                   INT2FIX( XK_Georgian_man));              
  rb_define_global_const( "WIN_Georgian_nar",                   INT2FIX( XK_Georgian_nar));              
  rb_define_global_const( "WIN_Georgian_on",                    INT2FIX( XK_Georgian_on));               
  rb_define_global_const( "WIN_Georgian_par",                   INT2FIX( XK_Georgian_par));              
  rb_define_global_const( "WIN_Georgian_zhar",                  INT2FIX( XK_Georgian_zhar));             
  rb_define_global_const( "WIN_Georgian_rae",                   INT2FIX( XK_Georgian_rae));              
  rb_define_global_const( "WIN_Georgian_san",                   INT2FIX( XK_Georgian_san));              
  rb_define_global_const( "WIN_Georgian_tar",                   INT2FIX( XK_Georgian_tar));              
  rb_define_global_const( "WIN_Georgian_un",                    INT2FIX( XK_Georgian_un));               
  rb_define_global_const( "WIN_Georgian_phar",                  INT2FIX( XK_Georgian_phar));             
  rb_define_global_const( "WIN_Georgian_khar",                  INT2FIX( XK_Georgian_khar));             
  rb_define_global_const( "WIN_Georgian_ghan",                  INT2FIX( XK_Georgian_ghan));             
  rb_define_global_const( "WIN_Georgian_qar",                   INT2FIX( XK_Georgian_qar));              
  rb_define_global_const( "WIN_Georgian_shin",                  INT2FIX( XK_Georgian_shin));             
  rb_define_global_const( "WIN_Georgian_chin",                  INT2FIX( XK_Georgian_chin));             
  rb_define_global_const( "WIN_Georgian_can",                   INT2FIX( XK_Georgian_can));              
  rb_define_global_const( "WIN_Georgian_jil",                   INT2FIX( XK_Georgian_jil));              
  rb_define_global_const( "WIN_Georgian_cil",                   INT2FIX( XK_Georgian_cil));              
  rb_define_global_const( "WIN_Georgian_char",                  INT2FIX( XK_Georgian_char));             
  rb_define_global_const( "WIN_Georgian_xan",                   INT2FIX( XK_Georgian_xan));              
  rb_define_global_const( "WIN_Georgian_jhan",                  INT2FIX( XK_Georgian_jhan));             
  rb_define_global_const( "WIN_Georgian_hae",                   INT2FIX( XK_Georgian_hae));              
  rb_define_global_const( "WIN_Georgian_he",                    INT2FIX( XK_Georgian_he));               
  rb_define_global_const( "WIN_Georgian_hie",                   INT2FIX( XK_Georgian_hie));              
  rb_define_global_const( "WIN_Georgian_we",                    INT2FIX( XK_Georgian_we));               
  rb_define_global_const( "WIN_Georgian_har",                   INT2FIX( XK_Georgian_har));              
  rb_define_global_const( "WIN_Georgian_hoe",                   INT2FIX( XK_Georgian_hoe));              
  rb_define_global_const( "WIN_Georgian_fi",                    INT2FIX( XK_Georgian_fi));               
  rb_define_global_const( "WIN_Xabovedot",                      INT2FIX( XK_Xabovedot));                 
  rb_define_global_const( "WIN_Ibreve",                         INT2FIX( XK_Ibreve));                    
  rb_define_global_const( "WIN_Zstroke",                        INT2FIX( XK_Zstroke));                   
  rb_define_global_const( "WIN_Gcaron",                         INT2FIX( XK_Gcaron));                    
  rb_define_global_const( "WIN_Ocaron",                         INT2FIX( XK_Ocaron));                    
  rb_define_global_const( "WIN_Obarred",                        INT2FIX( XK_Obarred));                   
  rb_define_global_const( "WIN_xabovedot",                      INT2FIX( XK_xabovedot));                 
  rb_define_global_const( "WIN_ibreve",                         INT2FIX( XK_ibreve));                    
  rb_define_global_const( "WIN_zstroke",                        INT2FIX( XK_zstroke));                   
  rb_define_global_const( "WIN_gcaron",                         INT2FIX( XK_gcaron));                    
  rb_define_global_const( "WIN_ocaron",                         INT2FIX( XK_ocaron));                    
  rb_define_global_const( "WIN_obarred",                        INT2FIX( XK_obarred));                   
  rb_define_global_const( "WIN_SCHWA",                          INT2FIX( XK_SCHWA));                     
  rb_define_global_const( "WIN_schwa",                          INT2FIX( XK_schwa));                     
  rb_define_global_const( "WIN_Lbelowdot",                      INT2FIX( XK_Lbelowdot));                 
  rb_define_global_const( "WIN_lbelowdot",                      INT2FIX( XK_lbelowdot));                 
  rb_define_global_const( "WIN_Abelowdot",                      INT2FIX( XK_Abelowdot));                 
  rb_define_global_const( "WIN_abelowdot",                      INT2FIX( XK_abelowdot));                 
  rb_define_global_const( "WIN_Ahook",                          INT2FIX( XK_Ahook));                     
  rb_define_global_const( "WIN_ahook",                          INT2FIX( XK_ahook));                     
  rb_define_global_const( "WIN_Acircumflexacute",               INT2FIX( XK_Acircumflexacute));          
  rb_define_global_const( "WIN_acircumflexacute",               INT2FIX( XK_acircumflexacute));          
  rb_define_global_const( "WIN_Acircumflexgrave",               INT2FIX( XK_Acircumflexgrave));          
  rb_define_global_const( "WIN_acircumflexgrave",               INT2FIX( XK_acircumflexgrave));          
  rb_define_global_const( "WIN_Acircumflexhook",                INT2FIX( XK_Acircumflexhook));           
  rb_define_global_const( "WIN_acircumflexhook",                INT2FIX( XK_acircumflexhook));           
  rb_define_global_const( "WIN_Acircumflextilde",               INT2FIX( XK_Acircumflextilde));          
  rb_define_global_const( "WIN_acircumflextilde",               INT2FIX( XK_acircumflextilde));          
  rb_define_global_const( "WIN_Acircumflexbelowdot",            INT2FIX( XK_Acircumflexbelowdot));       
  rb_define_global_const( "WIN_acircumflexbelowdot",            INT2FIX( XK_acircumflexbelowdot));       
  rb_define_global_const( "WIN_Abreveacute",                    INT2FIX( XK_Abreveacute));               
  rb_define_global_const( "WIN_abreveacute",                    INT2FIX( XK_abreveacute));               
  rb_define_global_const( "WIN_Abrevegrave",                    INT2FIX( XK_Abrevegrave));               
  rb_define_global_const( "WIN_abrevegrave",                    INT2FIX( XK_abrevegrave));               
  rb_define_global_const( "WIN_Abrevehook",                     INT2FIX( XK_Abrevehook));                
  rb_define_global_const( "WIN_abrevehook",                     INT2FIX( XK_abrevehook));                
  rb_define_global_const( "WIN_Abrevetilde",                    INT2FIX( XK_Abrevetilde));               
  rb_define_global_const( "WIN_abrevetilde",                    INT2FIX( XK_abrevetilde));               
  rb_define_global_const( "WIN_Abrevebelowdot",                 INT2FIX( XK_Abrevebelowdot));            
  rb_define_global_const( "WIN_abrevebelowdot",                 INT2FIX( XK_abrevebelowdot));            
  rb_define_global_const( "WIN_Ebelowdot",                      INT2FIX( XK_Ebelowdot));                 
  rb_define_global_const( "WIN_ebelowdot",                      INT2FIX( XK_ebelowdot));                 
  rb_define_global_const( "WIN_Ehook",                          INT2FIX( XK_Ehook));                     
  rb_define_global_const( "WIN_ehook",                          INT2FIX( XK_ehook));                     
  rb_define_global_const( "WIN_Etilde",                         INT2FIX( XK_Etilde));                    
  rb_define_global_const( "WIN_etilde",                         INT2FIX( XK_etilde));                    
  rb_define_global_const( "WIN_Ecircumflexacute",               INT2FIX( XK_Ecircumflexacute));          
  rb_define_global_const( "WIN_ecircumflexacute",               INT2FIX( XK_ecircumflexacute));          
  rb_define_global_const( "WIN_Ecircumflexgrave",               INT2FIX( XK_Ecircumflexgrave));          
  rb_define_global_const( "WIN_ecircumflexgrave",               INT2FIX( XK_ecircumflexgrave));          
  rb_define_global_const( "WIN_Ecircumflexhook",                INT2FIX( XK_Ecircumflexhook));           
  rb_define_global_const( "WIN_ecircumflexhook",                INT2FIX( XK_ecircumflexhook));           
  rb_define_global_const( "WIN_Ecircumflextilde",               INT2FIX( XK_Ecircumflextilde));          
  rb_define_global_const( "WIN_ecircumflextilde",               INT2FIX( XK_ecircumflextilde));          
  rb_define_global_const( "WIN_Ecircumflexbelowdot",            INT2FIX( XK_Ecircumflexbelowdot));       
  rb_define_global_const( "WIN_ecircumflexbelowdot",            INT2FIX( XK_ecircumflexbelowdot));       
  rb_define_global_const( "WIN_Ihook",                          INT2FIX( XK_Ihook));                     
  rb_define_global_const( "WIN_ihook",                          INT2FIX( XK_ihook));                     
  rb_define_global_const( "WIN_Ibelowdot",                      INT2FIX( XK_Ibelowdot));                 
  rb_define_global_const( "WIN_ibelowdot",                      INT2FIX( XK_ibelowdot));                 
  rb_define_global_const( "WIN_Obelowdot",                      INT2FIX( XK_Obelowdot));                 
  rb_define_global_const( "WIN_obelowdot",                      INT2FIX( XK_obelowdot));                 
  rb_define_global_const( "WIN_Ohook",                          INT2FIX( XK_Ohook));                     
  rb_define_global_const( "WIN_ohook",                          INT2FIX( XK_ohook));                     
  rb_define_global_const( "WIN_Ocircumflexacute",               INT2FIX( XK_Ocircumflexacute));          
  rb_define_global_const( "WIN_ocircumflexacute",               INT2FIX( XK_ocircumflexacute));          
  rb_define_global_const( "WIN_Ocircumflexgrave",               INT2FIX( XK_Ocircumflexgrave));          
  rb_define_global_const( "WIN_ocircumflexgrave",               INT2FIX( XK_ocircumflexgrave));          
  rb_define_global_const( "WIN_Ocircumflexhook",                INT2FIX( XK_Ocircumflexhook));           
  rb_define_global_const( "WIN_ocircumflexhook",                INT2FIX( XK_ocircumflexhook));           
  rb_define_global_const( "WIN_Ocircumflextilde",               INT2FIX( XK_Ocircumflextilde));          
  rb_define_global_const( "WIN_ocircumflextilde",               INT2FIX( XK_ocircumflextilde));          
  rb_define_global_const( "WIN_Ocircumflexbelowdot",            INT2FIX( XK_Ocircumflexbelowdot));       
  rb_define_global_const( "WIN_ocircumflexbelowdot",            INT2FIX( XK_ocircumflexbelowdot));       
  rb_define_global_const( "WIN_Ohornacute",                     INT2FIX( XK_Ohornacute));                
  rb_define_global_const( "WIN_ohornacute",                     INT2FIX( XK_ohornacute));                
  rb_define_global_const( "WIN_Ohorngrave",                     INT2FIX( XK_Ohorngrave));                
  rb_define_global_const( "WIN_ohorngrave",                     INT2FIX( XK_ohorngrave));                
  rb_define_global_const( "WIN_Ohornhook",                      INT2FIX( XK_Ohornhook));                 
  rb_define_global_const( "WIN_ohornhook",                      INT2FIX( XK_ohornhook));                 
  rb_define_global_const( "WIN_Ohorntilde",                     INT2FIX( XK_Ohorntilde));                
  rb_define_global_const( "WIN_ohorntilde",                     INT2FIX( XK_ohorntilde));                
  rb_define_global_const( "WIN_Ohornbelowdot",                  INT2FIX( XK_Ohornbelowdot));             
  rb_define_global_const( "WIN_ohornbelowdot",                  INT2FIX( XK_ohornbelowdot));             
  rb_define_global_const( "WIN_Ubelowdot",                      INT2FIX( XK_Ubelowdot));                 
  rb_define_global_const( "WIN_ubelowdot",                      INT2FIX( XK_ubelowdot));                 
  rb_define_global_const( "WIN_Uhook",                          INT2FIX( XK_Uhook));                     
  rb_define_global_const( "WIN_uhook",                          INT2FIX( XK_uhook));                     
  rb_define_global_const( "WIN_Uhornacute",                     INT2FIX( XK_Uhornacute));                
  rb_define_global_const( "WIN_uhornacute",                     INT2FIX( XK_uhornacute));                
  rb_define_global_const( "WIN_Uhorngrave",                     INT2FIX( XK_Uhorngrave));                
  rb_define_global_const( "WIN_uhorngrave",                     INT2FIX( XK_uhorngrave));                
  rb_define_global_const( "WIN_Uhornhook",                      INT2FIX( XK_Uhornhook));                 
  rb_define_global_const( "WIN_uhornhook",                      INT2FIX( XK_uhornhook));                 
  rb_define_global_const( "WIN_Uhorntilde",                     INT2FIX( XK_Uhorntilde));                
  rb_define_global_const( "WIN_uhorntilde",                     INT2FIX( XK_uhorntilde));                
  rb_define_global_const( "WIN_Uhornbelowdot",                  INT2FIX( XK_Uhornbelowdot));             
  rb_define_global_const( "WIN_uhornbelowdot",                  INT2FIX( XK_uhornbelowdot));             
  rb_define_global_const( "WIN_Ybelowdot",                      INT2FIX( XK_Ybelowdot));                 
  rb_define_global_const( "WIN_ybelowdot",                      INT2FIX( XK_ybelowdot));                 
  rb_define_global_const( "WIN_Yhook",                          INT2FIX( XK_Yhook));                     
  rb_define_global_const( "WIN_yhook",                          INT2FIX( XK_yhook));                     
  rb_define_global_const( "WIN_Ytilde",                         INT2FIX( XK_Ytilde));                    
  rb_define_global_const( "WIN_ytilde",                         INT2FIX( XK_ytilde));                    
  rb_define_global_const( "WIN_Ohorn",                          INT2FIX( XK_Ohorn));                     
  rb_define_global_const( "WIN_ohorn",                          INT2FIX( XK_ohorn));                     
  rb_define_global_const( "WIN_Uhorn",                          INT2FIX( XK_Uhorn));                     
  rb_define_global_const( "WIN_uhorn",                          INT2FIX( XK_uhorn));                     
  rb_define_global_const( "WIN_EcuSign",                        INT2FIX( XK_EcuSign));                   
  rb_define_global_const( "WIN_ColonSign",                      INT2FIX( XK_ColonSign));                 
  rb_define_global_const( "WIN_CruzeiroSign",                   INT2FIX( XK_CruzeiroSign));              
  rb_define_global_const( "WIN_FFrancSign",                     INT2FIX( XK_FFrancSign));                
  rb_define_global_const( "WIN_LiraSign",                       INT2FIX( XK_LiraSign));                  
  rb_define_global_const( "WIN_MillSign",                       INT2FIX( XK_MillSign));                  
  rb_define_global_const( "WIN_NairaSign",                      INT2FIX( XK_NairaSign));                 
  rb_define_global_const( "WIN_PesetaSign",                     INT2FIX( XK_PesetaSign));                
  rb_define_global_const( "WIN_RupeeSign",                      INT2FIX( XK_RupeeSign));                 
  rb_define_global_const( "WIN_WonSign",                        INT2FIX( XK_WonSign));                   
  rb_define_global_const( "WIN_NewSheqelSign",                  INT2FIX( XK_NewSheqelSign));             
  rb_define_global_const( "WIN_DongSign",                       INT2FIX( XK_DongSign));                  
  rb_define_global_const( "WIN_EuroSign",                       INT2FIX( XK_EuroSign));                  
  rb_define_global_const( "WIN_zerosuperior",                   INT2FIX( XK_zerosuperior));              
  rb_define_global_const( "WIN_foursuperior",                   INT2FIX( XK_foursuperior));              
  rb_define_global_const( "WIN_fivesuperior",                   INT2FIX( XK_fivesuperior));              
  rb_define_global_const( "WIN_sixsuperior",                    INT2FIX( XK_sixsuperior));               
  rb_define_global_const( "WIN_sevensuperior",                  INT2FIX( XK_sevensuperior));             
  rb_define_global_const( "WIN_eightsuperior",                  INT2FIX( XK_eightsuperior));             
  rb_define_global_const( "WIN_ninesuperior",                   INT2FIX( XK_ninesuperior));              
  rb_define_global_const( "WIN_zerosubscript",                  INT2FIX( XK_zerosubscript));             
  rb_define_global_const( "WIN_onesubscript",                   INT2FIX( XK_onesubscript));              
  rb_define_global_const( "WIN_twosubscript",                   INT2FIX( XK_twosubscript));              
  rb_define_global_const( "WIN_threesubscript",                 INT2FIX( XK_threesubscript));            
  rb_define_global_const( "WIN_foursubscript",                  INT2FIX( XK_foursubscript));             
  rb_define_global_const( "WIN_fivesubscript",                  INT2FIX( XK_fivesubscript));             
  rb_define_global_const( "WIN_sixsubscript",                   INT2FIX( XK_sixsubscript));              
  rb_define_global_const( "WIN_sevensubscript",                 INT2FIX( XK_sevensubscript));            
  rb_define_global_const( "WIN_eightsubscript",                 INT2FIX( XK_eightsubscript));            
  rb_define_global_const( "WIN_ninesubscript",                  INT2FIX( XK_ninesubscript));             
  rb_define_global_const( "WIN_partdifferential",               INT2FIX( XK_partdifferential));          
  rb_define_global_const( "WIN_emptyset",                       INT2FIX( XK_emptyset));                  
  rb_define_global_const( "WIN_elementof",                      INT2FIX( XK_elementof));                 
  rb_define_global_const( "WIN_notelementof",                   INT2FIX( XK_notelementof));              
  rb_define_global_const( "WIN_containsas",                     INT2FIX( XK_containsas));                
  rb_define_global_const( "WIN_squareroot",                     INT2FIX( XK_squareroot));                
  rb_define_global_const( "WIN_cuberoot",                       INT2FIX( XK_cuberoot));                  
  rb_define_global_const( "WIN_fourthroot",                     INT2FIX( XK_fourthroot));                
  rb_define_global_const( "WIN_dintegral",                      INT2FIX( XK_dintegral));                 
  rb_define_global_const( "WIN_tintegral",                      INT2FIX( XK_tintegral));                 
  rb_define_global_const( "WIN_because",                        INT2FIX( XK_because));                   
  rb_define_global_const( "WIN_approxeq",                       INT2FIX( XK_approxeq));                  
  rb_define_global_const( "WIN_notapproxeq",                    INT2FIX( XK_notapproxeq));               
  rb_define_global_const( "WIN_notidentical",                   INT2FIX( XK_notidentical));              
  rb_define_global_const( "WIN_stricteq",                       INT2FIX( XK_stricteq));                  
  rb_define_global_const( "WIN_braille_dot_1",                  INT2FIX( XK_braille_dot_1));             
  rb_define_global_const( "WIN_braille_dot_2",                  INT2FIX( XK_braille_dot_2));             
  rb_define_global_const( "WIN_braille_dot_3",                  INT2FIX( XK_braille_dot_3));             
  rb_define_global_const( "WIN_braille_dot_4",                  INT2FIX( XK_braille_dot_4));             
  rb_define_global_const( "WIN_braille_dot_5",                  INT2FIX( XK_braille_dot_5));             
  rb_define_global_const( "WIN_braille_dot_6",                  INT2FIX( XK_braille_dot_6));             
  rb_define_global_const( "WIN_braille_dot_7",                  INT2FIX( XK_braille_dot_7));             
  rb_define_global_const( "WIN_braille_dot_8",                  INT2FIX( XK_braille_dot_8));             
  rb_define_global_const( "WIN_braille_dot_9",                  INT2FIX( XK_braille_dot_9));             
  rb_define_global_const( "WIN_braille_dot_10",                 INT2FIX( XK_braille_dot_10));            
  rb_define_global_const( "WIN_braille_blank",                  INT2FIX( XK_braille_blank));             
  rb_define_global_const( "WIN_braille_dots_1",                 INT2FIX( XK_braille_dots_1));            
  rb_define_global_const( "WIN_braille_dots_2",                 INT2FIX( XK_braille_dots_2));            
  rb_define_global_const( "WIN_braille_dots_12",                INT2FIX( XK_braille_dots_12));           
  rb_define_global_const( "WIN_braille_dots_3",                 INT2FIX( XK_braille_dots_3));            
  rb_define_global_const( "WIN_braille_dots_13",                INT2FIX( XK_braille_dots_13));           
  rb_define_global_const( "WIN_braille_dots_23",                INT2FIX( XK_braille_dots_23));           
  rb_define_global_const( "WIN_braille_dots_123",               INT2FIX( XK_braille_dots_123));          
  rb_define_global_const( "WIN_braille_dots_4",                 INT2FIX( XK_braille_dots_4));            
  rb_define_global_const( "WIN_braille_dots_14",                INT2FIX( XK_braille_dots_14));           
  rb_define_global_const( "WIN_braille_dots_24",                INT2FIX( XK_braille_dots_24));           
  rb_define_global_const( "WIN_braille_dots_124",               INT2FIX( XK_braille_dots_124));          
  rb_define_global_const( "WIN_braille_dots_34",                INT2FIX( XK_braille_dots_34));           
  rb_define_global_const( "WIN_braille_dots_134",               INT2FIX( XK_braille_dots_134));          
  rb_define_global_const( "WIN_braille_dots_234",               INT2FIX( XK_braille_dots_234));          
  rb_define_global_const( "WIN_braille_dots_1234",              INT2FIX( XK_braille_dots_1234));         
  rb_define_global_const( "WIN_braille_dots_5",                 INT2FIX( XK_braille_dots_5));            
  rb_define_global_const( "WIN_braille_dots_15",                INT2FIX( XK_braille_dots_15));           
  rb_define_global_const( "WIN_braille_dots_25",                INT2FIX( XK_braille_dots_25));           
  rb_define_global_const( "WIN_braille_dots_125",               INT2FIX( XK_braille_dots_125));          
  rb_define_global_const( "WIN_braille_dots_35",                INT2FIX( XK_braille_dots_35));           
  rb_define_global_const( "WIN_braille_dots_135",               INT2FIX( XK_braille_dots_135));          
  rb_define_global_const( "WIN_braille_dots_235",               INT2FIX( XK_braille_dots_235));          
  rb_define_global_const( "WIN_braille_dots_1235",              INT2FIX( XK_braille_dots_1235));         
  rb_define_global_const( "WIN_braille_dots_45",                INT2FIX( XK_braille_dots_45));           
  rb_define_global_const( "WIN_braille_dots_145",               INT2FIX( XK_braille_dots_145));          
  rb_define_global_const( "WIN_braille_dots_245",               INT2FIX( XK_braille_dots_245));          
  rb_define_global_const( "WIN_braille_dots_1245",              INT2FIX( XK_braille_dots_1245));         
  rb_define_global_const( "WIN_braille_dots_345",               INT2FIX( XK_braille_dots_345));          
  rb_define_global_const( "WIN_braille_dots_1345",              INT2FIX( XK_braille_dots_1345));         
  rb_define_global_const( "WIN_braille_dots_2345",              INT2FIX( XK_braille_dots_2345));         
  rb_define_global_const( "WIN_braille_dots_12345",             INT2FIX( XK_braille_dots_12345));        
  rb_define_global_const( "WIN_braille_dots_6",                 INT2FIX( XK_braille_dots_6));            
  rb_define_global_const( "WIN_braille_dots_16",                INT2FIX( XK_braille_dots_16));           
  rb_define_global_const( "WIN_braille_dots_26",                INT2FIX( XK_braille_dots_26));           
  rb_define_global_const( "WIN_braille_dots_126",               INT2FIX( XK_braille_dots_126));          
  rb_define_global_const( "WIN_braille_dots_36",                INT2FIX( XK_braille_dots_36));           
  rb_define_global_const( "WIN_braille_dots_136",               INT2FIX( XK_braille_dots_136));          
  rb_define_global_const( "WIN_braille_dots_236",               INT2FIX( XK_braille_dots_236));          
  rb_define_global_const( "WIN_braille_dots_1236",              INT2FIX( XK_braille_dots_1236));         
  rb_define_global_const( "WIN_braille_dots_46",                INT2FIX( XK_braille_dots_46));           
  rb_define_global_const( "WIN_braille_dots_146",               INT2FIX( XK_braille_dots_146));          
  rb_define_global_const( "WIN_braille_dots_246",               INT2FIX( XK_braille_dots_246));          
  rb_define_global_const( "WIN_braille_dots_1246",              INT2FIX( XK_braille_dots_1246));         
  rb_define_global_const( "WIN_braille_dots_346",               INT2FIX( XK_braille_dots_346));          
  rb_define_global_const( "WIN_braille_dots_1346",              INT2FIX( XK_braille_dots_1346));         
  rb_define_global_const( "WIN_braille_dots_2346",              INT2FIX( XK_braille_dots_2346));         
  rb_define_global_const( "WIN_braille_dots_12346",             INT2FIX( XK_braille_dots_12346));        
  rb_define_global_const( "WIN_braille_dots_56",                INT2FIX( XK_braille_dots_56));           
  rb_define_global_const( "WIN_braille_dots_156",               INT2FIX( XK_braille_dots_156));          
  rb_define_global_const( "WIN_braille_dots_256",               INT2FIX( XK_braille_dots_256));          
  rb_define_global_const( "WIN_braille_dots_1256",              INT2FIX( XK_braille_dots_1256));         
  rb_define_global_const( "WIN_braille_dots_356",               INT2FIX( XK_braille_dots_356));          
  rb_define_global_const( "WIN_braille_dots_1356",              INT2FIX( XK_braille_dots_1356));         
  rb_define_global_const( "WIN_braille_dots_2356",              INT2FIX( XK_braille_dots_2356));         
  rb_define_global_const( "WIN_braille_dots_12356",             INT2FIX( XK_braille_dots_12356));        
  rb_define_global_const( "WIN_braille_dots_456",               INT2FIX( XK_braille_dots_456));          
  rb_define_global_const( "WIN_braille_dots_1456",              INT2FIX( XK_braille_dots_1456));         
  rb_define_global_const( "WIN_braille_dots_2456",              INT2FIX( XK_braille_dots_2456));         
  rb_define_global_const( "WIN_braille_dots_12456",             INT2FIX( XK_braille_dots_12456));        
  rb_define_global_const( "WIN_braille_dots_3456",              INT2FIX( XK_braille_dots_3456));         
  rb_define_global_const( "WIN_braille_dots_13456",             INT2FIX( XK_braille_dots_13456));        
  rb_define_global_const( "WIN_braille_dots_23456",             INT2FIX( XK_braille_dots_23456));        
  rb_define_global_const( "WIN_braille_dots_123456",            INT2FIX( XK_braille_dots_123456));       
  rb_define_global_const( "WIN_braille_dots_7",                 INT2FIX( XK_braille_dots_7));            
  rb_define_global_const( "WIN_braille_dots_17",                INT2FIX( XK_braille_dots_17));           
  rb_define_global_const( "WIN_braille_dots_27",                INT2FIX( XK_braille_dots_27));           
  rb_define_global_const( "WIN_braille_dots_127",               INT2FIX( XK_braille_dots_127));          
  rb_define_global_const( "WIN_braille_dots_37",                INT2FIX( XK_braille_dots_37));           
  rb_define_global_const( "WIN_braille_dots_137",               INT2FIX( XK_braille_dots_137));          
  rb_define_global_const( "WIN_braille_dots_237",               INT2FIX( XK_braille_dots_237));          
  rb_define_global_const( "WIN_braille_dots_1237",              INT2FIX( XK_braille_dots_1237));         
  rb_define_global_const( "WIN_braille_dots_47",                INT2FIX( XK_braille_dots_47));           
  rb_define_global_const( "WIN_braille_dots_147",               INT2FIX( XK_braille_dots_147));          
  rb_define_global_const( "WIN_braille_dots_247",               INT2FIX( XK_braille_dots_247));          
  rb_define_global_const( "WIN_braille_dots_1247",              INT2FIX( XK_braille_dots_1247));         
  rb_define_global_const( "WIN_braille_dots_347",               INT2FIX( XK_braille_dots_347));          
  rb_define_global_const( "WIN_braille_dots_1347",              INT2FIX( XK_braille_dots_1347));         
  rb_define_global_const( "WIN_braille_dots_2347",              INT2FIX( XK_braille_dots_2347));         
  rb_define_global_const( "WIN_braille_dots_12347",             INT2FIX( XK_braille_dots_12347));        
  rb_define_global_const( "WIN_braille_dots_57",                INT2FIX( XK_braille_dots_57));           
  rb_define_global_const( "WIN_braille_dots_157",               INT2FIX( XK_braille_dots_157));          
  rb_define_global_const( "WIN_braille_dots_257",               INT2FIX( XK_braille_dots_257));          
  rb_define_global_const( "WIN_braille_dots_1257",              INT2FIX( XK_braille_dots_1257));         
  rb_define_global_const( "WIN_braille_dots_357",               INT2FIX( XK_braille_dots_357));          
  rb_define_global_const( "WIN_braille_dots_1357",              INT2FIX( XK_braille_dots_1357));         
  rb_define_global_const( "WIN_braille_dots_2357",              INT2FIX( XK_braille_dots_2357));         
  rb_define_global_const( "WIN_braille_dots_12357",             INT2FIX( XK_braille_dots_12357));        
  rb_define_global_const( "WIN_braille_dots_457",               INT2FIX( XK_braille_dots_457));          
  rb_define_global_const( "WIN_braille_dots_1457",              INT2FIX( XK_braille_dots_1457));         
  rb_define_global_const( "WIN_braille_dots_2457",              INT2FIX( XK_braille_dots_2457));         
  rb_define_global_const( "WIN_braille_dots_12457",             INT2FIX( XK_braille_dots_12457));        
  rb_define_global_const( "WIN_braille_dots_3457",              INT2FIX( XK_braille_dots_3457));         
  rb_define_global_const( "WIN_braille_dots_13457",             INT2FIX( XK_braille_dots_13457));        
  rb_define_global_const( "WIN_braille_dots_23457",             INT2FIX( XK_braille_dots_23457));        
  rb_define_global_const( "WIN_braille_dots_123457",            INT2FIX( XK_braille_dots_123457));       
  rb_define_global_const( "WIN_braille_dots_67",                INT2FIX( XK_braille_dots_67));           
  rb_define_global_const( "WIN_braille_dots_167",               INT2FIX( XK_braille_dots_167));          
  rb_define_global_const( "WIN_braille_dots_267",               INT2FIX( XK_braille_dots_267));          
  rb_define_global_const( "WIN_braille_dots_1267",              INT2FIX( XK_braille_dots_1267));         
  rb_define_global_const( "WIN_braille_dots_367",               INT2FIX( XK_braille_dots_367));          
  rb_define_global_const( "WIN_braille_dots_1367",              INT2FIX( XK_braille_dots_1367));         
  rb_define_global_const( "WIN_braille_dots_2367",              INT2FIX( XK_braille_dots_2367));         
  rb_define_global_const( "WIN_braille_dots_12367",             INT2FIX( XK_braille_dots_12367));        
  rb_define_global_const( "WIN_braille_dots_467",               INT2FIX( XK_braille_dots_467));          
  rb_define_global_const( "WIN_braille_dots_1467",              INT2FIX( XK_braille_dots_1467));         
  rb_define_global_const( "WIN_braille_dots_2467",              INT2FIX( XK_braille_dots_2467));         
  rb_define_global_const( "WIN_braille_dots_12467",             INT2FIX( XK_braille_dots_12467));        
  rb_define_global_const( "WIN_braille_dots_3467",              INT2FIX( XK_braille_dots_3467));         
  rb_define_global_const( "WIN_braille_dots_13467",             INT2FIX( XK_braille_dots_13467));        
  rb_define_global_const( "WIN_braille_dots_23467",             INT2FIX( XK_braille_dots_23467));        
  rb_define_global_const( "WIN_braille_dots_123467",            INT2FIX( XK_braille_dots_123467));       
  rb_define_global_const( "WIN_braille_dots_567",               INT2FIX( XK_braille_dots_567));          
  rb_define_global_const( "WIN_braille_dots_1567",              INT2FIX( XK_braille_dots_1567));         
  rb_define_global_const( "WIN_braille_dots_2567",              INT2FIX( XK_braille_dots_2567));         
  rb_define_global_const( "WIN_braille_dots_12567",             INT2FIX( XK_braille_dots_12567));        
  rb_define_global_const( "WIN_braille_dots_3567",              INT2FIX( XK_braille_dots_3567));         
  rb_define_global_const( "WIN_braille_dots_13567",             INT2FIX( XK_braille_dots_13567));        
  rb_define_global_const( "WIN_braille_dots_23567",             INT2FIX( XK_braille_dots_23567));        
  rb_define_global_const( "WIN_braille_dots_123567",            INT2FIX( XK_braille_dots_123567));       
  rb_define_global_const( "WIN_braille_dots_4567",              INT2FIX( XK_braille_dots_4567));         
  rb_define_global_const( "WIN_braille_dots_14567",             INT2FIX( XK_braille_dots_14567));        
  rb_define_global_const( "WIN_braille_dots_24567",             INT2FIX( XK_braille_dots_24567));        
  rb_define_global_const( "WIN_braille_dots_124567",            INT2FIX( XK_braille_dots_124567));       
  rb_define_global_const( "WIN_braille_dots_34567",             INT2FIX( XK_braille_dots_34567));        
  rb_define_global_const( "WIN_braille_dots_134567",            INT2FIX( XK_braille_dots_134567));       
  rb_define_global_const( "WIN_braille_dots_234567",            INT2FIX( XK_braille_dots_234567));       
  rb_define_global_const( "WIN_braille_dots_1234567",           INT2FIX( XK_braille_dots_1234567));      
  rb_define_global_const( "WIN_braille_dots_8",                 INT2FIX( XK_braille_dots_8));            
  rb_define_global_const( "WIN_braille_dots_18",                INT2FIX( XK_braille_dots_18));           
  rb_define_global_const( "WIN_braille_dots_28",                INT2FIX( XK_braille_dots_28));           
  rb_define_global_const( "WIN_braille_dots_128",               INT2FIX( XK_braille_dots_128));          
  rb_define_global_const( "WIN_braille_dots_38",                INT2FIX( XK_braille_dots_38));           
  rb_define_global_const( "WIN_braille_dots_138",               INT2FIX( XK_braille_dots_138));          
  rb_define_global_const( "WIN_braille_dots_238",               INT2FIX( XK_braille_dots_238));          
  rb_define_global_const( "WIN_braille_dots_1238",              INT2FIX( XK_braille_dots_1238));         
  rb_define_global_const( "WIN_braille_dots_48",                INT2FIX( XK_braille_dots_48));           
  rb_define_global_const( "WIN_braille_dots_148",               INT2FIX( XK_braille_dots_148));          
  rb_define_global_const( "WIN_braille_dots_248",               INT2FIX( XK_braille_dots_248));          
  rb_define_global_const( "WIN_braille_dots_1248",              INT2FIX( XK_braille_dots_1248));         
  rb_define_global_const( "WIN_braille_dots_348",               INT2FIX( XK_braille_dots_348));          
  rb_define_global_const( "WIN_braille_dots_1348",              INT2FIX( XK_braille_dots_1348));         
  rb_define_global_const( "WIN_braille_dots_2348",              INT2FIX( XK_braille_dots_2348));         
  rb_define_global_const( "WIN_braille_dots_12348",             INT2FIX( XK_braille_dots_12348));        
  rb_define_global_const( "WIN_braille_dots_58",                INT2FIX( XK_braille_dots_58));           
  rb_define_global_const( "WIN_braille_dots_158",               INT2FIX( XK_braille_dots_158));          
  rb_define_global_const( "WIN_braille_dots_258",               INT2FIX( XK_braille_dots_258));          
  rb_define_global_const( "WIN_braille_dots_1258",              INT2FIX( XK_braille_dots_1258));         
  rb_define_global_const( "WIN_braille_dots_358",               INT2FIX( XK_braille_dots_358));          
  rb_define_global_const( "WIN_braille_dots_1358",              INT2FIX( XK_braille_dots_1358));         
  rb_define_global_const( "WIN_braille_dots_2358",              INT2FIX( XK_braille_dots_2358));         
  rb_define_global_const( "WIN_braille_dots_12358",             INT2FIX( XK_braille_dots_12358));        
  rb_define_global_const( "WIN_braille_dots_458",               INT2FIX( XK_braille_dots_458));          
  rb_define_global_const( "WIN_braille_dots_1458",              INT2FIX( XK_braille_dots_1458));         
  rb_define_global_const( "WIN_braille_dots_2458",              INT2FIX( XK_braille_dots_2458));         
  rb_define_global_const( "WIN_braille_dots_12458",             INT2FIX( XK_braille_dots_12458));        
  rb_define_global_const( "WIN_braille_dots_3458",              INT2FIX( XK_braille_dots_3458));         
  rb_define_global_const( "WIN_braille_dots_13458",             INT2FIX( XK_braille_dots_13458));        
  rb_define_global_const( "WIN_braille_dots_23458",             INT2FIX( XK_braille_dots_23458));        
  rb_define_global_const( "WIN_braille_dots_123458",            INT2FIX( XK_braille_dots_123458));       
  rb_define_global_const( "WIN_braille_dots_68",                INT2FIX( XK_braille_dots_68));           
  rb_define_global_const( "WIN_braille_dots_168",               INT2FIX( XK_braille_dots_168));          
  rb_define_global_const( "WIN_braille_dots_268",               INT2FIX( XK_braille_dots_268));          
  rb_define_global_const( "WIN_braille_dots_1268",              INT2FIX( XK_braille_dots_1268));         
  rb_define_global_const( "WIN_braille_dots_368",               INT2FIX( XK_braille_dots_368));          
  rb_define_global_const( "WIN_braille_dots_1368",              INT2FIX( XK_braille_dots_1368));         
  rb_define_global_const( "WIN_braille_dots_2368",              INT2FIX( XK_braille_dots_2368));         
  rb_define_global_const( "WIN_braille_dots_12368",             INT2FIX( XK_braille_dots_12368));        
  rb_define_global_const( "WIN_braille_dots_468",               INT2FIX( XK_braille_dots_468));          
  rb_define_global_const( "WIN_braille_dots_1468",              INT2FIX( XK_braille_dots_1468));         
  rb_define_global_const( "WIN_braille_dots_2468",              INT2FIX( XK_braille_dots_2468));         
  rb_define_global_const( "WIN_braille_dots_12468",             INT2FIX( XK_braille_dots_12468));        
  rb_define_global_const( "WIN_braille_dots_3468",              INT2FIX( XK_braille_dots_3468));         
  rb_define_global_const( "WIN_braille_dots_13468",             INT2FIX( XK_braille_dots_13468));        
  rb_define_global_const( "WIN_braille_dots_23468",             INT2FIX( XK_braille_dots_23468));        
  rb_define_global_const( "WIN_braille_dots_123468",            INT2FIX( XK_braille_dots_123468));       
  rb_define_global_const( "WIN_braille_dots_568",               INT2FIX( XK_braille_dots_568));          
  rb_define_global_const( "WIN_braille_dots_1568",              INT2FIX( XK_braille_dots_1568));         
  rb_define_global_const( "WIN_braille_dots_2568",              INT2FIX( XK_braille_dots_2568));         
  rb_define_global_const( "WIN_braille_dots_12568",             INT2FIX( XK_braille_dots_12568));        
  rb_define_global_const( "WIN_braille_dots_3568",              INT2FIX( XK_braille_dots_3568));         
  rb_define_global_const( "WIN_braille_dots_13568",             INT2FIX( XK_braille_dots_13568));        
  rb_define_global_const( "WIN_braille_dots_23568",             INT2FIX( XK_braille_dots_23568));        
  rb_define_global_const( "WIN_braille_dots_123568",            INT2FIX( XK_braille_dots_123568));       
  rb_define_global_const( "WIN_braille_dots_4568",              INT2FIX( XK_braille_dots_4568));         
  rb_define_global_const( "WIN_braille_dots_14568",             INT2FIX( XK_braille_dots_14568));        
  rb_define_global_const( "WIN_braille_dots_24568",             INT2FIX( XK_braille_dots_24568));        
  rb_define_global_const( "WIN_braille_dots_124568",            INT2FIX( XK_braille_dots_124568));       
  rb_define_global_const( "WIN_braille_dots_34568",             INT2FIX( XK_braille_dots_34568));        
  rb_define_global_const( "WIN_braille_dots_134568",            INT2FIX( XK_braille_dots_134568));       
  rb_define_global_const( "WIN_braille_dots_234568",            INT2FIX( XK_braille_dots_234568));       
  rb_define_global_const( "WIN_braille_dots_1234568",           INT2FIX( XK_braille_dots_1234568));      
  rb_define_global_const( "WIN_braille_dots_78",                INT2FIX( XK_braille_dots_78));           
  rb_define_global_const( "WIN_braille_dots_178",               INT2FIX( XK_braille_dots_178));          
  rb_define_global_const( "WIN_braille_dots_278",               INT2FIX( XK_braille_dots_278));          
  rb_define_global_const( "WIN_braille_dots_1278",              INT2FIX( XK_braille_dots_1278));         
  rb_define_global_const( "WIN_braille_dots_378",               INT2FIX( XK_braille_dots_378));          
  rb_define_global_const( "WIN_braille_dots_1378",              INT2FIX( XK_braille_dots_1378));         
  rb_define_global_const( "WIN_braille_dots_2378",              INT2FIX( XK_braille_dots_2378));         
  rb_define_global_const( "WIN_braille_dots_12378",             INT2FIX( XK_braille_dots_12378));        
  rb_define_global_const( "WIN_braille_dots_478",               INT2FIX( XK_braille_dots_478));          
  rb_define_global_const( "WIN_braille_dots_1478",              INT2FIX( XK_braille_dots_1478));         
  rb_define_global_const( "WIN_braille_dots_2478",              INT2FIX( XK_braille_dots_2478));         
  rb_define_global_const( "WIN_braille_dots_12478",             INT2FIX( XK_braille_dots_12478));        
  rb_define_global_const( "WIN_braille_dots_3478",              INT2FIX( XK_braille_dots_3478));         
  rb_define_global_const( "WIN_braille_dots_13478",             INT2FIX( XK_braille_dots_13478));        
  rb_define_global_const( "WIN_braille_dots_23478",             INT2FIX( XK_braille_dots_23478));        
  rb_define_global_const( "WIN_braille_dots_123478",            INT2FIX( XK_braille_dots_123478));       
  rb_define_global_const( "WIN_braille_dots_578",               INT2FIX( XK_braille_dots_578));          
  rb_define_global_const( "WIN_braille_dots_1578",              INT2FIX( XK_braille_dots_1578));         
  rb_define_global_const( "WIN_braille_dots_2578",              INT2FIX( XK_braille_dots_2578));         
  rb_define_global_const( "WIN_braille_dots_12578",             INT2FIX( XK_braille_dots_12578));        
  rb_define_global_const( "WIN_braille_dots_3578",              INT2FIX( XK_braille_dots_3578));         
  rb_define_global_const( "WIN_braille_dots_13578",             INT2FIX( XK_braille_dots_13578));        
  rb_define_global_const( "WIN_braille_dots_23578",             INT2FIX( XK_braille_dots_23578));        
  rb_define_global_const( "WIN_braille_dots_123578",            INT2FIX( XK_braille_dots_123578));       
  rb_define_global_const( "WIN_braille_dots_4578",              INT2FIX( XK_braille_dots_4578));         
  rb_define_global_const( "WIN_braille_dots_14578",             INT2FIX( XK_braille_dots_14578));        
  rb_define_global_const( "WIN_braille_dots_24578",             INT2FIX( XK_braille_dots_24578));        
  rb_define_global_const( "WIN_braille_dots_124578",            INT2FIX( XK_braille_dots_124578));       
  rb_define_global_const( "WIN_braille_dots_34578",             INT2FIX( XK_braille_dots_34578));        
  rb_define_global_const( "WIN_braille_dots_134578",            INT2FIX( XK_braille_dots_134578));       
  rb_define_global_const( "WIN_braille_dots_234578",            INT2FIX( XK_braille_dots_234578));       
  rb_define_global_const( "WIN_braille_dots_1234578",           INT2FIX( XK_braille_dots_1234578));      
  rb_define_global_const( "WIN_braille_dots_678",               INT2FIX( XK_braille_dots_678));          
  rb_define_global_const( "WIN_braille_dots_1678",              INT2FIX( XK_braille_dots_1678));         
  rb_define_global_const( "WIN_braille_dots_2678",              INT2FIX( XK_braille_dots_2678));         
  rb_define_global_const( "WIN_braille_dots_12678",             INT2FIX( XK_braille_dots_12678));        
  rb_define_global_const( "WIN_braille_dots_3678",              INT2FIX( XK_braille_dots_3678));         
  rb_define_global_const( "WIN_braille_dots_13678",             INT2FIX( XK_braille_dots_13678));        
  rb_define_global_const( "WIN_braille_dots_23678",             INT2FIX( XK_braille_dots_23678));        
  rb_define_global_const( "WIN_braille_dots_123678",            INT2FIX( XK_braille_dots_123678));       
  rb_define_global_const( "WIN_braille_dots_4678",              INT2FIX( XK_braille_dots_4678));         
  rb_define_global_const( "WIN_braille_dots_14678",             INT2FIX( XK_braille_dots_14678));        
  rb_define_global_const( "WIN_braille_dots_24678",             INT2FIX( XK_braille_dots_24678));        
  rb_define_global_const( "WIN_braille_dots_124678",            INT2FIX( XK_braille_dots_124678));       
  rb_define_global_const( "WIN_braille_dots_34678",             INT2FIX( XK_braille_dots_34678));        
  rb_define_global_const( "WIN_braille_dots_134678",            INT2FIX( XK_braille_dots_134678));       
  rb_define_global_const( "WIN_braille_dots_234678",            INT2FIX( XK_braille_dots_234678));       
  rb_define_global_const( "WIN_braille_dots_1234678",           INT2FIX( XK_braille_dots_1234678));      
  rb_define_global_const( "WIN_braille_dots_5678",              INT2FIX( XK_braille_dots_5678));         
  rb_define_global_const( "WIN_braille_dots_15678",             INT2FIX( XK_braille_dots_15678));        
  rb_define_global_const( "WIN_braille_dots_25678",             INT2FIX( XK_braille_dots_25678));        
  rb_define_global_const( "WIN_braille_dots_125678",            INT2FIX( XK_braille_dots_125678));       
  rb_define_global_const( "WIN_braille_dots_35678",             INT2FIX( XK_braille_dots_35678));        
  rb_define_global_const( "WIN_braille_dots_135678",            INT2FIX( XK_braille_dots_135678));       
  rb_define_global_const( "WIN_braille_dots_235678",            INT2FIX( XK_braille_dots_235678));       
  rb_define_global_const( "WIN_braille_dots_1235678",           INT2FIX( XK_braille_dots_1235678));      
  rb_define_global_const( "WIN_braille_dots_45678",             INT2FIX( XK_braille_dots_45678));        
  rb_define_global_const( "WIN_braille_dots_145678",            INT2FIX( XK_braille_dots_145678));       
  rb_define_global_const( "WIN_braille_dots_245678",            INT2FIX( XK_braille_dots_245678));       
  rb_define_global_const( "WIN_braille_dots_1245678",           INT2FIX( XK_braille_dots_1245678));      
  rb_define_global_const( "WIN_braille_dots_345678",            INT2FIX( XK_braille_dots_345678));       
  rb_define_global_const( "WIN_braille_dots_1345678",           INT2FIX( XK_braille_dots_1345678));      
  rb_define_global_const( "WIN_braille_dots_2345678",           INT2FIX( XK_braille_dots_2345678));      
  rb_define_global_const( "WIN_braille_dots_12345678",          INT2FIX( XK_braille_dots_12345678));     
}
